# include "Optimise.h"
# include "yyOptimise.w"
# include "System.h"
# include <stdio.h>
# include "Tree.h"

# define yyInline
# ifndef NULL
# define NULL 0L
# endif
# ifndef false
# define false 0
# endif
# ifndef true
# define true 1
# endif

# ifdef yyInline
# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
  free += nodesize [kind]; \
  ptr->yyHead.yyMark = 0; \
  ptr->Kind = kind;
# else
# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
# endif

# define yyWrite(s) (void) fputs (s, yyf)
# define yyWriteNl (void) fputc ('\n', yyf)

/* line 35 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */

  /* These flags can be used to enable/disable particular optimisations. */
  bool UnrollSeqRepFlag  = true;
  bool UnrollAltRepFlag  = true;
  bool UnrollParRepFlag  = true;
  bool UnrollIfRepFlag   = true;
  bool SimplifyIfFlag    = true;
  bool SimplifyCaseFlag  = true;
  bool InlineProcFlag    = true;
  bool InlineFuncFlag    = true;
  bool UniqueNamesFlag   = true;
  bool InlineArraysFlag  = true;
  bool RemoveUnusedFlag  = false; /* does not work in all cases, so best left disabled */

  static bool UnrollAgain;

  static last_type_was_chan;

  struct symbol_element
    {
    tIdent  Id;
    tSymbol Scope;
    tTree   Type;
    tTree   Kind;
    tVUsage Usage;
    };

  static struct symbol_element* symbol_array;

  /* This makes a copy of the current symbol table, so that it may be queried later on. */
  static void OPT_CopyOldSymbolTable(void)
    {
    int n;

    /* allocate memory for the symbols */
    symbol_array = (struct symbol_element*)malloc((SymSize+1)*sizeof(struct symbol_element));

    /* copy the symbols into the new array */
    for(n=0; n<=SymSize; n++)
      {
      symbol_array[n].Id    = SymTable[n].Id;
      symbol_array[n].Scope = SymTable[n].Scope;
      symbol_array[n].Type  = SymTable[n].Type;
      symbol_array[n].Kind  = SymTable[n].Kind;
      symbol_array[n].Usage = SymTable[n].Usage;
      }

    /* reinitialise the symbol module */
    EndSymbols();
    InitSymbols();
    }

  /* This deletes the old symbol table. */
  static void OPT_DeleteOldSymbolTable(void)
    {
    free(symbol_array);
    }

  /* This structure stores the info about mapped symbols. */
  /* A symbol will be mapped when it has been renamed. */
  struct OPT_ID_LIST
    {
    int                 depth;
    tSymbol             symbol;
    tIdent              newId;
    struct OPT_ID_LIST* next;
    };

  /* This is the start of the list used to hold mapped symbols. */
  static struct OPT_ID_LIST* ModifiedIdList=NULL;

  /* Generates a unqiue symbol, based on the supplied symbol. */
  /* The identity of the new symbol will be returned. */
  static tIdent OPT_MakeUniqueIdent (tSymbol Sym)
    {
    char* num_pos=NULL;
    int num=-1;
    tStringRef str=GetStringRef(symbol_array[Sym].Id);
    char* string=(char*)malloc(LengthSt(str)+1);
    char* s1=string;
    struct OPT_ID_LIST* NewEl;
    tIdent NewId;

    if (string==NULL) Message("optimise.trafo: malloc() failed in OPT_MakeUniqueIdent()",xxFatal,NoPosition);
    StGetString(str,string);

    /* iterate through the string, and look for '.x' */
    while(*s1!=0)
      {
      if ((*s1=='.') && (*(s1+1)=='x')) num_pos=s1+2;
      s1++;
      }

    /* if a '.x' was found in the string, then retrieve the number after the 'x' */
    if (num_pos!=NULL)
      {
      *(num_pos-2)=0;
      sscanf(num_pos,"%d",&num);  /* we assume that num is not changed if scanf fails */
      }

    /* allocate memory for the new symbol */
    s1=(char*)malloc(strlen(string)+2+16+1);  /* 2='.x', 16=num, 1=terminator */
    if (s1==NULL) Message("optimise.trafo: malloc() failed in OPT_MakeUniqueIdent()",xxFatal,NoPosition);

    /* generate the new Ident, by joining the old string, '.x', and the number */
    /* if the new Ident already exists, then increment num and try again */
    do{
      num++;
      sprintf(s1,"%s.x%d",string,num);
      NewId=MakeIdent(s1,strlen(s1));
      }while(LookupSymbolWithNoFail(NewId)!=NoSymbol);

    free(string);
    free(s1);

    /* store the mapping from the old symbol to the new symbol, so we can look them up again later */
    NewEl=(struct OPT_ID_LIST*)malloc(sizeof(struct OPT_ID_LIST));
    if (NewEl==NULL) Message("optimise.trafo: malloc() failed in OPT_MakeUniqueIdent()",xxFatal,NoPosition);
    NewEl->depth=Depth;
    NewEl->symbol=Sym;
    NewEl->newId=NewId;
    NewEl->next=ModifiedIdList;
    ModifiedIdList=NewEl;
    return NewId;
    }

  /* Returns a string comprising the name of the array with the number appended onto the end. */
  /* This is used during array inlining. */
  static char* OPT_GetIdealArrayName(tSymbol Sym,int num)
    {
    tStringRef str=GetStringRef(symbol_array[Sym].Id);

    /* allocate memory for the string */
    /* an additional 16 bytes are reserved for the dot and the number */
    char* string=(char*)malloc(LengthSt(str)+16+1);
    if (string==NULL) Message("optimise.trafo: malloc() failed in OPT_GetIdealArrayName()",xxFatal,NoPosition);

    /* retrieve the original string, and append the number to it */
    StGetString(str,string);
    sprintf(string+strlen(string),".%d",num);

    return string;
    }

  struct OPT_ARRAY_MAPPING
    {
    tSymbol OldSymbol;
    tSymbol NewSymbol;
    int num;
    int depth;
    struct OPT_ARRAY_MAPPING* next;
    };

  static struct OPT_ARRAY_MAPPING* OPT_array_mapping_list = NULL;

  /* Adds a mapping from an old array symbol, to the new symbol for a particular array element. */
  /* It is used during array inlining, so that references to an array element can be converted correctly. */
  static void OPT_AddArrayMapping(tSymbol OldSym,tSymbol NewSym,int num)
    {
    struct OPT_ARRAY_MAPPING* m = (struct OPT_ARRAY_MAPPING*)malloc(sizeof(struct OPT_ARRAY_MAPPING));
    if (m==NULL) Message("optimise.trafo: malloc() failed in OPT_AddArrayMapping()",xxFatal,NoPosition);
    m->OldSymbol = OldSym;
    m->NewSymbol = NewSym;
    m->num       = num;
    m->depth     = Depth;
    m->next      = OPT_array_mapping_list;
    OPT_array_mapping_list = m;
    }

  /* This is used during array inlining, to lookup the new symbol for a particular array element. */
  static tSymbol OPT_LookupArrayMapping(tSymbol OldSym,int num)
    {
    struct OPT_ARRAY_MAPPING* m = OPT_array_mapping_list;

    while(m!=NULL)
      {
      if ((m->OldSymbol==OldSym) && (m->num==num)) return m->NewSymbol;
      m=m->next;
      }

    return NoSymbol;
    }

  /* Adds a symbol to the new symbol table, based on an old symbol. */
  /* The Id of the new symbol will be taken from the old symbol table. */
  static tSymbol OPT_AddSymbol(tSymbol Sym)
    {
    /* obtain the Id from the old symbol table */
    tIdent Id = symbol_array[Sym].Id;

    if (UniqueNamesFlag==false) return AddSymbol(Id);
    if (LookupSymbolWithNoFail(Id)==NoSymbol) return AddSymbol(Id);
    return AddSymbol(OPT_MakeUniqueIdent(Sym));
    }

  /* Adds a channel to the new symbol table, based on an old channel. */
  /* The Id of the channel will be taken from the old symbol table. */
  tSymbol OPT_AddChanSymbol(tSymbol Sym)
    {
    /* obtain the Id from the old symbol table */
    tIdent Id = symbol_array[Sym].Id;

    if (UniqueNamesFlag==false) return AddChanSymbol(Id);
    if (LookupSymbolWithNoFail(Id)==NoSymbol) return AddChanSymbol(Id);
    return AddChanSymbol(OPT_MakeUniqueIdent(Sym));
    }

  /* Looks up the Id of an old symbol in the old symbol table. */
  /* We also check to see whether this symbol has been remapped - if so we return the new Id. */
  static tIdent OPT_GetIdFromOldSymbol(tSymbol Sym)
    {
    /* see whether this symbol has been remapped */
    struct OPT_ID_LIST* l=ModifiedIdList;
    while(l!=NULL)
      {
      if (l->symbol==Sym) return l->newId;
      l=l->next;
      }

    /* return the Id from the old symbol table */
    return symbol_array[Sym].Id;
    }

  /* Looks up an old symbol, and returns TRUE if that symbol is ever referenced by another part of the abstract syntax tree. */
  /* If the symbol is not used, false will be returned. */
  static bool OPT_IsOldSymbolUsed(tSymbol Sym)
    {
    return (symbol_array[Sym].Usage & (xxLRead | xxLWrite | xxNLRead | xxNLWrite)) ? true : false;
    }

  /* Looks up an old symbol, and returns the new symbol to be used instead. */
  /* The code is based on LookupSymbolWithoutFail(), with the difference being that we */
  /* do NOT abort the search when Id=NoIdent. This is required so that we can lookup   */
  /* the symbol from DefProcs, inserted around the children of PARs by transform.attr. */
  static tSymbol OPT_GetNewSymbolFromOldSymbol(tSymbol Sym)
    {
    tIdent Id = OPT_GetIdFromOldSymbol(Sym);
    tSymbol s = CurrentScope();

    while(s!=0)
      {
      if (Symbol(s).Id==Id) return s;
      else s=Symbol(s).Scope;
      }

    return NoSymbol;
    }

  /* Removes all mapped Symbols from the current depth. */
  /* These Symbols will never be referenced again once they have gone out of scope. */
  static void OPT_CloseScope(void)
    {
    struct OPT_ID_LIST** il = &ModifiedIdList;
    struct OPT_ARRAY_MAPPING** al = &OPT_array_mapping_list;

    while((*il)!=NULL)
      {
      if ((*il)->depth!=Depth) il=&((*il)->next);
      else
        {
        struct OPT_ID_LIST* in = (*il)->next;
        free(*il);
        *il=in;
        }
      };

    while((*al)!=NULL)
      {
      if ((*al)->depth!=Depth) al=&((*al)->next);
      else
        {
        struct OPT_ARRAY_MAPPING* an = (*al)->next;
        free(*al);
        *al=an;
        }
      }

    CloseScope();
    }
  

static void yyExit () { Exit (1); }

void (* Optimise_Exit) () = yyExit;

extern FILE *yyf;

static void yyAbort
# ifdef __cplusplus
 (char * yyFunction)
# else
 (yyFunction) char * yyFunction;
# endif
{
 (void) fprintf (stderr, "Error: module Optimise, routine %s failed\n", yyFunction);
 Optimise_Exit ();
}

tTree Optimise ARGS((tTree yyP1));
static tTree OPT_Tree ARGS((tTree yyP2));
static tTree OPT_OuterSpecList ARGS((tTree yyP3));
static tTree OPT_OuterSpec ARGS((tTree yyP4));
static tTree OPT_ProcList ARGS((tTree yyP5));
static tTree OPT_ParProcList ARGS((tTree yyP6));
static tTree OPT_Proc ARGS((tTree yyP7));
static tTree OPT_ReverseProcessList ARGS((tTree yyP9, tTree yyP8));
static bool OPT_IsConst_Expr ARGS((tTree yyP10));
static bool OPT_IsProcInlinable ARGS((tTree yyP11));
static tTree OPT_GetInstanceProcess ARGS((tTree yyP12));
static tTree OPT_GetInstanceSpecs ARGS((tTree yyP14, tTree yyP13));
static tTree OPT_ConvertInstanceFormals ARGS((tTree yyP16, tTree yyP15));
static tTree OPT_SpecList ARGS((tTree yyP17));
static tTree OPT_Spec ARGS((tTree yyP18));
static tTree OPT_ArrayIdList ARGS((tPosition yyP22, int yyP21, tTree yyP20, tTree yyP19));
static tTree OPT_TypeList ARGS((tTree yyP23));
static tTree OPT_Type ARGS((tTree yyP24));
static tTree OPT_IdList ARGS((tTree yyP26, tTree yyP25));
static tTree OPT_ExprList ARGS((tTree yyP27));
static tTree OPT_Expr ARGS((tTree yyP28));
static tTree OPT_ConvertDefFuncToExpr ARGS((tTree yyP30, tTree yyP29));
static tTree OPT_AltList ARGS((tTree yyP31));
static tTree OPT_AltList2 ARGS((tValue yyP36, tTree yyP35, tTree yyP34, tTree yyP33, tPosition yyP32));
static tTree OPT_GetTRUEChoice ARGS((tTree yyP38, tTree yyP37));
static tTree OPT_IfChoiceList ARGS((tTree yyP39));
static tTree OPT_CaseChoiceList ARGS((tTree yyP40));
static tTree OPT_GetTRUECase ARGS((tTree yyP43, tTree yyP42, tTree yyP41));
static bool OPT_IsExprInExprList ARGS((tTree yyP45, tTree yyP44));
static tTree OPT_ChoiceList2 ARGS((tValue yyP50, tTree yyP49, tTree yyP48, tTree yyP47, tPosition yyP46));
static tTree OPT_Guard ARGS((tTree yyP51));
static tTree OPT_Element ARGS((tTree yyP52));
static tTree OPT_CaseList ARGS((tTree yyP53));
static tTree OPT_Elemental ARGS((tTree yyP54));
static tTree OPT_Formals ARGS((tTree yyP55));
static tTree OPT_Replicator ARGS((tTree yyP56));
static tTree OPT_Op ARGS((tTree yyP57));
static tTree OPT_SeqProtocolList ARGS((tTree yyP58));
static tTree OPT_TagProtocolList ARGS((tTree yyP59));
static tTree OPT_Protocol ARGS((tTree yyP60));
static tTree OPT_AddSpecListToProc ARGS((tTree yyP62, tTree yyP61));
static tTree OPT_JoinSpecLists ARGS((tTree yyP64, tTree yyP63));

tTree Optimise
# if defined __STDC__ | defined __cplusplus
(register tTree yyP1)
# else
(yyP1)
 register tTree yyP1;
# endif
{
/* line 322 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree RT; int n;
/* line 323 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 323 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
{
          bool inline_arrays = InlineArraysFlag;
          InlineArraysFlag=false;
          if (InlineProcFlag==true) UniqueNamesFlag=true;
          RT=yyP1;
          do{
            
            
            BeginCoutAg();
            CoutAg(TreeRoot);
            CloseCoutAg();

            OPT_CopyOldSymbolTable();
            UnrollAgain=false;
            RT=OPT_Tree(RT);
            if (ModifiedIdList!=NULL) Message("optimise.trafo: invalid depth after identifier renaming",xxError,NoPosition);
            if (OPT_array_mapping_list!=NULL) Message("optimise.trafo: invalid depth after array inlining",xxError,NoPosition);
            if (!CheckTree(RT)) {fprintf(stderr,"Fatal-ABSTRACT SYNTAX TREE is INVALID\n"); exit(-1);}
            OPT_DeleteOldSymbolTable();

            
            BeginPreTypeChkAg();
              PreTypeChkAg(RT);
            ClosePreTypeChkAg();
            BeginTypeChkAg();
              TypeChkAg(RT);
            CloseTypeChkAg();
            BeginParUsageAg();
              ParUsageAg(RT);
            CloseParUsageAg();

            if ((UnrollAgain==false) && (inline_arrays==true)) {UnrollAgain=true; inline_arrays=false; InlineArraysFlag=true;}
            }while(UnrollAgain==true);
          }
  }
   return RT;

}

static tTree OPT_Tree
# if defined __STDC__ | defined __cplusplus
(register tTree yyP2)
# else
(yyP2)
 register tTree yyP2;
# endif
{
/* line 362 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree RT;
  if (yyP2->Kind == kStart) {
  if (yyP2->Start.Process->Kind == kNullProcess) {
/* line 363 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 364 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 365 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mStart (mNullProcess (yyP2->Start.Process->NullProcess.Pos, OPT_OuterSpecList (yyP2->Start.Process->NullProcess.SpecList)));
/* line 366 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return RT;

  }
  }
 yyAbort ("OPT_Tree");
}

static tTree OPT_OuterSpecList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP3)
# else
(yyP3)
 register tTree yyP3;
# endif
{
/* line 372 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewN; tTree NewS;
  if (yyP3->Kind == kASpec) {
/* line 373 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 374 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewS = OPT_OuterSpec (yyP3->ASpec.Spec);
/* line 375 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewN = OPT_OuterSpecList (yyP3->ASpec.Next);
  }
   return mASpec (NewS, NewN);

  }
  if (yyP3->Kind == kNoSpec) {
/* line 378 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoSpec ();

  }
 yyAbort ("OPT_OuterSpecList");
}

static tTree OPT_OuterSpec
# if defined __STDC__ | defined __cplusplus
(register tTree yyP4)
# else
(yyP4)
 register tTree yyP4;
# endif
{
/* line 384 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tSymbol Sy; tTree RT; tTree NewF; tTree NewP; tTree NewTL; tTree NewT; tTree NewE; tTree NewSPL; tTree NewTPL;

  switch (yyP4->Kind) {
  case kDefProc:
  case kInlineDefProc:
/* line 385 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 386 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 387 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP4->DefProc.Formals);
/* line 388 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP4->DefProc.Process);
/* line 389 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 390 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP4->DefProc.Sym);
/* line 391 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefProc (yyP4->DefProc.Pos, Sy, NewF, NewP);
/* line 392 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kPrototypeProc:
/* line 395 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 396 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP4->PrototypeProc.Formals);
/* line 397 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP4->PrototypeProc.Sym);
/* line 398 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mPrototypeProc (yyP4->PrototypeProc.Pos, Sy, NewF, yyP4->PrototypeProc.Module, yyP4->PrototypeProc.Lang);
/* line 399 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kDefFunc:
  case kInlineDefFunc:
/* line 402 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 403 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 404 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewTL = OPT_TypeList (yyP4->DefFunc.TypeList);
/* line 405 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP4->DefFunc.Formals);
/* line 406 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP4->DefFunc.Sym);
/* line 407 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 408 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefFunc (yyP4->DefFunc.Pos, Sy, NewTL, NewF, yyP4->DefFunc.Valof);
/* line 409 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: not sure what to do with DefFunc node", xxError, yyP4->DefFunc.Pos);
  }
   return RT;

  case kPrototypeFunc:
/* line 412 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 413 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewTL = OPT_TypeList (yyP4->PrototypeFunc.TypeList);
/* line 414 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP4->PrototypeFunc.Formals);
/* line 415 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP4->PrototypeFunc.Sym);
/* line 416 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mPrototypeFunc (yyP4->PrototypeFunc.Pos, Sy, NewTL, NewF, yyP4->PrototypeFunc.Module, yyP4->PrototypeFunc.Lang);
/* line 417 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 418 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewTL;
  }
   return RT;

  case kHCode:
/* line 421 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 422 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: not sure what to do with HCode node", xxError, yyP4->HCode.Pos);
/* line 423 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL5;
  }
yyL5:;

  break;
  case kPragmaTrack:
/* line 425 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return CopyTree (yyP4);

  case kNullSpec:
/* line 428 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNullSpec (yyP4->NullSpec.Pos);

  case kAVal:
  case kVal:
  case kValRetypes:
  case kValReshapes:
/* line 431 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 432 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP4->AVal.Type);
/* line 433 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Expr (yyP4->AVal.Expr);
/* line 434 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP4->AVal.Sym);
/* line 435 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mVal (yyP4->AVal.Pos, NewT, Sy, NewE);
/* line 436 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 437 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
  }
   return RT;

  case kDefSeqPro:
/* line 440 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 441 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP4->DefSeqPro.Sym);
/* line 442 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSPL = OPT_SeqProtocolList (yyP4->DefSeqPro.SeqProList);
/* line 443 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefSeqPro (yyP4->DefSeqPro.Pos, Sy, NewSPL);
/* line 444 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kDefTagPro:
/* line 447 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 448 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP4->DefTagPro.Sym);
/* line 449 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewTPL = OPT_TagProtocolList (yyP4->DefTagPro.TagProList);
/* line 450 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefTagPro (yyP4->DefTagPro.Pos, Sy, NewTPL);
/* line 451 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kDeclType:
/* line 454 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mDeclType (yyP4->DeclType.Pos, OPT_Type (yyP4->DeclType.Type));

  case kPlace:
/* line 457 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mPlace (yyP4->Place.Pos, OPT_GetNewSymbolFromOldSymbol (yyP4->Place.Sym), OPT_Expr (yyP4->Place.Expr));

  }

 yyAbort ("OPT_OuterSpec");
}

static tTree OPT_ProcList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP5)
# else
(yyP5)
 register tTree yyP5;
# endif
{
/* line 463 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewP;
  if (yyP5->Kind == kAProcess) {
/* line 464 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 465 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 466 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP5->AProcess.Process);
/* line 467 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mAProcess (OPT_ProcList (yyP5->AProcess.Next), NewP, NoSymbol);

  }
  if (yyP5->Kind == kNoProcess) {
/* line 470 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoProcess ();

  }
 yyAbort ("OPT_ProcList");
}

static tTree OPT_ParProcList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP6)
# else
(yyP6)
 register tTree yyP6;
# endif
{
/* line 476 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewP; tSymbol Sy; tTree RT;
  if (yyP6->Kind == kAProcess) {
/* line 477 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 478 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 479 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 480 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP6->AProcess.Process);
/* line 481 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 482 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (NoSymbol);
/* line 483 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 484 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mAProcess (OPT_ParProcList (yyP6->AProcess.Next), NewP, NoSymbol);
/* line 485 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  }
  if (yyP6->Kind == kNoProcess) {
/* line 488 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoProcess ();

  }
 yyAbort ("OPT_ParProcList");
}

static tTree OPT_Proc
# if defined __STDC__ | defined __cplusplus
(register tTree yyP7)
# else
(yyP7)
 register tTree yyP7;
# endif
{
/* line 494 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewPL; tTree RT; tValue NewV; tTree NewSL; tTree NewCL; tTree NewAL; tTree NewR; tTree NewP; tSymbol Sy; tTree NewEL; tTree NewE;

  switch (yyP7->Kind) {
  case kAssign:
/* line 495 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 495 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Assign.SpecList);
  }
   return mAssign (yyP7->Assign.Pos, NewSL, OPT_ExprList (yyP7->Assign.Lval), OPT_ExprList (yyP7->Assign.Rval));

  case kAlt:
/* line 497 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 497 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Alt.SpecList);
  }
   return mAlt (yyP7->Alt.Pos, NewSL, yyP7->Alt.Pri, OPT_AltList (yyP7->Alt.AltList));

  case kAltRep:
/* line 499 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 499 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((UnrollAltRepFlag == true))) goto yyL3;
  {
/* line 500 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP7->AltRep.Replicator -> Replicator . From) == true))) goto yyL3;
  {
/* line 501 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP7->AltRep.Replicator -> Replicator . For) == true))) goto yyL3;
  {
/* line 502 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   UnrollAgain = true;
/* line 503 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->AltRep.SpecList);
/* line 504 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewAL = mNoAlt ();
/* line 505 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */

                                  for(NewV.Number=yyP7->AltRep.Replicator->Replicator.FromLE->Value+yyP7->AltRep.Replicator->Replicator.ForLE->Value-1; NewV.Number>=yyP7->AltRep.Replicator->Replicator.FromLE->Value; NewV.Number--)
                                    NewAL = OPT_AltList2(NewV,yyP7->AltRep.AltList,NewAL,yyP7->AltRep.Replicator,yyP7->AltRep.Pos);
                                  
  }
  }
  }
  }
   return mAlt (yyP7->AltRep.Pos, NewSL, yyP7->AltRep.Pri, NewAL);
yyL3:;

/* line 510 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 510 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->AltRep.SpecList);
/* line 511 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 512 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewR = OPT_Replicator (yyP7->AltRep.Replicator);
/* line 513 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewAL = OPT_AltList (yyP7->AltRep.AltList);
/* line 514 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mAltRep (yyP7->AltRep.Pos, NewSL, yyP7->AltRep.Pri, NewR, NewAL);

  case kInput:
/* line 516 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 516 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Input.SpecList);
  }
   return mInput (yyP7->Input.Pos, NewSL, OPT_Element (yyP7->Input.Element), OPT_ExprList (yyP7->Input.ExprList));

  case kOutput:
/* line 518 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 518 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Output.SpecList);
  }
   return mOutput (yyP7->Output.Pos, NewSL, OPT_Element (yyP7->Output.Element), OPT_ExprList (yyP7->Output.ExprList));

  case kCaseIn:
/* line 520 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 520 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->CaseIn.SpecList);
  }
   return mCaseIn (yyP7->CaseIn.Pos, NewSL, yyP7->CaseIn.Element, yyP7->CaseIn.CaseList);

  case kAfter:
/* line 522 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 522 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->After.SpecList);
  }
   return mAfter (yyP7->After.Pos, NewSL, OPT_Element (yyP7->After.Element), OPT_Expr (yyP7->After.Expr));

  case kSeq:
/* line 524 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 524 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Seq.SpecList);
/* line 525 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewPL = OPT_ProcList (yyP7->Seq.ProcList);
  }
   return mSeq (yyP7->Seq.Pos, NewSL, NewPL);

  case kSeqRep:
/* line 527 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 527 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((UnrollSeqRepFlag == true))) goto yyL10;
  {
/* line 528 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP7->SeqRep.Replicator -> Replicator . From) == true))) goto yyL10;
  {
/* line 529 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP7->SeqRep.Replicator -> Replicator . For) == true))) goto yyL10;
  {
/* line 530 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   UnrollAgain = true;
/* line 531 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->SeqRep.SpecList);
/* line 532 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewPL = mNoProcess ();
/* line 533 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */

                                  for(NewV.Number=yyP7->SeqRep.Replicator->Replicator.FromLE->Value; NewV.Number<yyP7->SeqRep.Replicator->Replicator.FromLE->Value+yyP7->SeqRep.Replicator->Replicator.ForLE->Value; NewV.Number++)
                                    {
                                    OpenScope();
                                      {
                                      tTree T       = mInt();
                                      tSymbol S     = OPT_AddSymbol(yyP7->SeqRep.Replicator->Replicator.Sym);
                                      tTree NewVal  = mVal(yyP7->SeqRep.Pos,T,S,mConst(yyP7->SeqRep.Pos,NewV,mUntypedInt()));
                                      SymbolKind(S) = NewVal;
                                      SymbolType(S) = T;
                                      NewPL = mAProcess(NewPL,OPT_AddSpecListToProc(mASpec(NewVal,mNoSpec()),OPT_Proc(yyP7->SeqRep.Process)),NoSymbol);
                                      }
                                    OPT_CloseScope();
                                    }
                                  
/* line 548 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewPL = OPT_ReverseProcessList (NewPL, mNoProcess ());
  }
  }
  }
  }
   return mSeq (yyP7->SeqRep.Pos, NewSL, NewPL);
yyL10:;

/* line 550 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 550 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->SeqRep.SpecList);
/* line 551 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 552 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewR = OPT_Replicator (yyP7->SeqRep.Replicator);
/* line 553 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP7->SeqRep.Process);
/* line 554 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mSeqRep (yyP7->SeqRep.Pos, NewSL, NewR, NewP);

  case kPar:
/* line 556 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 556 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Par.SpecList);
  }
   return mPar (yyP7->Par.Pos, NewSL, yyP7->Par.PType, OPT_ParProcList (yyP7->Par.ProcList));

  case kParRep:
/* line 558 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 558 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((UnrollParRepFlag == true))) goto yyL13;
  {
/* line 559 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP7->ParRep.Replicator -> Replicator . From) == true))) goto yyL13;
  {
/* line 560 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP7->ParRep.Replicator -> Replicator . For) == true))) goto yyL13;
  {
/* line 561 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   UnrollAgain = true;
/* line 562 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->ParRep.SpecList);
/* line 563 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewPL = mNoProcess ();
/* line 564 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */

                                  for(NewV.Number=yyP7->ParRep.Replicator->Replicator.FromLE->Value+yyP7->ParRep.Replicator->Replicator.ForLE->Value-1; NewV.Number>=yyP7->ParRep.Replicator->Replicator.FromLE->Value; NewV.Number--)
                                    {
                                    OpenScope();
                                      {
                                      tTree T       = mInt();
                                      tSymbol S     = OPT_AddSymbol(yyP7->ParRep.Replicator->Replicator.Sym);
                                      tTree NewVal  = mVal(yyP7->ParRep.Pos,T,S,mConst(yyP7->ParRep.Pos,NewV,mUntypedInt()));
                                      SymbolKind(S) = NewVal;
                                      SymbolType(S) = T;
                                      NewPL = mAProcess(NewPL,OPT_AddSpecListToProc(mASpec(NewVal,mNoSpec()),OPT_Proc(yyP7->ParRep.Process)),NoSymbol);
                                      }
                                    OPT_CloseScope();
                                    }
                                  
  }
  }
  }
  }
   return mPar (yyP7->ParRep.Pos, NewSL, yyP7->ParRep.PType, NewPL);
yyL13:;

/* line 580 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 580 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->ParRep.SpecList);
/* line 581 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 582 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 583 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewR = OPT_Replicator (yyP7->ParRep.Replicator);
/* line 584 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 585 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (NoSymbol);
/* line 586 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP7->ParRep.Process);
/* line 587 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 588 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (NoSymbol);
/* line 589 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 590 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 591 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mParRep (yyP7->ParRep.Pos, NewSL, yyP7->ParRep.PType, NewR, NewP, Sy);
/* line 592 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kIf:
  if (yyP7->If.ChoiceList->Kind == kNoChoice) {
/* line 594 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 594 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->If.SpecList);
  }
   return mSkip (yyP7->If.Pos, NewSL);

  }
/* line 596 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 596 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((SimplifyIfFlag == true))) goto yyL16;
  {
/* line 597 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_GetTRUEChoice (yyP7->If.SpecList, yyP7->If.ChoiceList);
/* line 598 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
if (NewP!=NULL) return NewP;
/* line 599 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL16;
  }
  }
yyL16:;

/* line 600 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 600 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->If.SpecList);
/* line 601 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 602 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewCL = OPT_IfChoiceList (yyP7->If.ChoiceList);
/* line 603 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mIf (yyP7->If.Pos, NewSL, NewCL);

  case kIfRep:
/* line 605 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 605 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((UnrollIfRepFlag == true))) goto yyL18;
  {
/* line 606 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP7->IfRep.Replicator -> Replicator . From) == true))) goto yyL18;
  {
/* line 607 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP7->IfRep.Replicator -> Replicator . For) == true))) goto yyL18;
  {
/* line 608 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   UnrollAgain = true;
/* line 609 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->IfRep.SpecList);
/* line 610 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewCL = mNoChoice ();
/* line 611 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */

                                  for(NewV.Number=yyP7->IfRep.Replicator->Replicator.FromLE->Value+yyP7->IfRep.Replicator->Replicator.ForLE->Value-1; NewV.Number>=yyP7->IfRep.Replicator->Replicator.FromLE->Value; NewV.Number--)
                                    NewCL = OPT_ChoiceList2(NewV,yyP7->IfRep.ChoiceList,NewCL,yyP7->IfRep.Replicator,yyP7->IfRep.Pos);
                                  
  }
  }
  }
  }
   return mIf (yyP7->IfRep.Pos, NewSL, NewCL);
yyL18:;

/* line 616 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 616 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->IfRep.SpecList);
/* line 617 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 618 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewR = OPT_Replicator (yyP7->IfRep.Replicator);
/* line 619 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewCL = OPT_IfChoiceList (yyP7->IfRep.ChoiceList);
/* line 620 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mIfRep (yyP7->IfRep.Pos, NewSL, NewR, NewCL);

  case kCase:
/* line 622 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 622 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((SimplifyCaseFlag == true))) goto yyL20;
  {
/* line 623 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_GetTRUECase (yyP7->Case.SpecList, yyP7->Case.Selector, yyP7->Case.ChoiceList);
/* line 624 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
if (NewP!=NULL) return NewP;
/* line 625 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL20;
  }
  }
yyL20:;

/* line 626 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 626 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Case.SpecList);
/* line 627 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Expr (yyP7->Case.Selector);
/* line 628 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewCL = OPT_CaseChoiceList (yyP7->Case.ChoiceList);
  }
   return mCase (yyP7->Case.Pos, NewSL, NewE, NewCL);

  case kWhile:
/* line 630 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 630 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->While.SpecList);
/* line 631 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Expr (yyP7->While.Expr);
/* line 632 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP7->While.Process);
  }
   return mWhile (yyP7->While.Pos, NewSL, NewE, NewP);

  case kSkip:
/* line 634 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 635 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Skip.SpecList);
  }
   return mSkip (yyP7->Skip.Pos, NewSL);

  case kStop:
/* line 637 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 638 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Stop.SpecList);
  }
   return mStop (yyP7->Stop.Pos, NewSL);

  case kCCode:
/* line 640 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 640 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: not sure what to do with CCode node", xxError, yyP7->CCode.Pos);
/* line 641 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL25;
  }
yyL25:;

  break;
  case kInstance:
/* line 642 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 643 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((InlineProcFlag == true))) goto yyL26;
  {
/* line 644 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsProcInlinable (symbol_array [yyP7->Instance.Sym] . Kind) == true))) goto yyL26;
  {
/* line 645 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Instance.SpecList);
/* line 648 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewCL = OPT_GetInstanceSpecs (symbol_array [yyP7->Instance.Sym] . Kind, yyP7->Instance.ExprList);
/* line 649 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = OPT_GetInstanceProcess (symbol_array [yyP7->Instance.Sym] . Kind);
/* line 651 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = OPT_Proc (RT);
/* line 652 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = OPT_AddSpecListToProc (NewCL, RT);
/* line 653 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = OPT_AddSpecListToProc (NewSL, RT);
/* line 654 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   UnrollAgain = true;
  }
  }
  }
   return RT;
yyL26:;

/* line 656 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 656 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->Instance.SpecList);
/* line 657 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP7->Instance.ExprList);
/* line 658 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_GetNewSymbolFromOldSymbol (yyP7->Instance.Sym);
  }
   return mInstance (yyP7->Instance.Pos, NewSL, Sy, NewEL, CurrentScope ());

  case kNullProcess:
/* line 660 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 660 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP7->NullProcess.SpecList);
  }
   return mNullProcess (yyP7->NullProcess.Pos, NewSL);

  }

 yyAbort ("OPT_Proc");
}

static tTree OPT_ReverseProcessList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP9, register tTree yyP8)
# else
(yyP9, yyP8)
 register tTree yyP9;
 register tTree yyP8;
# endif
{
  if (yyP9->Kind == kAProcess) {
/* line 665 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 666 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   yyP8 = mAProcess (yyP8, yyP9->AProcess.Process, NoSymbol);
  }
   return OPT_ReverseProcessList (yyP9->AProcess.Next, yyP8);

  }
  if (yyP9->Kind == kNoProcess) {
/* line 669 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP8;

  }
 yyAbort ("OPT_ReverseProcessList");
}

static bool OPT_IsConst_Expr
# if defined __STDC__ | defined __cplusplus
(register tTree yyP10)
# else
(yyP10)
 register tTree yyP10;
# endif
{

  switch (yyP10->Kind) {
  case kStringElement:
  case kString:
  case kStringSubscript:
  case kStringRange:
/* line 675 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> StringElement . C;

  case kBinary:
/* line 676 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> Binary . C;

  case kUnary:
/* line 677 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> Unary . C;

  case kValof:
/* line 678 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> Valof . C;

  case kFuncCall:
/* line 679 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> FuncCall . C;

  case kConvert:
/* line 680 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> Convert . C;

  case kRound:
/* line 681 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> Round . C;

  case kTrunc:
/* line 682 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> Trunc . C;

  case kCountArray:
/* line 683 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> CountArray . C;

  case kElementalExpr:
/* line 684 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> ElementalExpr . C;

  case kUnBound:
/* line 685 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> UnBound . C;

  case kConst:
/* line 686 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> Const . C;

  case kDefault:
/* line 687 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP10 -> Default . C;

  }

 yyAbort ("OPT_IsConst_Expr");
}

static bool OPT_IsProcInlinable
# if defined __STDC__ | defined __cplusplus
(register tTree yyP11)
# else
(yyP11)
 register tTree yyP11;
# endif
{
  if (Tree_IsType (yyP11, kDefProc)) {
/* line 694 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return true;

  }
  if (yyP11->Kind == kPrototypeProc) {
/* line 697 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return false;

  }
 yyAbort ("OPT_IsProcInlinable");
}

static tTree OPT_GetInstanceProcess
# if defined __STDC__ | defined __cplusplus
(register tTree yyP12)
# else
(yyP12)
 register tTree yyP12;
# endif
{
  if (Tree_IsType (yyP12, kDefProc)) {
/* line 703 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP12->DefProc.Process;

  }
 yyAbort ("OPT_GetInstanceProcess");
}

static tTree OPT_GetInstanceSpecs
# if defined __STDC__ | defined __cplusplus
(register tTree yyP14, register tTree yyP13)
# else
(yyP14, yyP13)
 register tTree yyP14;
 register tTree yyP13;
# endif
{
  if (Tree_IsType (yyP14, kDefProc)) {
/* line 709 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return OPT_ConvertInstanceFormals (yyP14->DefProc.Formals, yyP13);

  }
/* line 712 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 713 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: invalid node detected in OPT_GetInstanceSpecs()", xxError, NoPosition);
/* line 714 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   WriteTree (stdout, yyP14);
/* line 715 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL2;
  }
yyL2:;

 yyAbort ("OPT_GetInstanceSpecs");
}

static tTree OPT_ConvertInstanceFormals
# if defined __STDC__ | defined __cplusplus
(register tTree yyP16, register tTree yyP15)
# else
(yyP16, yyP15)
 register tTree yyP16;
 register tTree yyP15;
# endif
{
/* line 721 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tSymbol Sy; tTree NewS; tTree NewT; tTree NewE;
  if (yyP16->Kind == kAFormal) {
  if (yyP16->AFormal.By->Kind == kByRef) {
  if (yyP15->Kind == kAnExpr) {
  if (yyP15->AnExpr.Expr->Kind == kElementalExpr) {
/* line 722 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 723 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Elemental (yyP15->AnExpr.Expr->ElementalExpr.Elemental);
/* line 724 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP16->AFormal.Type);
/* line 725 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
Sy=(last_type_was_chan==true)?OPT_AddChanSymbol(yyP16->AFormal.Sym):OPT_AddSymbol(yyP16->AFormal.Sym);
/* line 726 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewS = mIs (yyP16->AFormal.Pos, NewT, Sy, NewE);
/* line 727 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = NewS;
/* line 728 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
  }
   return mASpec (NewS, OPT_ConvertInstanceFormals (yyP16->AFormal.Next, yyP15->AnExpr.Next));

  }
  }
  }
  if (yyP16->AFormal.By->Kind == kByVal) {
  if (yyP15->Kind == kAnExpr) {
  if (yyP15->AnExpr.Expr->Kind == kString) {
/* line 730 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 731 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP16->AFormal.Sym);
/* line 732 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewS = mVal (yyP15->AnExpr.Expr->String.Pos, yyP15->AnExpr.Expr->String.Type, Sy, yyP15->AnExpr.Expr);
/* line 733 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = NewS;
/* line 734 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = yyP15->AnExpr.Expr->String.Type;
  }
   return mASpec (NewS, OPT_ConvertInstanceFormals (yyP16->AFormal.Next, yyP15->AnExpr.Next));

  }
/* line 736 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 737 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Expr (yyP15->AnExpr.Expr);
/* line 738 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP16->AFormal.Type);
/* line 739 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
Sy=(last_type_was_chan==true)?OPT_AddChanSymbol(yyP16->AFormal.Sym):OPT_AddSymbol(yyP16->AFormal.Sym);
/* line 740 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewS = mVal (yyP16->AFormal.Pos, NewT, Sy, NewE);
/* line 741 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = NewS;
/* line 742 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
  }
   return mASpec (NewS, OPT_ConvertInstanceFormals (yyP16->AFormal.Next, yyP15->AnExpr.Next));

  }
  }
/* line 746 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 747 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: invalid node detected in OPT_ConvertInstanceFormals()", xxError, NoPosition);
/* line 748 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL5;
  }
yyL5:;

  }
  if (Tree_IsType (yyP16, kNoFormal)) {
  if (yyP15->Kind == kNoExpr) {
/* line 744 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoSpec ();

  }
  }
 yyAbort ("OPT_ConvertInstanceFormals");
}

static tTree OPT_SpecList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP17)
# else
(yyP17)
 register tTree yyP17;
# endif
{
/* line 755 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewS; tTree NewN;
  if (yyP17->Kind == kASpec) {
  if (yyP17->ASpec.Spec->Kind == kDecl) {
  if (Tree_IsType (yyP17->ASpec.Spec->Decl.Type, kArray)) {
/* line 756 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 757 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((InlineArraysFlag == true))) goto yyL1;
  {
/* line 758 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP17->ASpec.Spec->Decl.Type->Array.Expr -> Expr . C == true))) goto yyL1;
  {
/* line 759 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! (UnrollAgain = true)) goto yyL1;
  {
/* line 760 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewS = OPT_ArrayIdList (yyP17->ASpec.Spec->Decl.Pos, yyP17->ASpec.Spec->Decl.Type->Array.Expr -> Expr . CV . Number, yyP17->ASpec.Spec->Decl.Type->Array.Type, yyP17->ASpec.Spec->Decl.IdList);
/* line 761 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewN = OPT_SpecList (yyP17->ASpec.Next);
  }
  }
  }
  }
   return OPT_JoinSpecLists (NewS, NewN);
yyL1:;

  }
  }
/* line 764 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 765 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewS = OPT_Spec (yyP17->ASpec.Spec);
/* line 766 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewN = OPT_SpecList (yyP17->ASpec.Next);
/* line 769 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
if (NewS==NULL) return NewN;
  }
   return mASpec (NewS, NewN);

  }
  if (yyP17->Kind == kNoSpec) {
/* line 774 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoSpec ();

  }
 yyAbort ("OPT_SpecList");
}

static tTree OPT_Spec
# if defined __STDC__ | defined __cplusplus
(register tTree yyP18)
# else
(yyP18)
 register tTree yyP18;
# endif
{
/* line 783 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tSymbol Sy; tTree NewT; tTree NewIL; tTree RT; tTree NewE; tTree NewF; tTree NewP; tTree NewTL; tTree NewEL; tTree NewSPL; tTree NewTPL; tTree TL;

  switch (yyP18->Kind) {
  case kIs:
/* line 784 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 785 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP18->Is.Type);
/* line 786 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Elemental (yyP18->Is.Elemental);
/* line 787 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->Is.Sym);
/* line 788 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mIs (yyP18->Is.Pos, NewT, Sy, NewE);
/* line 789 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 790 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
  }
   return RT;

  case kRetypes:
  case kReshapes:
/* line 793 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 794 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: not sure what to do with Retypes node", xxError, yyP18->Retypes.Pos);
/* line 795 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP18->Retypes.Type);
  }
   return mRetypes (yyP18->Retypes.Pos, NewT, yyP18->Retypes.Sym, yyP18->Retypes.Elemental);

  case kVal:
/* line 798 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 799 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP18->Val.Type);
/* line 800 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Expr (yyP18->Val.Expr);
/* line 801 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->Val.Sym);
/* line 802 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mVal (yyP18->Val.Pos, NewT, Sy, NewE);
/* line 803 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 804 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
  }
   return RT;

  case kValRetypes:
  case kValReshapes:
/* line 807 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 808 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: not sure what to do with ValRetypes node", xxError, yyP18->ValRetypes.Pos);
/* line 809 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP18->ValRetypes.Type);
  }
   return mValRetypes (yyP18->ValRetypes.Pos, NewT, yyP18->ValRetypes.Sym, yyP18->ValRetypes.Expr);

  case kDefProc:
  case kInlineDefProc:
/* line 812 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 813 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((InlineProcFlag == true))) goto yyL5;
  }
   return NULL;
yyL5:;

/* line 817 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 818 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 819 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP18->DefProc.Formals);
/* line 820 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP18->DefProc.Process);
/* line 821 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 822 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->DefProc.Sym);
/* line 823 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefProc (yyP18->DefProc.Pos, Sy, NewF, NewP);
/* line 824 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kDefIsFunc:
  case kInlineDefIsFunc:
/* line 827 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 828 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((InlineFuncFlag == true))) goto yyL7;
  }
   return NULL;
yyL7:;

/* line 832 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 833 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 834 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP18->DefIsFunc.Formals);
/* line 835 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP18->DefIsFunc.ExprList);
/* line 836 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 837 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewTL = OPT_TypeList (yyP18->DefIsFunc.TypeList);
/* line 838 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->DefIsFunc.Sym);
/* line 839 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefIsFunc (yyP18->DefIsFunc.Pos, Sy, NewTL, NewF, NewEL);
/* line 840 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 841 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewTL;
  }
   return RT;

  case kDefFunc:
  case kInlineDefFunc:
/* line 844 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 845 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((InlineFuncFlag == true))) goto yyL9;
  }
   return NULL;
yyL9:;

/* line 849 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 850 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 851 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP18->DefFunc.Formals);
/* line 852 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Expr (yyP18->DefFunc.Valof);
/* line 853 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 854 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewTL = OPT_TypeList (yyP18->DefFunc.TypeList);
/* line 855 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->DefFunc.Sym);
/* line 856 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefFunc (yyP18->DefFunc.Pos, Sy, NewTL, NewF, NewE);
/* line 857 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 858 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewTL;
/* line 859 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE -> Valof . Sym = Sy;
  }
   return RT;

  case kPrototypeProc:
/* line 862 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 863 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP18->PrototypeProc.Formals);
/* line 864 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->PrototypeProc.Sym);
/* line 865 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mPrototypeProc (yyP18->PrototypeProc.Pos, Sy, NewF, yyP18->PrototypeProc.Module, yyP18->PrototypeProc.Lang);
/* line 866 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kPrototypeFunc:
/* line 869 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 870 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewTL = OPT_TypeList (TL);
/* line 871 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewF = OPT_Formals (yyP18->PrototypeFunc.Formals);
/* line 872 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->PrototypeFunc.Sym);
/* line 873 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mPrototypeFunc (yyP18->PrototypeFunc.Pos, Sy, NewTL, NewF, yyP18->PrototypeFunc.Module, yyP18->PrototypeFunc.Lang);
/* line 874 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 875 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewTL;
  }
   return RT;

  case kDeclType:
/* line 878 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 879 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP18->DeclType.Type);
  }
   return mDeclType (yyP18->DeclType.Pos, NewT);

  case kDecl:
/* line 882 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 883 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP18->Decl.Type);
/* line 884 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewIL = OPT_IdList (NewT, yyP18->Decl.IdList);
/* line 885 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
{
    if (NewIL->Kind==kNoId) return NULL;
    }
  }
   return mDecl (yyP18->Decl.Pos, NewT, NewIL);

  case kNullSpec:
/* line 890 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNullSpec (yyP18->NullSpec.Pos);

  case kDefSeqPro:
/* line 893 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 894 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->DefSeqPro.Sym);
/* line 895 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSPL = OPT_SeqProtocolList (yyP18->DefSeqPro.SeqProList);
/* line 896 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefSeqPro (yyP18->DefSeqPro.Pos, Sy, NewSPL);
/* line 897 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kDefTagPro:
/* line 900 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 901 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP18->DefTagPro.Sym);
/* line 902 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewTPL = OPT_TagProtocolList (yyP18->DefTagPro.TagProList);
/* line 903 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mDefTagPro (yyP18->DefTagPro.Pos, Sy, NewTPL);
/* line 904 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kPlace:
/* line 907 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mPlace (yyP18->Place.Pos, OPT_GetNewSymbolFromOldSymbol (yyP18->Place.Sym), OPT_Expr (yyP18->Place.Expr));

  case kHCode:
/* line 910 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 911 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: not sure what to do with HCode node", xxError, yyP18->HCode.Pos);
/* line 912 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL19;
  }
yyL19:;

  break;
  case kPragmaTrack:
/* line 914 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return CopyTree (yyP18);

  }

/* line 917 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 918 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: unknown node detected", xxError, NoPosition);
/* line 919 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL21;
  }
yyL21:;

 yyAbort ("OPT_Spec");
}

static tTree OPT_ArrayIdList
# if defined __STDC__ | defined __cplusplus
(register tPosition yyP22, register int yyP21, register tTree yyP20, register tTree yyP19)
# else
(yyP22, yyP21, yyP20, yyP19)
 register tPosition yyP22;
 register int yyP21;
 register tTree yyP20;
 register tTree yyP19;
# endif
{
/* line 925 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewIL; char* NewSt; tIdent NewId; tSymbol NewSy; tTree NewS1; tTree NewT; tTree NewEx;
         tTree NewCL; int n; tSymbol NewSy2; tTree NewFL;
  if (yyP19->Kind == kAnId) {
/* line 927 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 928 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
{
    NewIL = mNoId();

    
    
    
    for(n=(yyP21-1); n>=0; n--)
      {
      NewSt = OPT_GetIdealArrayName(yyP19->AnId.Sym,n);
      NewId = MakeIdent(NewSt,strlen(NewSt));
      NewSy = AddSymbol(NewId);
      NewIL = mAnId(NewIL,NewSy);
      NewT  = OPT_Type(yyP20);
      SymbolKind(NewSy) = NewIL;
      SymbolType(NewSy) = NewT;
      free(NewSt);

      
      OPT_AddArrayMapping(yyP19->AnId.Sym,NewSy,n);
      }
    NewS1 = mDecl(yyP22,NewT,NewIL);
    }
  }
   return mASpec (NewS1, OPT_ArrayIdList (yyP22, yyP21, yyP20, yyP19->AnId.Next));

  }
  if (yyP19->Kind == kNoId) {
/* line 952 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoSpec ();

  }
 yyAbort ("OPT_ArrayIdList");
}

static tTree OPT_TypeList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP23)
# else
(yyP23)
 register tTree yyP23;
# endif
{
/* line 959 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewT; tTree NewN;
  if (yyP23->Kind == kAType) {
/* line 960 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 961 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP23->AType.Type);
/* line 962 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewN = OPT_TypeList (yyP23->AType.Next);
  }
   return mAType (NewN, NewT);

  }
  if (yyP23->Kind == kNoType) {
/* line 965 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoType ();

  }
 yyAbort ("OPT_TypeList");
}

static tTree OPT_Type
# if defined __STDC__ | defined __cplusplus
(register tTree yyP24)
# else
(yyP24)
 register tTree yyP24;
# endif
{
/* line 973 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewE; tTree NewP;

  switch (yyP24->Kind) {
  case kBool:
  case kUserBool:
/* line 974 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 974 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mBool ();

  case kByte:
  case kUserByte:
/* line 975 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 975 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mByte ();

  case kInt:
  case kUserInt:
/* line 976 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 976 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mInt ();

  case kInt16:
  case kUserInt16:
/* line 977 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 977 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mInt16 ();

  case kInt32:
  case kUserInt32:
/* line 978 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 978 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mInt32 ();

  case kInt64:
  case kUserInt64:
/* line 979 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 979 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mInt64 ();

  case kReal32:
  case kUserReal32:
/* line 980 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 980 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mReal32 ();

  case kReal64:
  case kUserReal64:
/* line 981 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 981 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mReal64 ();

  case kUntypedByte:
/* line 982 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 982 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mUntypedByte ();

  case kUntypedInt:
/* line 983 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 983 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mUntypedInt ();

  case kUntyped:
  case kUntypedReal:
  case kUntypedInteger:
  case kUntypedHex:
/* line 984 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 984 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mUntyped ();

  case kArray:
  case kUserArray:
/* line 985 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 985 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewE = OPT_Expr (yyP24->Array.Expr);
  }
   return mArray (NewE, OPT_Type (yyP24->Array.Type));

  case kChan:
/* line 986 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 986 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Protocol (yyP24->Chan.Protocol);
/* line 986 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = true;
  }
   return mChan (NewP);

  case kTimer:
/* line 987 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 987 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   last_type_was_chan = false;
  }
   return mTimer ();

  }

 yyAbort ("OPT_Type");
}

static tTree OPT_IdList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP26, register tTree yyP25)
# else
(yyP26, yyP25)
 register tTree yyP26;
 register tTree yyP25;
# endif
{
/* line 993 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree RT; tSymbol Sy;
  if (yyP25->Kind == kAnId) {
/* line 994 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 995 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((RemoveUnusedFlag == true))) goto yyL1;
  {
/* line 996 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsOldSymbolUsed (yyP25->AnId.Sym) == false))) goto yyL1;
  }
  }
   return OPT_IdList (yyP26, yyP25->AnId.Next);
yyL1:;

/* line 999 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1000 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
Sy = (last_type_was_chan==true) ? OPT_AddChanSymbol(yyP25->AnId.Sym) : OPT_AddSymbol(yyP25->AnId.Sym);
/* line 1001 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mAnId (OPT_IdList (yyP26, yyP25->AnId.Next), Sy);
/* line 1002 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 1003 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = yyP26;
  }
   return RT;

  }
  if (yyP25->Kind == kNoId) {
/* line 1006 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoId ();

  }
 yyAbort ("OPT_IdList");
}

static tTree OPT_ExprList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP27)
# else
(yyP27)
 register tTree yyP27;
# endif
{
  if (yyP27->Kind == kAnExpr) {
/* line 1012 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mAnExpr (OPT_ExprList (yyP27->AnExpr.Next), OPT_Expr (yyP27->AnExpr.Expr));

  }
  if (yyP27->Kind == kNoExpr) {
/* line 1013 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoExpr ();

  }
 yyAbort ("OPT_ExprList");
}

static tTree OPT_Expr
# if defined __STDC__ | defined __cplusplus
(register tTree yyP28)
# else
(yyP28)
 register tTree yyP28;
# endif
{
/* line 1017 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree RT; tSymbol Sy; tTree NewP; tTree NewSL; tTree NewEL; tTree NewE; tTree NewS; tTree NewT;
  if (yyP28->Kind == kString) {
/* line 1018 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1019 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (NoSymbol);
/* line 1020 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mString (yyP28->String.Pos, yyP28->String.Value, OPT_Type (yyP28->String.Type), Depth, Sy);
/* line 1021 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  }
  if (yyP28->Kind == kElementalExpr) {
  if (yyP28->ElementalExpr.Elemental->Kind == kUnmappedIdentifier) {
/* line 1024 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1025 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP28 -> ElementalExpr . C == true))) goto yyL2;
  }
   return mConst (yyP28->ElementalExpr.Pos, yyP28 -> ElementalExpr . CV, OPT_Type (yyP28 -> ElementalExpr . SynType));
yyL2:;

  }
  if (Tree_IsType (yyP28->ElementalExpr.Elemental, kIdentifier)) {
/* line 1028 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1029 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP28 -> ElementalExpr . C == true))) goto yyL3;
  }
   return mConst (yyP28->ElementalExpr.Pos, yyP28 -> ElementalExpr . CV, OPT_Type (yyP28 -> ElementalExpr . SynType));
yyL3:;

  }
  if (yyP28->ElementalExpr.Elemental->Kind == kSubscript) {
/* line 1032 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1033 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP28 -> ElementalExpr . C == true))) goto yyL4;
  }
   return mConst (yyP28->ElementalExpr.Pos, yyP28 -> ElementalExpr . CV, OPT_Type (yyP28 -> ElementalExpr . SynType));
yyL4:;

  }
/* line 1036 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mElementalExpr (yyP28->ElementalExpr.Pos, OPT_Elemental (yyP28->ElementalExpr.Elemental));

  }
/* line 1039 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1040 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL6;
  {
/* line 1041 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mConst (yyP28 -> Expr . Pos, yyP28 -> Expr . CV, OPT_Type (yyP28 -> Expr . SynType));
  }
  }
   return RT;
yyL6:;


  switch (yyP28->Kind) {
  case kBinary:
/* line 1044 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mBinary (yyP28->Binary.Pos, OPT_Expr (yyP28->Binary.Lop), OPT_Expr (yyP28->Binary.Rop), OPT_Op (yyP28->Binary.Op));

  case kUnary:
/* line 1047 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mUnary (yyP28->Unary.Pos, OPT_Expr (yyP28->Unary.Expr), OPT_Op (yyP28->Unary.Op));

  case kValof:
/* line 1050 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1051 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1052 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1053 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP28->Valof.SpecList);
/* line 1054 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1055 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP28->Valof.Process);
/* line 1056 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1057 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP28->Valof.ExprList);
/* line 1058 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1059 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (NoSymbol);
/* line 1060 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1061 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mValof (yyP28->Valof.Pos, NewSL, NewP, NewEL, Sy, Depth);
/* line 1062 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  case kFuncCall:
/* line 1065 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1066 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((InlineFuncFlag == true))) goto yyL10;
  {
/* line 1067 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   UnrollAgain = true;
  }
  }
   return OPT_ConvertDefFuncToExpr (symbol_array [yyP28->FuncCall.Sym] . Kind, yyP28->FuncCall.ExprList);
yyL10:;

/* line 1070 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1071 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP28->FuncCall.ExprList);
  }
   return mFuncCall (yyP28->FuncCall.Pos, OPT_GetNewSymbolFromOldSymbol (yyP28->FuncCall.Sym), NewEL, Depth);

  case kConvert:
/* line 1074 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1075 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP28->Convert.Type);
  }
   return mConvert (yyP28->Convert.Pos, NewT, OPT_Expr (yyP28->Convert.Expr));

  case kRound:
/* line 1078 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1079 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP28->Round.Type);
  }
   return mRound (yyP28->Round.Pos, NewT, OPT_Expr (yyP28->Round.Expr));

  case kTrunc:
/* line 1082 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1083 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP28->Trunc.Type);
  }
   return mTrunc (yyP28->Trunc.Pos, NewT, OPT_Expr (yyP28->Trunc.Expr));

  case kCountArray:
/* line 1086 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mCountArray (yyP28->CountArray.Pos, yyP28->CountArray.Length, OPT_Expr (yyP28->CountArray.Expr));

  case kUnBound:
/* line 1089 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mUnBound (yyP28->UnBound.Pos);

  case kConst:
/* line 1092 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mConst (yyP28->Const.Pos, yyP28->Const.Value, OPT_Type (yyP28->Const.Type));

  case kDefault:
/* line 1095 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mDefault (yyP28->Default.Pos);

  }

/* line 1098 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1099 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Message ("optimise.trafo: unknown node detected", xxError, NoPosition);
/* line 1100 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   goto yyL19;
  }
yyL19:;

 yyAbort ("OPT_Expr");
}

static tTree OPT_ConvertDefFuncToExpr
# if defined __STDC__ | defined __cplusplus
(register tTree yyP30, register tTree yyP29)
# else
(yyP30, yyP29)
 register tTree yyP30;
 register tTree yyP29;
# endif
{
/* line 1105 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewSL; tTree NewP; tTree NewEL; tTree RT; tSymbol Sy;
  if (Tree_IsType (yyP30, kDefFunc)) {
/* line 1106 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1107 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1108 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_ConvertInstanceFormals (yyP30->DefFunc.Formals, yyP29);
/* line 1109 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_JoinSpecLists (NewSL, OPT_SpecList (yyP30->DefFunc.Valof->Valof.SpecList));
/* line 1110 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1111 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP30->DefFunc.Valof->Valof.Process);
/* line 1112 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1113 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP30->DefFunc.Valof->Valof.ExprList);
/* line 1114 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1115 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (NoSymbol);
/* line 1116 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mValof (yyP30->DefFunc.Pos, NewSL, NewP, NewEL, Sy, Depth);
/* line 1117 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  }
 yyAbort ("OPT_ConvertDefFuncToExpr");
}

static tTree OPT_AltList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP31)
# else
(yyP31)
 register tTree yyP31;
# endif
{
/* line 1124 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewSL; tTree NewP; tTree NewG;
  if (yyP31->Kind == kAnAlt) {
/* line 1125 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1126 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1127 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP31->AnAlt.SpecList);
/* line 1128 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewG = OPT_Guard (yyP31->AnAlt.Guard);
/* line 1129 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mAnAlt (NewSL, OPT_AltList (yyP31->AnAlt.Next), NewG);

  }
  if (yyP31->Kind == kANewAlt) {
/* line 1132 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1133 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1134 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP31->ANewAlt.SpecList);
/* line 1135 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP31->ANewAlt.AltType);
/* line 1136 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mANewAlt (NewSL, OPT_AltList (yyP31->ANewAlt.Next), NewP);

  }
  if (yyP31->Kind == kNoAlt) {
/* line 1139 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoAlt ();

  }
 yyAbort ("OPT_AltList");
}

static tTree OPT_AltList2
# if defined __STDC__ | defined __cplusplus
(tValue yyP36, register tTree yyP35, register tTree yyP34, register tTree yyP33, register tPosition yyP32)
# else
(yyP36, yyP35, yyP34, yyP33, yyP32)
 tValue yyP36;
 register tTree yyP35;
 register tTree yyP34;
 register tTree yyP33;
 register tPosition yyP32;
# endif
{
/* line 1147 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewSL; tTree NewP; tTree NewG; tTree NewT; tSymbol Sy; tTree NewV;
  if (yyP35->Kind == kAnAlt) {
/* line 1148 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1149 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1150 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = mInt ();
/* line 1151 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP33 -> Replicator . Sym);
/* line 1152 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewV = mVal (yyP32, NewT, Sy, mConst (yyP32, yyP36, mUntypedInt ()));
/* line 1153 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = NewV;
/* line 1154 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
/* line 1155 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP35->AnAlt.SpecList);
/* line 1156 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewG = OPT_Guard (yyP35->AnAlt.Guard);
/* line 1157 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mAnAlt (mASpec (NewV, NewSL), OPT_AltList2 (yyP36, yyP35->AnAlt.Next, yyP34, yyP33, yyP32), NewG);

  }
  if (yyP35->Kind == kANewAlt) {
/* line 1160 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1161 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1162 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = mInt ();
/* line 1163 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP33 -> Replicator . Sym);
/* line 1164 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewV = mVal (yyP32, NewT, Sy, mConst (yyP32, yyP36, mUntypedInt ()));
/* line 1165 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = NewV;
/* line 1166 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
/* line 1167 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP35->ANewAlt.SpecList);
/* line 1168 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP35->ANewAlt.AltType);
/* line 1169 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mANewAlt (mASpec (NewV, NewSL), OPT_AltList2 (yyP36, yyP35->ANewAlt.Next, yyP34, yyP33, yyP32), OPT_Proc (yyP35->ANewAlt.AltType));

  }
  if (yyP35->Kind == kNoAlt) {
/* line 1172 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP34;

  }
 yyAbort ("OPT_AltList2");
}

static tTree OPT_GetTRUEChoice
# if defined __STDC__ | defined __cplusplus
(register tTree yyP38, register tTree yyP37)
# else
(yyP38, yyP37)
 register tTree yyP38;
 register tTree yyP37;
# endif
{
/* line 1179 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewP; tTree NewSL;
  if (yyP37->Kind == kAChoice) {
  if (yyP37->AChoice.ExprList->Kind == kAnExpr) {
/* line 1180 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1182 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP37->AChoice.ExprList->AnExpr.Expr) == true))) goto yyL1;
  {
/* line 1183 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP37->AChoice.ExprList->AnExpr.Expr -> Expr . CV . Number == true))) goto yyL1;
  {
/* line 1184 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP38);
/* line 1185 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_JoinSpecLists (NewSL, OPT_SpecList (yyP37->AChoice.SpecList));
/* line 1186 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP37->AChoice.Process);
/* line 1187 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_AddSpecListToProc (NewSL, yyP37->AChoice.Process);
  }
  }
  }
   return NewP;
yyL1:;

  }
  if (yyP37->AChoice.SpecList->Kind == kNoSpec) {
  if (yyP37->AChoice.ExprList->Kind == kAnExpr) {
/* line 1190 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1192 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP37->AChoice.ExprList->AnExpr.Expr) == true))) goto yyL2;
  {
/* line 1193 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP37->AChoice.ExprList->AnExpr.Expr -> Expr . CV . Number == false))) goto yyL2;
  }
  }
   return OPT_GetTRUEChoice (yyP38, yyP37->AChoice.Next);
yyL2:;

  }
  }
  }
/* line 1196 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return NULL;

}

static tTree OPT_IfChoiceList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP39)
# else
(yyP39)
 register tTree yyP39;
# endif
{
/* line 1204 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewSL; tTree RT; tTree NewEL; tTree NewP;
  if (yyP39->Kind == kAChoice) {
  if (yyP39->AChoice.SpecList->Kind == kNoSpec) {
  if (yyP39->AChoice.ExprList->Kind == kAnExpr) {
/* line 1205 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1206 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((SimplifyIfFlag == true))) goto yyL1;
  {
/* line 1207 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsConst_Expr (yyP39->AChoice.ExprList->AnExpr.Expr) == true))) goto yyL1;
  {
/* line 1208 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP39->AChoice.ExprList->AnExpr.Expr -> Expr . CV . Number == false))) goto yyL1;
  {
/* line 1209 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   UnrollAgain = true;
  }
  }
  }
  }
   return OPT_IfChoiceList (yyP39->AChoice.Next);
yyL1:;

  }
  }
/* line 1212 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1213 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP39->AChoice.SpecList);
/* line 1214 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP39->AChoice.ExprList);
/* line 1215 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP39->AChoice.Process);
/* line 1216 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1217 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
  }
   return mAChoice (NewSL, OPT_IfChoiceList (yyP39->AChoice.Next), NewEL, NewP);

  }
  if (yyP39->Kind == kANewIf) {
/* line 1220 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1221 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP39->ANewIf.SpecList);
/* line 1222 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP39->ANewIf.IfType);
/* line 1223 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1224 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
  }
   return mANewIf (NewSL, OPT_IfChoiceList (yyP39->ANewIf.Next), NewP);

  }
  if (yyP39->Kind == kNoChoice) {
/* line 1227 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoChoice ();

  }
 yyAbort ("OPT_IfChoiceList");
}

static tTree OPT_CaseChoiceList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP40)
# else
(yyP40)
 register tTree yyP40;
# endif
{
/* line 1234 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewSL; tTree RT; tTree NewEL; tTree NewP;
  if (yyP40->Kind == kAChoice) {
/* line 1235 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1236 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP40->AChoice.SpecList);
/* line 1237 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP40->AChoice.ExprList);
/* line 1238 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP40->AChoice.Process);
/* line 1239 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1240 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
  }
   return mAChoice (NewSL, OPT_CaseChoiceList (yyP40->AChoice.Next), NewEL, NewP);

  }
  if (yyP40->Kind == kANewIf) {
/* line 1243 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1244 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP40->ANewIf.SpecList);
/* line 1245 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP40->ANewIf.IfType);
/* line 1246 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
/* line 1247 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
  }
   return mANewIf (NewSL, OPT_CaseChoiceList (yyP40->ANewIf.Next), NewP);

  }
  if (yyP40->Kind == kNoChoice) {
/* line 1250 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoChoice ();

  }
 yyAbort ("OPT_CaseChoiceList");
}

static tTree OPT_GetTRUECase
# if defined __STDC__ | defined __cplusplus
(register tTree yyP43, register tTree yyP42, register tTree yyP41)
# else
(yyP43, yyP42, yyP41)
 register tTree yyP43;
 register tTree yyP42;
 register tTree yyP41;
# endif
{
/* line 1256 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewSL; tTree NewP;
  if (yyP41->Kind == kAChoice) {
/* line 1257 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1258 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((OPT_IsExprInExprList (yyP42, yyP41->AChoice.ExprList) == true))) goto yyL1;
  {
/* line 1259 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP43);
/* line 1260 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_JoinSpecLists (NewSL, OPT_SpecList (yyP41->AChoice.SpecList));
/* line 1261 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP41->AChoice.Process);
/* line 1262 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_AddSpecListToProc (NewSL, yyP41->AChoice.Process);
  }
  }
   return NewP;
yyL1:;

  if (yyP41->AChoice.SpecList->Kind == kNoSpec) {
/* line 1265 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return OPT_GetTRUECase (yyP43, yyP42, yyP41->AChoice.Next);

  }
  }
/* line 1268 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return NULL;

}

static bool OPT_IsExprInExprList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP45, register tTree yyP44)
# else
(yyP45, yyP44)
 register tTree yyP45;
 register tTree yyP44;
# endif
{
  if (yyP44->Kind == kAnExpr) {
  if (yyP44->AnExpr.Expr->Kind == kDefault) {
/* line 1274 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1275 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP45 -> Expr . C == true))) goto yyL1;
  }
   return true;
yyL1:;

  }
/* line 1278 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1279 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP45 -> Expr . C == true))) goto yyL2;
  {
/* line 1280 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP44->AnExpr.Expr -> Expr . C == true))) goto yyL2;
  {
/* line 1281 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP45 -> Expr . CV . Number == yyP44->AnExpr.Expr -> Expr . CV . Number))) goto yyL2;
  }
  }
  }
   return true;
yyL2:;

/* line 1284 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return OPT_IsExprInExprList (yyP45, yyP44->AnExpr.Next);

  }
  if (yyP44->Kind == kNoExpr) {
/* line 1287 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return false;

  }
 yyAbort ("OPT_IsExprInExprList");
}

static tTree OPT_ChoiceList2
# if defined __STDC__ | defined __cplusplus
(tValue yyP50, register tTree yyP49, register tTree yyP48, register tTree yyP47, register tPosition yyP46)
# else
(yyP50, yyP49, yyP48, yyP47, yyP46)
 tValue yyP50;
 register tTree yyP49;
 register tTree yyP48;
 register tTree yyP47;
 register tPosition yyP46;
# endif
{
/* line 1295 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewSL; tTree RT; tTree NewEL; tTree NewP; tTree NewT; tSymbol Sy; tTree NewV;
  if (yyP49->Kind == kAChoice) {
/* line 1296 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1297 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1298 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = mInt ();
/* line 1299 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP47 -> Replicator . Sym);
/* line 1300 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewV = mVal (yyP46, NewT, Sy, mConst (yyP46, yyP50, mUntypedInt ()));
/* line 1301 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = NewV;
/* line 1302 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
/* line 1303 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP49->AChoice.SpecList);
/* line 1304 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP49->AChoice.ExprList);
/* line 1305 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP49->AChoice.Process);
/* line 1306 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mAChoice (mASpec (NewV, NewSL), OPT_ChoiceList2 (yyP50, yyP49->AChoice.Next, yyP48, yyP47, yyP46), NewEL, NewP);

  }
  if (yyP49->Kind == kANewIf) {
/* line 1309 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1310 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1311 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = mInt ();
/* line 1312 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP47 -> Replicator . Sym);
/* line 1313 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewV = mVal (yyP46, NewT, Sy, mConst (yyP46, yyP50, mUntypedInt ()));
/* line 1314 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = NewV;
/* line 1315 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
/* line 1316 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP49->ANewIf.SpecList);
/* line 1317 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP49->ANewIf.IfType);
/* line 1318 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mANewIf (mASpec (NewV, NewSL), OPT_ChoiceList2 (yyP50, yyP49->ANewIf.Next, yyP48, yyP47, yyP46), NewP);

  }
  if (yyP49->Kind == kNoChoice) {
/* line 1321 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP48;

  }
 yyAbort ("OPT_ChoiceList2");
}

static tTree OPT_Guard
# if defined __STDC__ | defined __cplusplus
(register tTree yyP51)
# else
(yyP51)
 register tTree yyP51;
# endif
{
/* line 1327 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewP; tTree NewSL;
  if (yyP51->Kind == kAltInput) {
/* line 1328 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1329 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP51->AltInput.Input->Input.SpecList);
/* line 1330 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1331 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP51->AltInput.Process);
/* line 1332 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mAltInput (yyP51->AltInput.Pos, OPT_Expr (yyP51->AltInput.Expr), mInput (yyP51->AltInput.Input->Input.Pos, NewSL, OPT_Element (yyP51->AltInput.Input->Input.Element), OPT_ExprList (yyP51->AltInput.Input->Input.ExprList)), NewP);

  }
  if (yyP51->Kind == kAltTimer) {
/* line 1335 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1336 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP51->AltTimer.After->After.SpecList);
/* line 1337 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1338 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP51->AltTimer.Process);
/* line 1339 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mAltTimer (yyP51->AltTimer.Pos, OPT_Expr (yyP51->AltTimer.Expr), mAfter (yyP51->AltTimer.After->After.Pos, NewSL, OPT_Element (yyP51->AltTimer.After->After.Element), OPT_Expr (yyP51->AltTimer.After->After.Expr)), NewP);

  }
  if (yyP51->Kind == kAltCaseIn) {
/* line 1342 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1343 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP51->AltCaseIn.CaseIn->CaseIn.SpecList);
  }
   return mAltCaseIn (yyP51->AltCaseIn.Pos, OPT_Expr (yyP51->AltCaseIn.Expr), mCaseIn (yyP51->AltCaseIn.CaseIn->CaseIn.Pos, NewSL, OPT_Element (yyP51->AltCaseIn.CaseIn->CaseIn.Element), OPT_CaseList (yyP51->AltCaseIn.CaseIn->CaseIn.CaseList)));

  }
  if (yyP51->Kind == kNoInput) {
/* line 1346 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1347 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OpenScope ();
/* line 1348 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP51->NoInput.Process);
/* line 1349 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   OPT_CloseScope ();
  }
   return mNoInput (yyP51->NoInput.Pos, OPT_Expr (yyP51->NoInput.Expr), NewP);

  }
 yyAbort ("OPT_Guard");
}

static tTree OPT_Element
# if defined __STDC__ | defined __cplusplus
(register tTree yyP52)
# else
(yyP52)
 register tTree yyP52;
# endif
{
/* line 1355 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tSymbol Sy;
  if (yyP52->Kind == kUnmappedIdentifier) {
/* line 1356 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1357 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_GetNewSymbolFromOldSymbol (yyP52->UnmappedIdentifier.Sym);
  }
   return mUnmappedIdentifier (yyP52->UnmappedIdentifier.Pos, NoSymbol, CurrentScope (), Symbol (Sy) . Id);

  }
  if (Tree_IsType (yyP52, kIdentifier)) {
/* line 1360 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1361 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_GetNewSymbolFromOldSymbol (yyP52->Identifier.Sym);
  }
   return mIdentifier (yyP52->Identifier.Pos, Sy, CurrentScope ());

  }
  if (yyP52->Kind == kSubscript) {
  if (yyP52->Subscript.Next->Kind == kUnmappedIdentifier) {
/* line 1364 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1365 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((InlineArraysFlag == true))) goto yyL3;
  {
/* line 1366 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP52->Subscript.Expr -> Expr . C == true))) goto yyL3;
  {
/* line 1367 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_LookupArrayMapping (yyP52->Subscript.Next->UnmappedIdentifier.Sym, yyP52->Subscript.Expr -> Expr . CV . Number);
  }
  }
  }
   return mUnmappedIdentifier (yyP52->Subscript.Pos, Sy, CurrentScope (), Symbol (Sy) . Id);
yyL3:;

  }
/* line 1370 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mSubscript (yyP52->Subscript.Pos, OPT_Element (yyP52->Subscript.Next), OPT_Expr (yyP52->Subscript.Expr));

  }
  if (yyP52->Kind == kRange) {
  if (yyP52->Range.For->Kind == kFuncCall) {
/* line 1373 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mRange (yyP52->Range.Pos, OPT_Element (yyP52->Range.Next), OPT_Expr (yyP52->Range.From), OPT_Expr (yyP52->Range.For));

  }
/* line 1376 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mRange (yyP52->Range.Pos, OPT_Element (yyP52->Range.Next), OPT_Expr (yyP52->Range.From), OPT_Expr (yyP52->Range.For));

  }
 yyAbort ("OPT_Element");
}

static tTree OPT_CaseList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP53)
# else
(yyP53)
 register tTree yyP53;
# endif
{
/* line 1382 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewSL; tTree NewEL; tTree NewP;
  if (yyP53->Kind == kACase) {
/* line 1383 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1384 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   yyP53->ACase.ExprList = OPT_ExprList (yyP53->ACase.ExprList);
/* line 1385 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewSL = OPT_SpecList (yyP53->ACase.SpecList);
/* line 1386 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Proc (yyP53->ACase.Process);
  }
   return mACase (OPT_CaseList (yyP53->ACase.Next), NewSL, NewEL, NewP);

  }
  if (yyP53->Kind == kNoCase) {
/* line 1389 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoCase ();

  }
 yyAbort ("OPT_CaseList");
}

static tTree OPT_Elemental
# if defined __STDC__ | defined __cplusplus
(register tTree yyP54)
# else
(yyP54)
 register tTree yyP54;
# endif
{
/* line 1395 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewEL; tTree RT; tSymbol Sy; tSymbol SC;
  if (yyP54->Kind == kUnmappedIdentifier) {
/* line 1396 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1397 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_GetNewSymbolFromOldSymbol (yyP54->UnmappedIdentifier.Sym);
  }
   return mUnmappedIdentifier (yyP54->UnmappedIdentifier.Pos, Sy, CurrentScope (), Symbol (Sy) . Id);

  }
  if (Tree_IsType (yyP54, kIdentifier)) {
/* line 1400 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1401 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_GetNewSymbolFromOldSymbol (yyP54->Identifier.Sym);
/* line 1402 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mIdentifier (yyP54->Identifier.Pos, Sy, CurrentScope ());
  }
   return RT;

  }
  if (yyP54->Kind == kSubscript) {
  if (yyP54->Subscript.Next->Kind == kUnmappedIdentifier) {
/* line 1405 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1406 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((InlineArraysFlag == true))) goto yyL3;
  {
/* line 1407 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   if (! ((yyP54->Subscript.Expr -> Expr . C == true))) goto yyL3;
  {
/* line 1408 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_LookupArrayMapping (yyP54->Subscript.Next->UnmappedIdentifier.Sym, yyP54->Subscript.Expr -> Expr . CV . Number);
  }
  }
  }
   return mUnmappedIdentifier (yyP54->Subscript.Pos, Sy, CurrentScope (), Symbol (Sy) . Id);
yyL3:;

  }
/* line 1411 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mSubscript (yyP54->Subscript.Pos, OPT_Element (yyP54->Subscript.Next), OPT_Expr (yyP54->Subscript.Expr));

  }
  if (Tree_IsType (yyP54, kTable)) {
/* line 1414 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1415 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewEL = OPT_ExprList (yyP54->Table.ExprList);
/* line 1416 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP54 -> Table . Sym);
/* line 1417 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mTable (yyP54->Table.Pos, NewEL, Depth, Sy);
/* line 1418 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
  }
   return RT;

  }
 yyAbort ("OPT_Elemental");
}

static tTree OPT_Formals
# if defined __STDC__ | defined __cplusplus
(register tTree yyP55)
# else
(yyP55)
 register tTree yyP55;
# endif
{
/* line 1424 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tSymbol Sy; tTree RT; tTree NewT;
  if (Tree_IsType (yyP55, kNoFormal)) {
/* line 1425 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoFormal ();

  }
  if (yyP55->Kind == kAFormal) {
/* line 1428 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1429 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP55->AFormal.Sym);
/* line 1430 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP55->AFormal.Type);
/* line 1431 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mAFormal (OPT_Formals (yyP55->AFormal.Next), yyP55->AFormal.Pos, yyP55->AFormal.By, NewT, Sy);
/* line 1432 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 1433 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = NewT;
  }
   return RT;

  }
 yyAbort ("OPT_Formals");
}

static tTree OPT_Replicator
# if defined __STDC__ | defined __cplusplus
(register tTree yyP56)
# else
(yyP56)
 register tTree yyP56;
# endif
{
/* line 1439 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree RT; tSymbol Sy; tTree NewFrom; tTree NewFor;
/* line 1440 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1441 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewFrom = OPT_Expr (yyP56->Replicator.From);
/* line 1442 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewFor = OPT_Expr (yyP56->Replicator.For);
/* line 1443 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   Sy = OPT_AddSymbol (yyP56->Replicator.Sym);
/* line 1444 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   RT = mReplicator (Sy, NewFrom, NewFor);
/* line 1445 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolKind (Sy) = RT;
/* line 1446 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   SymbolType (Sy) = mInt ();
  }
   return RT;

}

static tTree OPT_Op
# if defined __STDC__ | defined __cplusplus
(register tTree yyP57)
# else
(yyP57)
 register tTree yyP57;
# endif
{

  switch (yyP57->Kind) {
  case kLess:
/* line 1452 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mLess ();

  case kGreater:
/* line 1453 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mGreater ();

  case kLessEqual:
/* line 1454 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mLessEqual ();

  case kGreaterEqual:
/* line 1455 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mGreaterEqual ();

  case kEqual:
/* line 1456 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mEqual ();

  case kNotEqual:
/* line 1457 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNotEqual ();

  case kAdd:
/* line 1458 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mAdd ();

  case kSubtract:
/* line 1459 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mSubtract ();

  case kMultiply:
/* line 1460 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mMultiply ();

  case kDivide:
/* line 1461 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mDivide ();

  case kRem:
/* line 1462 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mRem ();

  case kBitAnd:
/* line 1463 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mBitAnd ();

  case kBitOr:
/* line 1464 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mBitOr ();

  case kBitEor:
/* line 1465 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mBitEor ();

  case kBitNot:
/* line 1466 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mBitNot ();

  case kLeftShift:
/* line 1467 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mLeftShift ();

  case kRightShift:
/* line 1468 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mRightShift ();

  case kSize:
/* line 1469 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mSize ();

  case kPlus:
/* line 1470 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mPlus ();

  case kMinus:
/* line 1471 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mMinus ();

  case kTimes:
/* line 1472 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mTimes ();

  case kAfterOp:
/* line 1473 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mAfterOp ();

  case kAnd:
/* line 1474 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mAnd ();

  case kOr:
/* line 1475 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mOr ();

  case kNot:
/* line 1476 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNot ();

  }

 yyAbort ("OPT_Op");
}

static tTree OPT_SeqProtocolList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP58)
# else
(yyP58)
 register tTree yyP58;
# endif
{
/* line 1480 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewN; tTree NewP;
  if (yyP58->Kind == kASeqPro) {
/* line 1481 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1482 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewP = OPT_Protocol (yyP58->ASeqPro.Protocol);
/* line 1483 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewN = OPT_SeqProtocolList (yyP58->ASeqPro.Next);
  }
   return mASeqPro (NewN, NewP);

  }
  if (yyP58->Kind == kNoSeqPro) {
/* line 1486 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoSeqPro ();

  }
 yyAbort ("OPT_SeqProtocolList");
}

static tTree OPT_TagProtocolList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP59)
# else
(yyP59)
 register tTree yyP59;
# endif
{
  if (yyP59->Kind == kATagPro) {
/* line 1492 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mATagPro (yyP59->ATagPro.Next, yyP59->ATagPro.Sym, yyP59->ATagPro.SeqProList);

  }
  if (yyP59->Kind == kNoTagPro) {
/* line 1495 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNoTagPro ();

  }
 yyAbort ("OPT_TagProtocolList");
}

static tTree OPT_Protocol
# if defined __STDC__ | defined __cplusplus
(register tTree yyP60)
# else
(yyP60)
 register tTree yyP60;
# endif
{
/* line 1501 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
tTree NewT;
  if (yyP60->Kind == kCounted) {
/* line 1502 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1503 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP60->Counted.Type);
  }
   return mCounted (yyP60->Counted.Pos, yyP60->Counted.Length, NewT);

  }
  if (yyP60->Kind == kNamed) {
/* line 1506 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNamed (yyP60->Named.Pos, OPT_GetNewSymbolFromOldSymbol (yyP60->Named.Sym));

  }
  if (yyP60->Kind == kSimple) {
/* line 1509 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
  {
/* line 1510 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   NewT = OPT_Type (yyP60->Simple.Type);
  }
   return mSimple (yyP60->Simple.Pos, NewT);

  }
 yyAbort ("OPT_Protocol");
}

static tTree OPT_AddSpecListToProc
# if defined __STDC__ | defined __cplusplus
(register tTree yyP62, register tTree yyP61)
# else
(yyP62, yyP61)
 register tTree yyP62;
 register tTree yyP61;
# endif
{

  switch (yyP61->Kind) {
  case kAssign:
/* line 1517 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mAssign (yyP61->Assign.Pos, OPT_JoinSpecLists (yyP62, yyP61->Assign.SpecList), yyP61->Assign.Lval, yyP61->Assign.Rval);

  case kAlt:
/* line 1518 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mAlt (yyP61->Alt.Pos, OPT_JoinSpecLists (yyP62, yyP61->Alt.SpecList), yyP61->Alt.Pri, yyP61->Alt.AltList);

  case kAltRep:
/* line 1519 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mAltRep (yyP61->AltRep.Pos, OPT_JoinSpecLists (yyP62, yyP61->AltRep.SpecList), yyP61->AltRep.Pri, yyP61->AltRep.Replicator, yyP61->AltRep.AltList);

  case kInput:
/* line 1520 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mInput (yyP61->Input.Pos, OPT_JoinSpecLists (yyP62, yyP61->Input.SpecList), yyP61->Input.Element, yyP61->Input.ExprList);

  case kOutput:
/* line 1521 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mOutput (yyP61->Output.Pos, OPT_JoinSpecLists (yyP62, yyP61->Output.SpecList), yyP61->Output.Element, yyP61->Output.ExprList);

  case kCaseIn:
/* line 1522 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mCaseIn (yyP61->CaseIn.Pos, OPT_JoinSpecLists (yyP62, yyP61->CaseIn.SpecList), yyP61->CaseIn.Element, yyP61->CaseIn.CaseList);

  case kAfter:
/* line 1523 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mAfter (yyP61->After.Pos, OPT_JoinSpecLists (yyP62, yyP61->After.SpecList), yyP61->After.Element, yyP61->After.Expr);

  case kSeq:
/* line 1524 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mSeq (yyP61->Seq.Pos, OPT_JoinSpecLists (yyP62, yyP61->Seq.SpecList), yyP61->Seq.ProcList);

  case kSeqRep:
/* line 1525 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mSeqRep (yyP61->SeqRep.Pos, OPT_JoinSpecLists (yyP62, yyP61->SeqRep.SpecList), yyP61->SeqRep.Replicator, yyP61->SeqRep.Process);

  case kPar:
/* line 1526 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mPar (yyP61->Par.Pos, OPT_JoinSpecLists (yyP62, yyP61->Par.SpecList), yyP61->Par.PType, yyP61->Par.ProcList);

  case kParRep:
/* line 1527 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mParRep (yyP61->ParRep.Pos, OPT_JoinSpecLists (yyP62, yyP61->ParRep.SpecList), yyP61->ParRep.PType, yyP61->ParRep.Replicator, yyP61->ParRep.Process, NoSymbol);

  case kIf:
/* line 1528 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mIf (yyP61->If.Pos, OPT_JoinSpecLists (yyP62, yyP61->If.SpecList), yyP61->If.ChoiceList);

  case kIfRep:
/* line 1529 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mIfRep (yyP61->IfRep.Pos, OPT_JoinSpecLists (yyP62, yyP61->IfRep.SpecList), yyP61->IfRep.Replicator, yyP61->IfRep.ChoiceList);

  case kCase:
/* line 1530 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mCase (yyP61->Case.Pos, OPT_JoinSpecLists (yyP62, yyP61->Case.SpecList), yyP61->Case.Selector, yyP61->Case.ChoiceList);

  case kWhile:
/* line 1531 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mWhile (yyP61->While.Pos, OPT_JoinSpecLists (yyP62, yyP61->While.SpecList), yyP61->While.Expr, yyP61->While.Process);

  case kSkip:
/* line 1532 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mSkip (yyP61->Skip.Pos, OPT_JoinSpecLists (yyP62, yyP61->Skip.SpecList));

  case kStop:
/* line 1533 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mStop (yyP61->Stop.Pos, OPT_JoinSpecLists (yyP62, yyP61->Stop.SpecList));

  case kCCode:
/* line 1534 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mCCode (yyP61->CCode.Pos, OPT_JoinSpecLists (yyP62, yyP61->CCode.SpecList), yyP61->CCode.Code, NoSymbol);

  case kInstance:
/* line 1535 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mInstance (yyP61->Instance.Pos, OPT_JoinSpecLists (yyP62, yyP61->Instance.SpecList), yyP61->Instance.Sym, yyP61->Instance.ExprList, NoSymbol);

  case kNullProcess:
/* line 1536 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mNullProcess (yyP61->NullProcess.Pos, OPT_JoinSpecLists (yyP62, yyP61->NullProcess.SpecList));

  }

 yyAbort ("OPT_AddSpecListToProc");
}

static tTree OPT_JoinSpecLists
# if defined __STDC__ | defined __cplusplus
(register tTree yyP64, register tTree yyP63)
# else
(yyP64, yyP63)
 register tTree yyP64;
 register tTree yyP63;
# endif
{
  if (yyP64->Kind == kASpec) {
  if (yyP63->Kind == kASpec) {
/* line 1541 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return mASpec (yyP64->ASpec.Spec, OPT_JoinSpecLists (yyP64->ASpec.Next, yyP63));

  }
  }
  if (yyP64->Kind == kNoSpec) {
  if (yyP63->Kind == kASpec) {
/* line 1542 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP63;

  }
  }
  if (yyP63->Kind == kNoSpec) {
/* line 1543 "/home/simon/spoc/source/spoc.1.3//source/gmd/optimise.trafo" */
   return yyP64;

  }
 yyAbort ("OPT_JoinSpecLists");
}

void BeginOptimise ()
{
}

void CloseOptimise ()
{
}
