# include "SMVout.h"
# include "yySMVout.w"
# include "System.h"
# include <stdio.h>
# include "Tree.h"

# define yyInline
# ifndef NULL
# define NULL 0L
# endif
# ifndef false
# define false 0
# endif
# ifndef true
# define true 1
# endif

# ifdef yyInline
# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
  free += nodesize [kind]; \
  ptr->yyHead.yyMark = 0; \
  ptr->Kind = kind;
# else
# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
# endif

# define yyWrite(s) (void) fputs (s, yyf)
# define yyWriteNl (void) fputc ('\n', yyf)

/* line 90 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */

  #include <stdio.h>

  static void SMV_Tree(tTree);

  static FILE* fh;
  static FILE* datfh;
  static char strbuff[100];
  static int resbuff[15];
  static char* strbuff2;

  SMV_PCs PCHead = NULL;
  SMV_PCs PCTail = NULL;
  SMV_Channels Channels = NULL;
  SMV_Variables Variables = NULL;

  static int NextIntId;
  static int NextPCId;
  static int NextResult;

  /* Adds a Program Counter entry to the .dat file */
  static SMV_AddDatPCEntry(tPosition pos,int pcid,int pcval)
  {
    fprintf(datfh,"pc%d_%d:#%d\n",pcid,pcval,pos.Line);
  }

  /* Creates a new program counter */
  static int SMV_NewPC()
  {
    if (PCTail == NULL) {
      PCHead = (SMV_PCs)malloc(sizeof(struct pcs));
      PCTail = PCHead;
    }
    else {
      PCTail->next = (SMV_PCs)malloc(sizeof(struct pcs));
      PCTail = PCTail->next;
    }
    if (PCTail==NULL) Message("SMV.trafo: malloc error in NewPC",xxFatal,NoPosition);
    PCTail->id = NextPCId;
    PCTail->next = NULL;
    PCTail->cases = NULL;
    PCTail->maximum = 0;
    return NextPCId++;
  }

  /* Creates a new channel */
  static int SMV_NewChan()
  {
    if (Channels==NULL) {
      Channels = (SMV_Channels)malloc(sizeof(struct channel_cases));
      Channels->chanid = 1;
      Channels->next = NULL;
    }
    else {
      SMV_Channels newchan = (SMV_Channels)malloc(sizeof(struct channel_cases));
      newchan->chanid = Channels->chanid+1;
      newchan->next=Channels;
      Channels=newchan;
    }
    Channels->cases=NULL;

    return Channels->chanid;
  }

  /* Creates a new integer */
  static int SMV_NewInt(int min, int max)
  {
    if (Variables==NULL) {
      Variables = (SMV_Variables)malloc(sizeof(struct variable_cases));
      Variables->varid = 1;
      Variables->next = NULL;
    }
    else {
      SMV_Variables newvar = (SMV_Variables)malloc(sizeof(struct variable_cases));
      newvar->varid = Variables->varid+1;
      newvar->next=Variables;
      Variables=newvar;
    }
    Variables->cases=NULL;
    Variables->max=max;
    Variables->min=min;

    return Variables->varid;
  }

  static void SMV_AddPCcase(int id, char* condition, char* result)
  {
    SMV_PCs currentvar;
    SMV_PC_Cases currentcase;
    char *cond, *res;
    int i;

    cond = (char*)malloc(strlen(condition)+1);
    if (cond==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);
    strcpy(cond,condition);
    res = (char*)malloc(strlen(result)+1);
    if (cond==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);
    strcpy(res,result);
    /*printf("AddVarCase: id=%d, cond=%s, result=%s\n",id,cond,result);*/

    for(currentvar=PCHead;currentvar && (currentvar->id!=id);currentvar=currentvar->next);
    if (currentvar==NULL) Message("SMV.trafo: can't find Variable 1",xxFatal,NoPosition);
    currentcase = (SMV_PC_Cases)malloc(sizeof(struct pc_cases));
    if (currentcase==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);

    currentcase->condition = cond;
    currentcase->result = res;
    currentcase->next = currentvar->cases;
    currentvar->cases = currentcase;
  }

  static void SMV_AddPCcase1(int pc,char *cond, int res)
  {
    char result[5];
    sprintf(result,"%d",res);
    SMV_AddPCcase(pc,cond,result);
  }

  static void SMV_AddPCcase2(int pc,char *cond,int res, int alt)
  {
    char result[10];
    sprintf(result,"{%d,%d",res,alt);
    SMV_AddPCcase(pc,cond,result);
  }

  static void SMV_AddIfCase(int pc,int val1,int val2)
  {
    int i;
    char result[30];
    sprintf(strbuff+strlen(strbuff),"&(pc%d=%d)",pc,val1);
    if(NextResult==0)
      sprintf(result,"%d",val2);
    else {
      sprintf(result,"{");
      for(i=0; i<NextResult; i++)
        sprintf(result+strlen(result),"%d,",resbuff[i]);
      sprintf(result+strlen(result),"%d",result,val2);
    }
    SMV_AddPCcase(pc,strbuff,result);
  }

  static void SMV_AddCaseCase(int pc,int val1,int val2)
  {
    int i;
    char result[30];
    sprintf(strbuff+strlen(strbuff),"=%s&(pc%d=%d)",strbuff2,pc,val1);
    if(NextResult==0)
      sprintf(result,"%d",val2);
    else {
      sprintf(result,"{");
      for(i=0; i<NextResult; i++)
        sprintf(result+strlen(result),"%d,",resbuff[i]);
      sprintf(result+strlen(result),"%d",val2);
    }
    SMV_AddPCcase(pc,strbuff,result);
  }

  static void SMV_AddIntCase3(int varid,int pcid,int pcval,int pcid2,int pcval2,
                              ioType what,char* boolexpr,void *data)
  {
    SMV_Variables currentvar;
    struct variable_cases *thecase;
    /*printf("AddVarCase: id=%d, pcid=%d, pcval=%d, what=%d\n",varid,pcid,pcval,what);*/

    for(currentvar=Variables; currentvar && currentvar->varid!=varid;
        currentvar=currentvar->next);
    if (currentvar==NULL) Message("SMV.trafo: Can't find Variable 2",xxFatal,NoPosition);

    thecase = (struct variable_cases*)malloc(sizeof(struct variable_cases));
    if (thecase==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);

    if (what==from_tracked_expr) {
      thecase->data = (void*)malloc(strlen((char*)data)+1);
      if (thecase->data==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);
      strcpy((char*)thecase->data,(char*)data);
    }
    else thecase->data = data;

    if (boolexpr) {
      thecase->boolexpr = (char*)malloc(strlen(boolexpr)+1);
      if (thecase->boolexpr==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);
      strcpy(thecase->boolexpr,boolexpr);
    }
    else thecase->boolexpr = boolexpr;

    /*printf("AddVarCase: id=%d, pcid=%d, pcval=%d, what=%d\n",varid,pcid,pcval,what);*/

    thecase->what=what;
    thecase->pcid=pcid;
    thecase->pcval=pcval;
    thecase->pcid2=pcid2;
    thecase->pcval2=pcval2;
    thecase->next=currentvar->cases;
    currentvar->cases=thecase;
  }

  static void SMV_AddIntCase2(int varid,int pcid,int pcval,int pcid2,int pcval2,
                              ioType what,void *data)
  {
    SMV_AddIntCase3(varid,pcid,pcval,pcid2,pcval2,what,NULL,data);
  }

  static void SMV_AddIntCase(int varid,int pcid,int pcval,ioType what,void *data)
  {
    SMV_AddIntCase3(varid,pcid,pcval,-1,-1,what,NULL,data);
  }

  static void SMV_AddChanCase(int chanid,int pcid,int pcval,ioType what,void *data)
  {
    SMV_Channels currentvar;
    struct channel_cases *thecase;

    for(currentvar=Channels; currentvar && currentvar->chanid!=chanid;
        currentvar=currentvar->next);
    if (currentvar==NULL) Message("SMV.trafo: can't find Variable 3",xxFatal,NoPosition);

    thecase = (struct channel_cases*)malloc(sizeof(struct channel_cases));
    if (thecase==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);

    if (what==from_tracked_expr) {
      thecase->data = (void*)malloc(strlen((char*)data)+1);
      if (thecase->data==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);
      strcpy((char*)thecase->data,(char*)data);
    }
    else thecase->data = data;

    /*printf("AddChanCase: id=%d, pcid=%d, pcval=%d, what=%d\n",chanid,pcid,pcval,what);*/

    thecase->what=what;
    thecase->pcid=pcid;
    thecase->pcval=pcval;
    thecase->next=currentvar->cases;
    currentvar->cases=thecase;
  }

  static void SMV_AddAltCase(int chanid,int pcid,int pcval,char *boolexpr,int varid,int pcval2)
  {
    SMV_Alternation alt = (SMV_Alternation)malloc(sizeof(struct alternation));
    if (alt==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);
    if(boolexpr) {
      alt->boolexpr = (char*)malloc(strlen(boolexpr)+1);
      if (alt->boolexpr==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);
      strcpy(alt->boolexpr,boolexpr);
    }
    else
      boolexpr=NULL;
    alt->varid=varid;
    alt->pcval2=pcval2;
    SMV_AddChanCase(chanid,pcid,pcval,varid>0 ? alt_tracked_var : chanid>0 ? alt_unknown : alt_skip,(void*)alt);
  }

  static void SMV_SetPCmax(int pc, int max)
  {
    SMV_PCs current;
    /*printf("SetPCmax: pc=%d, max=%d\n",pc,max);*/
    for(current=PCHead; current && current->id!=pc; current=current->next);
    if (current==NULL) Message("SMV.trafo: can't find Program Counter 1",xxFatal,NoPosition);
    current->maximum = max;
  }

  static void SMV_StartPC(int pcid,int pcval,int newpc)
  {
    char cond[10];
    /*printf("StartPC: pc=%d, val=%d, newpc=%d\n",pcid,pcval,newpc);*/
    sprintf(cond,"pc%d=%d",pcid,pcval);
    SMV_AddPCcase1(newpc,cond,1);
  }

  SMV_Variable_Cases SMV_AmendAltList(SMV_Variable_Cases list,int pcid,int pcval,
    int destval,int outid,int outval,char *boolexpr)
  {
    SMV_Variable_Cases newentry,newlist,next;

    newentry = (SMV_Variable_Cases)malloc(sizeof(struct variable_cases));
    newentry->pcid=pcid;
    newentry->pcval=pcval;
    newentry->pcid2=outid;
    newentry->pcval2=outval;
    newentry->data=(void*)destval;
    newentry->boolexpr=boolexpr;
    newentry->next=NULL;

    if(list==NULL) return newentry;
    for(next=list; next; next=next->next) {
      if((pcid<next->pcid)||(pcid==next->pcid&&pcval<next->pcval)||
         (pcid==next->pcid&&pcval==next->pcval&&destval<=(int)next->data))
      {
        newentry->next=next;
        if(next==list)
          return newentry;
        else {
          newlist->next=newentry;
          return list;
        }
      }
      else
        newlist=next;
    }
    newlist->next = newentry;
    return list;
  }

  SMV_PC_Cases SMV_addtolist(int bitlist,int len,SMV_Variable_Cases current,SMV_PC_Cases cases)
  {
    int j,include,stop;
    SMV_Variable_Cases curcase,next;
    SMV_PC_Cases newcase;

    *strbuff=0;
    NextResult=0;
    stop=1;

    /*printcode(bitlist,len);*/

    curcase=current;
    for(j=1<<(len-1);j;j=j>>1) {
      include=j&bitlist;
      if(include && current->pcid2)
        sprintf(strbuff+strlen(strbuff),"&((pc%d=%d)",current->pcid2,current->pcval2);
      for(next=current->next;next && next->pcid==current->pcid && next->pcval==current->pcval
           && next->data==current->data;next=next->next)
        if(include && next->pcid2)
          sprintf(strbuff+strlen(strbuff),"|(pc%d=%d)",next->pcid2,next->pcval2);
      if(include)
        if(current->boolexpr) {
          stop=0;
          if(current->pcid2)
            strcat(strbuff,")");
          if(*current->boolexpr!='1' || *(current->boolexpr+1))
            sprintf(strbuff+strlen(strbuff),"&%s",current->boolexpr);
        }
        else
          if(current->pcid2)
            strcat(strbuff,")");
      if(include)
        resbuff[NextResult++]=(int)current->data;

      current=next;
    }

    newcase=(SMV_PC_Cases)malloc(sizeof(struct pc_cases));
    newcase->condition=(char*)malloc(strlen(strbuff)+10);
    if(*strbuff==0)
      sprintf(newcase->condition,"(pc%d=%d)",curcase->pcid,curcase->pcval);
    else
      sprintf(newcase->condition,"(pc%d=%d)%s",curcase->pcid,curcase->pcval,strbuff);

    if(NextResult)
      if(stop)
        sprintf(strbuff,"{%d,%d",curcase->pcval+1,resbuff[0]);
      else if(NextResult==1)
        sprintf(strbuff,"%d",resbuff[0]);
      else
        sprintf(strbuff,"{%d",resbuff[0]);
    else
      sprintf(strbuff,"%d",curcase->pcval+1);
    while(NextResult>1)
      sprintf(strbuff+strlen(strbuff),",%d",resbuff[--NextResult]);
    newcase->result=(char*)malloc(strlen(strbuff)+1);
    strcpy(newcase->result,strbuff);
    newcase->next=cases;
    return newcase;
  }

  SMV_PC_Cases SMV_codelist(int bitlist,int pos,int len,SMV_Variable_Cases current,SMV_PC_Cases cases)
  {
    int i,j;
    cases=SMV_addtolist(bitlist,len,current,cases);
    for(i=pos;i && (bitlist&i)==0;i=i>>1);
    if(i==0) return cases;

    for(j=i<<1;j<=pos;j=j<<1) {
      bitlist&=~(j>>1);
      bitlist|=j;
      cases=SMV_codelist(bitlist,j>>1,len,current,cases);
    }
    bitlist|=i;
    bitlist&=~(j>>1);
    return cases;
  }

  void SMV_DoAlts(SMV_Variable_Cases list)
  {
    SMV_Variable_Cases current,previous,next;
    SMV_PC_Cases cases,curcase;
    SMV_PCs curpc;
    int pcid,pcval,len;
    int i,bitlist;

    for(current=list;current;)
    {
      /*find len*/
      for(len=0,next=current;next && next->pcid==current->pcid && next->pcval==current->pcval;
          next=next->next,len++)
        if(next->data==current->data) len--;
      /*printf("len is: %d\n",len);*/

      /*now use len to make a list of cases*/
      for(cases=NULL,i=0;i<=len;i++) {
        /*printf("here%d...",i);*/
        bitlist=(1<<i)-1;
        cases=SMV_codelist(bitlist,1<<(len-1),len,current,cases);
      }
      /*printf("finding pc%d\n",current->pcid);*/
      /*now add the pc case list to the pc list*/
      for(curpc=PCHead; curpc && curpc->id!=current->pcid; curpc=curpc->next);
      if (curpc==NULL) Message("SMV.trafo: can't find Program Counter 2",xxFatal,NoPosition);
      for(curcase=curpc->cases;curcase && curcase->next;curcase=curcase->next);
      if(curcase==NULL)
        curpc->cases=cases;
      else
        curcase->next=cases;

      /*now free up the alt cases that are finished with*/
      for(previous=current;current!=next;previous=previous->next) {
        current=previous->next;
        free(previous);
      }
    }
  }

  static void SMV_Do_Channels()
  {
    SMV_Channels chan;
    SMV_Alternation alt;
    SMV_Channel_Cases incase, outcase;
    SMV_Variable_Cases altlist = NULL;

    for (chan=Channels; chan; chan=chan->next)
      for(incase=chan->cases; incase; incase=incase->next)
        if(incase->what & INPUT)
        {
          for(outcase=chan->cases; outcase; outcase=outcase->next)
            if(outcase->what & OUTPUT)
            {
              sprintf(strbuff,"(pc%d=%d)&(pc%d=%d)",incase->pcid,
                 incase->pcval,outcase->pcid,outcase->pcval);
              SMV_AddPCcase1(incase->pcid,strbuff,incase->pcval+1);
              SMV_AddPCcase1(outcase->pcid,strbuff,outcase->pcval+1);
              if(incase->what & TRACKED)
                SMV_AddIntCase2((int)incase->data,incase->pcid,incase->pcval,
                  outcase->pcid,outcase->pcval,outcase->what,outcase->data);
            }
        }
        else if(incase->what & ALTGUARD)
        {
          alt=(SMV_Alternation)incase->data;
          if(incase->what == alt_skip)
            altlist=SMV_AmendAltList(altlist,incase->pcid,incase->pcval,
               alt->pcval2,0,0,alt->boolexpr);
          else
           for(outcase=chan->cases; outcase; outcase=outcase->next)
            if(outcase->what & OUTPUT)
            {
              altlist=SMV_AmendAltList(altlist,incase->pcid,incase->pcval,
                 alt->pcval2,outcase->pcid,outcase->pcval,alt->boolexpr);

              if(alt->boolexpr)
                sprintf(strbuff,"(pc%d=%d)&(pc%d=%d)&%s",incase->pcid,incase->pcval,
                  outcase->pcid,outcase->pcval,alt->boolexpr);
              else
                sprintf(strbuff,"(pc%d=%d)&(pc%d=%d)",incase->pcid,incase->pcval,
                  outcase->pcid,outcase->pcval);

              SMV_AddPCcase1(outcase->pcid,strbuff,outcase->pcval+1);
              if(incase->what & TRACKED) {
                /*printf("here %d\n",incase->what);*/
                SMV_AddIntCase3(alt->varid,incase->pcid,incase->pcval,outcase->pcid,
                  outcase->pcval,outcase->what,alt->boolexpr,outcase->data);
                /*printf("there\n");*/

              }
            }
        }
        /*printf("Traversing tree-hehehe\n");*/

    SMV_DoAlts(altlist);
  }

/********************************************************************************/
/* Output file modules */
/********************************************************************************/

  /* Outputs code at the start of the SMV program. */
  static void SMV_Header(void)
  {
    fprintf(fh,"-- SMV output from occ2c\n");
    fprintf(fh,"\nMODULE main\n");
  }

  /* Outputs a list of variables in the SMV program */
  static void SMV_OutputVariables(void)
  {
    SMV_PCs ctr;
    SMV_Variables var;
    fprintf(fh,"\nVAR\n");
    fprintf(fh,"  termination : boolean;\n");
    if(Variables) fprintf(fh,"  rangeerror : boolean;\n");
    for(var=Variables; var; var=var->next)
      fprintf(fh,"  int%d : %d..%d;\n",var->varid,var->min,var->max);
    for(ctr=PCHead; ctr; ctr=ctr->next)
      fprintf(fh,"  pc%d : 0..%d;\n",ctr->id,ctr->maximum);
  }

  SMV_PC_Cases SMV_AddRangeCheck(SMV_PC_Cases list,int pcid,int pcval,
                                 int pcid2,int pcval2,char *boolexpr,char *expr,int min,int max)
  {
    SMV_PC_Cases newlist;

    newlist = (SMV_PC_Cases)malloc(sizeof(struct pc_cases));

    if(pcid2==-1) {
      fprintf(fh,"    (pc%d=%d)&(%s in %d..%d) : %s;\n",pcid,pcval,expr,min,max,expr);
      sprintf(strbuff,"(pc%d=%d)&~(%s in %d..%d)",pcid,pcval,expr,min,max);
    }
    else {
      if(boolexpr==NULL) {
        fprintf(fh,"    (pc%d=%d)&(pc%d=%d)&~(%s in %d..%d) : %s;\n",pcid,pcval,pcid2,
                pcval2,expr,min,max,expr);
        sprintf(strbuff,"(pc%d=%d)&(pc%d=%d)&~(%s in %d..%d)",pcid,pcval,pcid2,pcval2,
                expr,min,max);
      }
      else {
        fprintf(fh,"    %s&(pc%d=%d)&(pc%d=%d)&~(%s in %d..%d) : %s;\n",boolexpr,pcid,pcval,pcid2,
                pcval2,expr,min,max,expr);
        sprintf(strbuff,"%s&(pc%d=%d)&(pc%d=%d)&~(%s in %d..%d)",boolexpr,pcid,pcval,pcid2,pcval2,
                expr,min,max);
      }
    }

    newlist->condition = (char*)malloc(strlen(strbuff)+1);
    if (newlist->condition==NULL) Message("SMV.trafo: malloc() error",xxFatal,NoPosition);
    strcpy(newlist->condition,strbuff);
    newlist->next = list;
    return newlist;
  }

  static void SMV_Cases(void)
  {
    SMV_PCs ctr;
    SMV_Variables var;
    SMV_PC_Cases ranges;
    SMV_PC_Cases pccase;
    SMV_Variable_Cases varcase;
    int i;

    fprintf(fh,"\nASSIGN\n");
    for(ctr=PCHead; ctr; ctr=ctr->next) {
      fprintf(fh,"  init(pc%d) := 0;\n",ctr->id);
      fprintf(fh,"  next(pc%d) := case\n",ctr->id);
      for(pccase=ctr->cases; pccase; pccase=pccase->next)
        fprintf(fh,"    %s : %s%s;\n",pccase->condition,pccase->result,
                pccase->result[0]=='{' ? "}" : "");
      fprintf(fh,"    1 : pc%d;\n",ctr->id);
      fprintf(fh,"  esac;\n");
    }

    ranges = NULL;
    for(var=Variables; var; var=var->next) {
      fprintf(fh,"  next(int%d) := case\n",var->varid);
      for(varcase=var->cases; varcase; varcase=varcase->next)
        if(varcase->what==from_constant)
          if(varcase->pcid2==-1)
            fprintf(fh,"    pc%d=%d : %d;\n",varcase->pcid,varcase->pcval,(int)varcase->data);
          else if(varcase->boolexpr==NULL)
            fprintf(fh,"    (pc%d=%d)&(pc%d=%d) : %d;\n",varcase->pcid,varcase->pcval,
              varcase->pcid2,varcase->pcval2,(int)varcase->data);
          else
            fprintf(fh,"    %s&(pc%d=%d)&(pc%d=%d) : %d;\n",varcase->boolexpr,varcase->pcid,varcase->pcval,
              varcase->pcid2,varcase->pcval2,(int)varcase->data);

        else {
          ranges = SMV_AddRangeCheck(ranges,varcase->pcid,varcase->pcval,
              varcase->pcid2,varcase->pcval2,varcase->boolexpr,(char*)varcase->data,var->min,var->max);
        }
      fprintf(fh,"    1 : int%d;\n",var->varid);
      fprintf(fh,"  esac;\n");
    }

    fprintf(fh,"  termination := pc0=%d;\n",PCHead->maximum);
    if(ranges) {
      fprintf(fh,"  rangeerror := ");
      for(pccase=ranges;ranges;ranges=pccase) {
        pccase=ranges->next;
        fprintf(fh,"(%s)%s",ranges->condition,pccase ? "\n              | " : ";\n");
        free(ranges->condition);
        free(ranges);
      }
    }
    else if(Variables)
      fprintf(fh,"  rangeerror := 0;\n");
  }

  /* This outputs some gubbins at the prog foot. */
  static void SMV_Footer(void)
  {
    int finished = PCHead->maximum;
    fprintf(fh,"\nSPEC\n");
    fprintf(fh,"-- Strong deadlock test\n");
    fprintf(fh,"AG AF termination");
    if(Variables) fprintf(fh," & !rangeerror");
    fprintf(fh,"\n");
  }

  /* This is the function which ParserDrv.c calls when it wants to output SMV. */
  /* If the filename is '-', then the SMV will be printed to stdout. */
  /* Otherwise a new file will be opened, and the output will be writen to it. */
  void OutputSMV(char* filename, tTree tree)
  {
    /*printf("Opening file \"%s\" for writing\n",filename);*/
    fh = (strcmp(filename,"-")==0) ? stdout : fopen(filename,"w");
    if(fh!=stdout) {
      int len = strlen(filename);
      filename[--len] = 't';
      filename[--len] = 'a';
      filename[--len] = 'd';
      datfh=fopen(filename,"w");
    }
    if (fh==NULL) MessageI("SMV.trafo: cannot open file ",xxFatal,NoPosition,xxString,filename);

    /*printf("Traversing tree\n");*/
    SMV_Tree(tree);
    /*printf("Traversing tree\n");*/

    SMV_Do_Channels();

    /*printf("Outputting file contents\n");*/
    SMV_Header();
    SMV_OutputVariables();
    SMV_Cases();
    SMV_Footer();

    fclose(fh);
  }

  

static void yyExit () { Exit (1); }

void (* SMVout_Exit) () = yyExit;

extern FILE *yyf;

static void yyAbort
# ifdef __cplusplus
 (char * yyFunction)
# else
 (yyFunction) char * yyFunction;
# endif
{
 (void) fprintf (stderr, "Error: module SMVout, routine %s failed\n", yyFunction);
 SMVout_Exit ();
}

static void SMV_Tree ARGS((tTree yyP1));
static void SMV_OuterSpecList ARGS((tTree yyP2));
static void SMV_OuterSpec ARGS((tTree yyP3));
static int SMV_Proc ARGS((tTree Me));
static int SMV_Seq ARGS((int yyP6, int yyP5, tTree yyP4));
static void SMV_Par1 ARGS((int yyP9, int yyP8, tTree yyP7));
static int SMV_Par2 ARGS((tTree yyP10));
static int SMV_If1 ARGS((int yyP13, int yyP12, tTree yyP11));
static void SMV_If2 ARGS((int yyP16, int yyP15, tTree yyP14));
static int SMV_Case1 ARGS((int yyP19, int yyP18, tTree yyP17));
static void SMV_Case2 ARGS((bool yyP23, int yyP22, int yyP21, tTree yyP20));
static int SMV_Alt1 ARGS((int yyP26, int yyP25, tTree yyP24));
static void SMV_Alt2 ARGS((int yyP28, int yyP27, tTree Me));
static bool SMV_Assign ARGS((int yyP32, int yyP31, tTree yyP30, tTree yyP29));
static bool SMV_EvaluateExpr ARGS((tTree yyP33));
static bool SMV_Expr_Evaluate ARGS((tTree yyP34));
static bool SMV_EvaluateBinary ARGS((tTree yyP35));
static bool SMV_EvaluateUnary ARGS((tTree yyP36));
static int SMV_Instance_SpecList ARGS((int yyP39, int yyP38, tTree yyP37));
static int SMV_Instance_Spec ARGS((int yyP42, int yyP41, tTree yyP40));
static void SMV_SpecList ARGS((tTree yyP43));
static void SMV_Spec ARGS((tTree yyP44));
static void SMV_Is ARGS((tSymbol yyP46, tTree yyP45));
static void SMV_Decl ARGS((tTree yyP48, tTree yyP47));
static void SMV_AllocateChanIDs ARGS((tTree yyP49));
static int SMV_GetSymbolID ARGS((tSymbol yyP50));
static bool SMV_Type_IsTimer ARGS((tTree yyP51));

static void SMV_Tree
# if defined __STDC__ | defined __cplusplus
(register tTree yyP1)
# else
(yyP1)
 register tTree yyP1;
# endif
{
  if (yyP1->Kind == kStart) {
  if (yyP1->Start.Process->Kind == kNullProcess) {
/* line 793 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 794 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_OuterSpecList (yyP1->Start.Process->NullProcess.SpecList);
  }
   return;

  }
  }
 yyAbort ("SMV_Tree");
;
}

static void SMV_OuterSpecList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP2)
# else
(yyP2)
 register tTree yyP2;
# endif
{
  if (yyP2->Kind == kASpec) {
/* line 799 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 800 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_OuterSpec (yyP2->ASpec.Spec);
/* line 801 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_OuterSpecList (yyP2->ASpec.Next);
  }
   return;

  }
  if (yyP2->Kind == kNoSpec) {
/* line 803 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  }
 yyAbort ("SMV_OuterSpecList");
;
}

static void SMV_OuterSpec
# if defined __STDC__ | defined __cplusplus
(register tTree yyP3)
# else
(yyP3)
 register tTree yyP3;
# endif
{
/* line 807 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
int pc; int val;

  switch (yyP3->Kind) {
  case kDefProc:
  case kInlineDefProc:
/* line 808 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 811 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pc = SMV_NewPC ();
/* line 812 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP3->DefProc.Process -> Process . SMV_pcid = pc;
/* line 813 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP3->DefProc.Process -> Process . SMV_pcval = 0;
/* line 814 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = SMV_Proc (yyP3->DefProc.Process);
/* line 815 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SetPCmax (pc, val);
  }
   return;

  case kPrototypeProc:
/* line 817 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 818 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring PrototypeProc node", xxRestriction, yyP3->PrototypeProc.Pos);
  }
   return;

  case kDefFunc:
  case kInlineDefFunc:
/* line 820 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 821 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring DefFunc node", xxRestriction, yyP3->DefFunc.Pos);
  }
   return;

  case kPrototypeFunc:
/* line 823 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 824 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring PrototypeFunc node", xxRestriction, yyP3->PrototypeFunc.Pos);
  }
   return;

  case kHCode:
/* line 826 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 827 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring HCode node", xxRestriction, yyP3->HCode.Pos);
  }
   return;

  case kAVal:
  case kVal:
  case kValRetypes:
  case kValReshapes:
/* line 829 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 830 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring AVal node", xxRestriction, yyP3->AVal.Pos);
  }
   return;

  case kDefSeqPro:
/* line 832 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 833 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring DefSeqPro node", xxRestriction, yyP3->DefSeqPro.Pos);
  }
   return;

  case kDefTagPro:
/* line 835 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 836 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring DefTagPro node", xxRestriction, yyP3->DefTagPro.Pos);
  }
   return;

  case kDeclType:
/* line 838 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 839 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring DeclType node", xxRestriction, yyP3->DeclType.Pos);
  }
   return;

  case kNullSpec:
/* line 841 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  }

 yyAbort ("SMV_OuterSpec");
;
}

static int SMV_Proc
# if defined __STDC__ | defined __cplusplus
(register tTree Me)
# else
(Me)
 register tTree Me;
# endif
{
/* line 851 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
int id,val,newval; bool t; char cond[30];

  switch (Me->Kind) {
  case kAssign:
/* line 852 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 854 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Assign.SpecList);
/* line 855 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 856 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = Me -> Process . SMV_pcid;
/* line 857 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = val;
/* line 858 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (SMV_Assign(id,val,Me->Assign.Lval,Me->Assign.Rval)) {
         newval++;
         sprintf(cond,"pc%d=%d",id,val,newval);
         SMV_AddDatPCEntry(Me->Assign.Pos,id,val);
         SMV_AddPCcase1(id,cond,newval);
       } }
  }
   return newval;

  case kAlt:
/* line 866 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 868 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Alt.SpecList);
/* line 869 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 870 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = Me -> Process . SMV_pcid;
/* line 871 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Alt.Pos, id, val);
/* line 872 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Alt1 (id, val + 1, Me->Alt.AltList) + 1;
/* line 873 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   NextResult = 0;
/* line 874 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Alt2 (val, newval, Me->Alt.AltList);
/* line 875 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ sprintf(strbuff,"pc%d in {%d",id,resbuff[0]);
       while(NextResult>1)
         sprintf(strbuff+strlen(strbuff),",%d",resbuff[--NextResult]);
       strcat(strbuff,"}");
       SMV_AddPCcase1(id,strbuff,newval);
    }
  }
   return newval;

  case kAltRep:
/* line 883 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 885 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->AltRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL3;
  {
/* line 886 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->AltRep.Replicator -> Replicator . For -> Expr . CV . Number < 0))) goto yyL3;
  {
/* line 887 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->AltRep.SpecList);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL3:;

/* line 890 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 892 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->AltRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL4;
  {
/* line 893 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->AltRep.Replicator -> Replicator . From -> Expr . CV . Number == 0))) goto yyL4;
  {
/* line 894 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->AltRep.SpecList);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL4:;

/* line 897 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 899 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: replicated ALT is not supported", xxRestriction, Me->AltRep.Pos);
/* line 900 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->AltRep.SpecList);
  }
   return Me -> Process . SMV_pcval;

  case kInput:
  if (Me->Input.ExprList->Kind == kAnExpr) {
  if (Me->Input.ExprList->AnExpr.Next->Kind == kNoExpr) {
/* line 903 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 905 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((SMV_Type_IsTimer (Me->Input.Element -> UnmappedIdentifier . SynType) == true))) goto yyL6;
  {
/* line 906 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: timer input is not supported", xxRestriction, Me->Input.Pos);
/* line 907 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Input.SpecList);
  }
  }
   return Me -> Process . SMV_pcval;
yyL6:;

/* line 910 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 912 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((SymbolKind (Me->Input.Element -> UnmappedIdentifier . Sym) -> AnId . Place != NoTree))) goto yyL7;
  {
/* line 913 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: port input is not supported", xxRestriction, Me->Input.Pos);
/* line 914 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Input.SpecList);
  }
  }
   return Me -> Process . SMV_pcval;
yyL7:;

  if (Me->Input.ExprList->AnExpr.Expr->Kind == kElementalExpr) {
  if (Me->Input.ExprList->AnExpr.Expr->ElementalExpr.Elemental->Kind == kUnmappedIdentifier) {
/* line 917 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 919 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Input.SpecList);
/* line 920 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Input.Pos, Me -> Process . SMV_pcid, Me -> Process . SMV_pcval);
/* line 922 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = SMV_GetSymbolID (Me->Input.ExprList->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym);
/* line 923 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_GetSymbolID (Me->Input.Element -> UnmappedIdentifier . Sym);
/* line 924 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 925 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{
      if(id>0)
        SMV_AddChanCase(newval,Me->Process.SMV_pcid,val,to_tracked_var,(void*)id);
      else
        SMV_AddChanCase(newval,Me->Process.SMV_pcid,val,to_unknown,0);
    }
  }
   return val + 1;

  }
  }
  }
  }
  break;
  case kOutput:
  if (Me->Output.ExprList->Kind == kAnExpr) {
  if (Me->Output.ExprList->AnExpr.Next->Kind == kNoExpr) {
/* line 933 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 935 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((SymbolKind (Me->Output.Element -> UnmappedIdentifier . Sym) -> AnId . Place != NoTree))) goto yyL9;
  {
/* line 936 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: port output is not supported", xxRestriction, Me->Output.Pos);
/* line 937 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Output.SpecList);
  }
  }
   return Me -> Process . SMV_pcval;
yyL9:;

/* line 940 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 941 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->Output.ExprList->AnExpr.Expr -> Expr . C == true))) goto yyL10;
  {
/* line 943 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Output.SpecList);
/* line 944 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Output.Pos, Me -> Process . SMV_pcid, Me -> Process . SMV_pcval);
/* line 946 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = SMV_GetSymbolID (Me->Output.Element -> UnmappedIdentifier . Sym);
/* line 947 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 948 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddChanCase (id, Me -> Process . SMV_pcid, val, from_constant, Me->Output.ExprList->AnExpr.Expr -> Expr . CV . Number);
  }
  }
   return val + 1;
yyL10:;

/* line 951 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 953 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Output.SpecList);
/* line 954 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Output.Pos, Me -> Process . SMV_pcid, Me -> Process . SMV_pcval);
/* line 956 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = SMV_GetSymbolID (Me->Output.Element -> UnmappedIdentifier . Sym);
/* line 957 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 958 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{
      if(!SMV_EvaluateExpr(Me->Output.ExprList->AnExpr.Expr))
        SMV_AddChanCase(id,Me->Process.SMV_pcid,val,from_unknown,0);
      else
        SMV_AddChanCase(id,Me->Process.SMV_pcid,val,from_tracked_expr,strbuff);
    }
  }
   return val + 1;

  }
  }
  break;
  case kCaseIn:
/* line 966 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 967 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring CaseIn node", xxRestriction, Me->CaseIn.Pos);
/* line 968 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->CaseIn.SpecList);
  }
   return Me -> Process . SMV_pcval;

  case kAfter:
/* line 971 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 972 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->After.SpecList);
/* line 973 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: timer input is not supported", xxRestriction, Me->After.Pos);
  }
   return Me -> Process . SMV_pcval;

  case kSeq:
/* line 976 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 977 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Seq.SpecList);
/* line 978 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = Me -> Process . SMV_pcid;
/* line 979 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Seq.Pos, id, Me -> Process . SMV_pcval);
/* line 980 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Seq (id, Me -> Process . SMV_pcval, Me->Seq.ProcList);
/* line 981 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   fprintf (datfh, "pc%d_%d:END SEQ [%d]\n", id, newval, Me->Seq.Pos . Line);
  }
   return newval;

  case kSeqRep:
/* line 984 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 986 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->SeqRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL15;
  {
/* line 987 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->SeqRep.Replicator -> Replicator . For -> Expr . CV . Number < 0))) goto yyL15;
  {
/* line 988 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->SeqRep.SpecList);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL15:;

/* line 991 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 993 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->SeqRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL16;
  {
/* line 994 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->SeqRep.Replicator -> Replicator . From -> Expr . CV . Number == 0))) goto yyL16;
  {
/* line 995 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->SeqRep.SpecList);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL16:;

/* line 998 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1000 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->SeqRep.SpecList);
/* line 1001 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: replicated SEQ is not supported", xxRestriction, Me->SeqRep.Pos);
  }
   return Me -> Process . SMV_pcval;

  case kPar:
/* line 1004 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1006 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = Me -> Process . SMV_pcid;
/* line 1007 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 1008 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Par.Pos, id, val);
/* line 1009 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Par.SpecList);
/* line 1010 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Par1 (id, val, Me->Par.ProcList);
/* line 1011 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ *strbuff = 0; }
/* line 1012 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! (SMV_Par2 (Me->Par.ProcList))) goto yyL18;
  {
/* line 1013 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ sprintf(cond,"pc%d=%d",id,val); }
/* line 1014 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddPCcase1 (id, cond, val + 1);
/* line 1015 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddPCcase1 (id, strbuff, val + 2);
  }
  }
   return val + 2;
yyL18:;

  break;
  case kParRep:
/* line 1018 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1020 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->ParRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL19;
  {
/* line 1021 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->ParRep.Replicator -> Replicator . For -> Expr . CV . Number < 0))) goto yyL19;
  {
/* line 1022 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->ParRep.SpecList);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL19:;

/* line 1025 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1027 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->ParRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL20;
  {
/* line 1028 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->ParRep.Replicator -> Replicator . From -> Expr . CV . Number == 0))) goto yyL20;
  {
/* line 1029 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->ParRep.SpecList);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL20:;

/* line 1032 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1035 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: replicated PAR is not supported", xxRestriction, Me->ParRep.Pos);
/* line 1036 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->ParRep.SpecList);
  }
   return Me -> Process . SMV_pcval;

  case kIf:
/* line 1039 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1041 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->If.SpecList);
/* line 1042 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = Me -> Process . SMV_pcid;
/* line 1043 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 1044 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->If.Pos, id, val);
/* line 1046 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_If1 (id, val, Me->If.ChoiceList) + 1;
/* line 1047 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   NextResult = 0;
/* line 1048 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_If2 (val, newval, Me->If.ChoiceList);
  }
   return newval;

  case kIfRep:
/* line 1051 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1053 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->IfRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL23;
  {
/* line 1054 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->IfRep.Replicator -> Replicator . For -> Expr . CV . Number < 0))) goto yyL23;
  {
/* line 1055 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->IfRep.SpecList);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL23:;

/* line 1058 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1060 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->IfRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL24;
  {
/* line 1061 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->IfRep.Replicator -> Replicator . From -> Expr . CV . Number == 0))) goto yyL24;
  {
/* line 1062 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->IfRep.SpecList);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL24:;

/* line 1065 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1067 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: replicated IF is not supported", xxRestriction, Me->IfRep.Pos);
/* line 1068 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->IfRep.SpecList);
  }
   return Me -> Process . SMV_pcval;

  case kCase:
/* line 1071 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1073 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Case.SpecList);
/* line 1074 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 1075 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = Me -> Process . SMV_pcid;
/* line 1076 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Case.Pos, id, val);
/* line 1078 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Case1 (id, val, Me->Case.ChoiceList) + 1;
/* line 1079 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   NextResult = 0;
/* line 1080 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_EvaluateExpr (Me->Case.Selector);
/* line 1081 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (t) {
         strbuff2=(char*)malloc(strlen(strbuff)+1);
         strcpy(strbuff2,strbuff);
       } }
/* line 1085 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Case2 (t, val, newval, Me->Case.ChoiceList);
/* line 1086 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ free(strbuff2); }
  }
   return newval;

  case kWhile:
/* line 1089 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1091 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->While.Expr -> Expr . C == true))) goto yyL27;
  {
/* line 1092 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->While.Expr -> Expr . CV . Number == true))) goto yyL27;
  {
/* line 1093 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->While.SpecList);
/* line 1094 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = Me -> Process . SMV_pcid;
/* line 1095 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 1096 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->While.Pos, id, val);
/* line 1098 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Me->While.Process -> Process . SMV_pcid = id;
/* line 1099 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Me->While.Process -> Process . SMV_pcval = val;
/* line 1100 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Proc (Me->While.Process);
/* line 1101 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ sprintf(cond,"pc%d=%d",id,newval); }
/* line 1102 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddPCcase1 (id, cond, val);
/* line 1103 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   fprintf (datfh, "pc%d_%d:END WHILE [%d]\n", id, newval, Me->While.Pos . Line);
  }
  }
  }
   return newval + 1;
yyL27:;

/* line 1106 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1108 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->While.Expr -> Expr . C == true))) goto yyL28;
  {
/* line 1109 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((Me->While.Expr -> Expr . CV . Number == false))) goto yyL28;
  {
/* line 1110 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->While.SpecList);
/* line 1111 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->While.Pos, Me -> Process . SMV_pcid, Me -> Process . SMV_pcval);
  }
  }
  }
   return Me -> Process . SMV_pcval;
yyL28:;

/* line 1114 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1116 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->While.SpecList);
/* line 1117 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = Me -> Process . SMV_pcid;
/* line 1118 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = Me -> Process . SMV_pcval;
/* line 1119 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->While.Pos, id, val);
/* line 1121 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Me->While.Process -> Process . SMV_pcid = id;
/* line 1122 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Me->While.Process -> Process . SMV_pcval = val + 1;
/* line 1123 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Proc (Me->While.Process);
/* line 1125 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_EvaluateExpr (Me->While.Expr);
/* line 1126 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (t) {
         int i,len;
         sprintf(cond,"(pc%d=%d)",id,val);
         len = strlen(cond)+1;
         for(i=strlen(strbuff);i>=0;i--)
           strbuff[i+len]=strbuff[i];
         strcpy(strbuff,cond);
         SMV_AddPCcase1(id,strbuff,newval+1);
         strbuff[len-1]='&';
         SMV_AddPCcase1(id,strbuff,val+1);
         sprintf(cond,"pc%d=%d",id,newval);
         SMV_AddPCcase1(id,cond,val);
       }
       else {
         sprintf(cond,"pc%d=%d",id,newval);
         SMV_AddPCcase2(id,cond,val,newval+1);
       } }
/* line 1143 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   fprintf (datfh, "pc%d_%d:(END WHILE [%d])\n", id, newval, Me->While.Pos . Line);
  }
   return newval + 1;

  case kSkip:
/* line 1147 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1149 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Skip.Pos, Me -> Process . SMV_pcid, Me -> Process . SMV_pcval);
/* line 1150 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Skip.SpecList);
  }
   return Me -> Process . SMV_pcval;

  case kStop:
/* line 1153 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1155 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (Me->Stop.Pos, Me -> Process . SMV_pcid, Me -> Process . SMV_pcval);
/* line 1156 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->Stop.SpecList);
  }
   return Me -> Process . SMV_pcval + 1;

  case kCCode:
/* line 1159 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1160 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: ignoring CCode node.", xxRestriction, Me->CCode.Pos);
/* line 1161 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->CCode.SpecList);
  }
   return Me -> Process . SMV_pcval;

  case kInstance:
/* line 1164 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return SMV_Instance_SpecList (Me -> Process . SMV_pcid, Me -> Process . SMV_pcval, Me->Instance.SpecList);

  case kNullProcess:
/* line 1168 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1169 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (Me->NullProcess.SpecList);
  }
   return Me -> Process . SMV_pcval;

  }

 yyAbort ("SMV_Proc");
}

static int SMV_Seq
# if defined __STDC__ | defined __cplusplus
(register int yyP6, register int yyP5, register tTree yyP4)
# else
(yyP6, yyP5, yyP4)
 register int yyP6;
 register int yyP5;
 register tTree yyP4;
# endif
{
/* line 1176 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
int val;
  if (yyP4->Kind == kAProcess) {
/* line 1177 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1178 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP4->AProcess.Process -> Process . SMV_pcid = yyP6;
/* line 1179 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP4->AProcess.Process -> Process . SMV_pcval = yyP5;
/* line 1180 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = SMV_Proc (yyP4->AProcess.Process);
  }
   return SMV_Seq (yyP6, val, yyP4->AProcess.Next);

  }
  if (yyP4->Kind == kNoProcess) {
/* line 1183 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return yyP5;

  }
 yyAbort ("SMV_Seq");
}

static void SMV_Par1
# if defined __STDC__ | defined __cplusplus
(register int yyP9, register int yyP8, register tTree yyP7)
# else
(yyP9, yyP8, yyP7)
 register int yyP9;
 register int yyP8;
 register tTree yyP7;
# endif
{
/* line 1188 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
int newpc;int val;
  if (yyP7->Kind == kAProcess) {
/* line 1189 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1190 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newpc = SMV_NewPC ();
/* line 1191 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP7->AProcess.Process -> Process . SMV_pcid = newpc;
/* line 1192 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP7->AProcess.Process -> Process . SMV_pcval = 1;
/* line 1193 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_StartPC (yyP9, yyP8, newpc);
/* line 1194 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = SMV_Proc (yyP7->AProcess.Process);
/* line 1195 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SetPCmax (newpc, val);
/* line 1196 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP7->AProcess.Process -> Process . SMV_endval = val;
/* line 1197 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Par1 (yyP9, yyP8, yyP7->AProcess.Next);
  }
   return;

  }
  if (yyP7->Kind == kNoProcess) {
/* line 1199 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  }
 yyAbort ("SMV_Par1");
;
}

static int SMV_Par2
# if defined __STDC__ | defined __cplusplus
(register tTree yyP10)
# else
(yyP10)
 register tTree yyP10;
# endif
{
/* line 1203 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
int pc;int val;int rest;
  if (yyP10->Kind == kAProcess) {
/* line 1204 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1205 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pc = yyP10->AProcess.Process -> Process . SMV_pcid;
/* line 1206 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = yyP10->AProcess.Process -> Process . SMV_endval;
/* line 1207 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   rest = SMV_Par2 (yyP10->AProcess.Next);
/* line 1208 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if(rest) strcat(strbuff,"&");
       sprintf(strbuff+strlen(strbuff),"(pc%d=%d)",pc,val);
    }
  }
   return rest + 1;

  }
  if (yyP10->Kind == kNoProcess) {
/* line 1213 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return 0;

  }
 yyAbort ("SMV_Par2");
}

static int SMV_If1
# if defined __STDC__ | defined __cplusplus
(register int yyP13, register int yyP12, register tTree yyP11)
# else
(yyP13, yyP12, yyP11)
 register int yyP13;
 register int yyP12;
 register tTree yyP11;
# endif
{
/* line 1219 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
 int newval; 
  if (yyP11->Kind == kAChoice) {
  if (yyP11->AChoice.ExprList->Kind == kAnExpr) {
/* line 1220 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1221 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (yyP11->AChoice.SpecList);
/* line 1222 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP11->AChoice.Process -> Process . SMV_pcid = yyP13;
/* line 1223 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP11->AChoice.Process -> Process . SMV_pcval = yyP12 + 1;
/* line 1224 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Proc (yyP11->AChoice.Process);
/* line 1225 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP11->AChoice.Process -> Process . SMV_endval = newval;
  }
   return SMV_If1 (yyP13, newval, yyP11->AChoice.Next);

  }
  }
  if (yyP11->Kind == kANewIf) {
/* line 1228 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1229 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP11->ANewIf.IfType -> Process . SMV_pcid = yyP13;
/* line 1230 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP11->ANewIf.IfType -> Process . SMV_pcval = yyP12;
/* line 1231 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Proc (yyP11->ANewIf.IfType);
/* line 1232 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP11->ANewIf.IfType -> Process . SMV_endval = newval;
  }
   return SMV_If1 (yyP13, newval, yyP11->ANewIf.Next);

  }
  if (yyP11->Kind == kNoChoice) {
/* line 1235 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return yyP12;

  }
 yyAbort ("SMV_If1");
}

static void SMV_If2
# if defined __STDC__ | defined __cplusplus
(register int yyP16, register int yyP15, register tTree yyP14)
# else
(yyP16, yyP15, yyP14)
 register int yyP16;
 register int yyP15;
 register tTree yyP14;
# endif
{
/* line 1240 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
 char cond[10]; bool t; int pcid; int pcval2; 
  if (yyP14->Kind == kAChoice) {
  if (yyP14->AChoice.ExprList->Kind == kAnExpr) {
/* line 1241 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1242 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcid = yyP14->AChoice.Process -> Process . SMV_pcid;
/* line 1243 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcval2 = yyP14->AChoice.Process -> Process . SMV_pcval;
/* line 1244 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ sprintf(cond,"pc%d=%d",pcid,yyP14->AChoice.Process->Process.SMV_endval); }
/* line 1245 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddPCcase1 (yyP14->AChoice.Process -> Process . SMV_pcid, cond, yyP15);
/* line 1246 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_EvaluateExpr (yyP14->AChoice.ExprList->AnExpr.Expr);
/* line 1247 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (t)
         SMV_AddIfCase(pcid,yyP16,pcval2);
       else
         resbuff[NextResult++] = pcval2;
    }
/* line 1252 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_If2 (yyP16, yyP15, yyP14->AChoice.Next);
  }
   return;

  }
  }
  if (yyP14->Kind == kANewIf) {
/* line 1254 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1255 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcid = yyP14->ANewIf.IfType -> Process . SMV_pcid;
/* line 1256 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ sprintf(cond,"pc%d=%d",pcid,yyP14->ANewIf.IfType->Process.SMV_endval); }
/* line 1257 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddPCcase1 (pcid, cond, yyP15);
/* line 1258 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_If2 (yyP16, yyP15, yyP14->ANewIf.Next);
  }
   return;

  }
  if (yyP14->Kind == kNoChoice) {
/* line 1260 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  }
 yyAbort ("SMV_If2");
;
}

static int SMV_Case1
# if defined __STDC__ | defined __cplusplus
(register int yyP19, register int yyP18, register tTree yyP17)
# else
(yyP19, yyP18, yyP17)
 register int yyP19;
 register int yyP18;
 register tTree yyP17;
# endif
{
/* line 1265 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
 int newval; 
  if (yyP17->Kind == kAChoice) {
  if (yyP17->AChoice.ExprList->Kind == kAnExpr) {
/* line 1266 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1267 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (yyP17->AChoice.SpecList);
/* line 1268 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP17->AChoice.Process -> Process . SMV_pcid = yyP19;
/* line 1269 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP17->AChoice.Process -> Process . SMV_pcval = yyP18 + 1;
/* line 1270 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Proc (yyP17->AChoice.Process);
/* line 1271 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP17->AChoice.Process -> Process . SMV_endval = newval;
  }
   return SMV_Case1 (yyP19, newval, yyP17->AChoice.Next);

  }
  }
  if (yyP17->Kind == kANewIf) {
/* line 1274 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1275 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: indented CASE not supported.", xxRestriction, NoPosition);
  }
   return SMV_Case1 (yyP19, yyP18, yyP17->ANewIf.Next);

  }
  if (yyP17->Kind == kNoChoice) {
/* line 1278 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return yyP18;

  }
 yyAbort ("SMV_Case1");
}

static void SMV_Case2
# if defined __STDC__ | defined __cplusplus
(register bool yyP23, register int yyP22, register int yyP21, register tTree yyP20)
# else
(yyP23, yyP22, yyP21, yyP20)
 register bool yyP23;
 register int yyP22;
 register int yyP21;
 register tTree yyP20;
# endif
{
/* line 1283 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
 char cond[10]; bool t; int pcid; int pcval2; 
  if (yyP20->Kind == kAChoice) {
  if (yyP20->AChoice.ExprList->Kind == kAnExpr) {
/* line 1284 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1285 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcid = yyP20->AChoice.Process -> Process . SMV_pcid;
/* line 1286 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcval2 = yyP20->AChoice.Process -> Process . SMV_pcval;
/* line 1287 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ sprintf(cond,"pc%d=%d",pcid,yyP20->AChoice.Process->Process.SMV_endval); }
/* line 1288 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddPCcase1 (yyP20->AChoice.Process -> Process . SMV_pcid, cond, yyP21);
/* line 1289 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_EvaluateExpr (yyP20->AChoice.ExprList->AnExpr.Expr);
/* line 1290 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (yyP23 && t)
         SMV_AddCaseCase(pcid,yyP22,pcval2);
       else
         resbuff[NextResult++] = pcval2;
    }
/* line 1295 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Case2 (yyP23, yyP22, yyP21, yyP20->AChoice.Next);
  }
   return;

  }
  }
  if (yyP20->Kind == kANewIf) {
/* line 1297 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1298 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Case2 (yyP23, yyP22, yyP21, yyP20->ANewIf.Next);
  }
   return;

  }
  if (yyP20->Kind == kNoChoice) {
/* line 1300 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  }
 yyAbort ("SMV_Case2");
;
}

static int SMV_Alt1
# if defined __STDC__ | defined __cplusplus
(register int yyP26, register int yyP25, register tTree yyP24)
# else
(yyP26, yyP25, yyP24)
 register int yyP26;
 register int yyP25;
 register tTree yyP24;
# endif
{
/* line 1305 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
 int newval; 
  if (yyP24->Kind == kAnAlt) {
  if (yyP24->AnAlt.Guard->Kind == kAltInput) {
  if (yyP24->AnAlt.Guard->AltInput.Input->Input.ExprList->Kind == kAnExpr) {
  if (yyP24->AnAlt.Guard->AltInput.Input->Input.ExprList->AnExpr.Next->Kind == kNoExpr) {
/* line 1306 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1307 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (yyP24->AnAlt.SpecList);
/* line 1308 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP24->AnAlt.Guard->AltInput.Process -> Process . SMV_pcid = yyP26;
/* line 1309 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP24->AnAlt.Guard->AltInput.Process -> Process . SMV_pcval = yyP25 + 1;
/* line 1310 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (yyP24->AnAlt.Guard->AltInput.Input->Input.Pos, yyP26, yyP25 + 1);
/* line 1311 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Proc (yyP24->AnAlt.Guard->AltInput.Process);
/* line 1312 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP24->AnAlt.Guard->AltInput.Process -> Process . SMV_endval = newval;
  }
   return SMV_Alt1 (yyP26, newval, yyP24->AnAlt.Next);

  }
  }
  }
  if (yyP24->AnAlt.Guard->Kind == kNoInput) {
/* line 1315 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1316 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (yyP24->AnAlt.SpecList);
/* line 1317 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP24->AnAlt.Guard->NoInput.Process -> Process . SMV_pcid = yyP26;
/* line 1318 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP24->AnAlt.Guard->NoInput.Process -> Process . SMV_pcval = yyP25 + 1;
/* line 1319 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AddDatPCEntry (yyP24->AnAlt.Guard->NoInput.Pos, yyP26, yyP25 + 1);
/* line 1320 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Proc (yyP24->AnAlt.Guard->NoInput.Process);
/* line 1321 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP24->AnAlt.Guard->NoInput.Process -> Process . SMV_endval = newval;
  }
   return SMV_Alt1 (yyP26, newval, yyP24->AnAlt.Next);

  }
  }
  if (yyP24->Kind == kANewAlt) {
/* line 1324 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1325 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: indented ALT not supported.", xxRestriction, NoPosition);
  }
   return yyP25;

  }
  if (yyP24->Kind == kNoAlt) {
/* line 1328 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return yyP25;

  }
 yyAbort ("SMV_Alt1");
}

static void SMV_Alt2
# if defined __STDC__ | defined __cplusplus
(register int yyP28, register int yyP27, register tTree Me)
# else
(yyP28, yyP27, Me)
 register int yyP28;
 register int yyP27;
 register tTree Me;
# endif
{
/* line 1333 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
 char cond[10]; bool t; int pcid,pcval2,id,newval,val;
  if (Me->Kind == kAnAlt) {
  if (Me->AnAlt.Guard->Kind == kAltInput) {
  if (Me->AnAlt.Guard->AltInput.Input->Input.ExprList->Kind == kAnExpr) {
  if (Me->AnAlt.Guard->AltInput.Input->Input.ExprList->AnExpr.Next->Kind == kNoExpr) {
  if (Me->AnAlt.Guard->AltInput.Input->Input.ExprList->AnExpr.Expr->Kind == kElementalExpr) {
  if (Me->AnAlt.Guard->AltInput.Input->Input.ExprList->AnExpr.Expr->ElementalExpr.Elemental->Kind == kUnmappedIdentifier) {
/* line 1334 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1336 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcid = Me->AnAlt.Guard->AltInput.Process -> Process . SMV_pcid;
/* line 1337 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcval2 = Me->AnAlt.Guard->AltInput.Process -> Process . SMV_pcval;
/* line 1338 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ resbuff[NextResult++] = Me->AnAlt.Guard->AltInput.Process->Process.SMV_endval; }
/* line 1340 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = SMV_GetSymbolID (Me->AnAlt.Guard->AltInput.Input->Input.ExprList->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym);
/* line 1341 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_GetSymbolID (Me->AnAlt.Guard->AltInput.Input->Input.Element -> UnmappedIdentifier . Sym);
/* line 1342 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = yyP28;
/* line 1343 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_EvaluateExpr (Me->AnAlt.Guard->AltInput.Expr);
/* line 1344 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (t)
         SMV_AddAltCase(newval,Me->AnAlt.Guard->AltInput.Process->Process.SMV_pcid,val,strbuff,id,pcval2);
       else
         SMV_AddAltCase(newval,Me->AnAlt.Guard->AltInput.Process->Process.SMV_pcid,val,NULL,id,pcval2);
    }
/* line 1350 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Alt2 (yyP28, yyP27, Me->AnAlt.Next);
  }
   return;

  }
  }
  }
  }
  }
  if (Me->AnAlt.Guard->Kind == kNoInput) {
/* line 1352 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1354 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcid = Me->AnAlt.Guard->NoInput.Process -> Process . SMV_pcid;
/* line 1355 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   pcval2 = Me->AnAlt.Guard->NoInput.Process -> Process . SMV_pcval;
/* line 1356 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ resbuff[NextResult++] = Me->AnAlt.Guard->NoInput.Process->Process.SMV_endval; }
/* line 1358 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   val = yyP28;
/* line 1359 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_EvaluateExpr (Me->AnAlt.Guard->NoInput.Expr);
/* line 1361 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (t)
         SMV_AddAltCase(0,Me->AnAlt.Guard->NoInput.Process->Process.SMV_pcid,val,strbuff,0,pcval2);
       else
         SMV_AddAltCase(0,Me->AnAlt.Guard->NoInput.Process->Process.SMV_pcid,val,NULL,0,pcval2);
    }
/* line 1367 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Alt2 (yyP28, yyP27, Me->AnAlt.Next);
  }
   return;

  }
  }
  if (Me->Kind == kANewAlt) {
/* line 1369 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1370 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Alt2 (yyP28, yyP27, Me->ANewAlt.Next);
  }
   return;

  }
  if (Me->Kind == kNoAlt) {
/* line 1372 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  }
 yyAbort ("SMV_Alt2");
;
}

static bool SMV_Assign
# if defined __STDC__ | defined __cplusplus
(register int yyP32, register int yyP31, register tTree yyP30, register tTree yyP29)
# else
(yyP32, yyP31, yyP30, yyP29)
 register int yyP32;
 register int yyP31;
 register tTree yyP30;
 register tTree yyP29;
# endif
{
/* line 1377 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
 int id; bool t; char cond[40]; 
  if (yyP29->Kind == kAnExpr) {
  if (yyP29->AnExpr.Expr->Kind == kValof) {
/* line 1378 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1380 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: VALOF is not supported.", xxRestriction, yyP29->AnExpr.Expr->Valof.Pos);
  }
   return SMV_Assign (yyP32, yyP31, yyP30, yyP29->AnExpr.Expr->Valof.ExprList);

  }
  }
  if (yyP30->Kind == kAnExpr) {
  if (yyP30->AnExpr.Expr->Kind == kElementalExpr) {
  if (yyP30->AnExpr.Expr->ElementalExpr.Elemental->Kind == kUnmappedIdentifier) {
  if (yyP29->Kind == kAnExpr) {
/* line 1383 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1385 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((yyP29->AnExpr.Expr -> Expr . C == true))) goto yyL2;
  {
/* line 1386 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_Assign (yyP32, yyP31, yyP30->AnExpr.Next, yyP29->AnExpr.Next);
/* line 1387 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = SMV_GetSymbolID (yyP30->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym);
/* line 1389 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (id>0) { 
          sprintf(cond,"pc%d=%d",yyP32,yyP31);
          SMV_AddIntCase(id,yyP32,yyP31,from_constant,yyP29->AnExpr.Expr->Expr.CV.Number);
          t=true;
       } }
  }
  }
   return t;
yyL2:;

/* line 1396 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1397 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_Assign (yyP32, yyP31, yyP30->AnExpr.Next, yyP29->AnExpr.Next);
/* line 1398 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = SMV_GetSymbolID (yyP30->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym);
/* line 1400 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (id>0) {
          if(!SMV_EvaluateExpr(yyP29->AnExpr.Expr))
            SMV_AddIntCase(id,yyP32,yyP31,from_unknown,0);
          else
            SMV_AddIntCase(id,yyP32,yyP31,from_tracked_expr,strbuff);
          t=true;
       } }
  }
   return t;

  }
  }
  }
  }
  if (yyP30->Kind == kNoExpr) {
  if (yyP29->Kind == kNoExpr) {
/* line 1410 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return false;

  }
  }
 yyAbort ("SMV_Assign");
}

static bool SMV_EvaluateExpr
# if defined __STDC__ | defined __cplusplus
(register tTree yyP33)
# else
(yyP33)
 register tTree yyP33;
# endif
{
/* line 1421 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
bool t;
/* line 1422 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1423 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ *strbuff=0; }
/* line 1424 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t = SMV_Expr_Evaluate (yyP33);
/* line 1425 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (!t) strcpy(strbuff,"undefined"); }
  }
   return t;

}

static bool SMV_Expr_Evaluate
# if defined __STDC__ | defined __cplusplus
(register tTree yyP34)
# else
(yyP34)
 register tTree yyP34;
# endif
{
/* line 1432 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
bool t1,t2,t3; int id; char cond[20];
/* line 1433 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1435 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((yyP34 -> Expr . C == true))) goto yyL1;
  {
/* line 1436 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ sprintf(cond,"%d",yyP34->Expr.CV.Number);
       strcat(strbuff,cond); }
  }
  }
   return true;
yyL1:;


  switch (yyP34->Kind) {
  case kBinary:
/* line 1440 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1441 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"("); }
/* line 1442 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t1 = SMV_Expr_Evaluate (yyP34->Binary.Lop);
/* line 1443 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t2 = SMV_EvaluateBinary (yyP34->Binary.Op);
/* line 1444 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t3 = SMV_Expr_Evaluate (yyP34->Binary.Rop);
/* line 1445 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,")"); }
  }
   return (t1 && t2 && t3);

  case kUnary:
/* line 1448 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1449 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"("); }
/* line 1450 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t1 = SMV_EvaluateUnary (yyP34->Unary.Op);
/* line 1451 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   t2 = SMV_Expr_Evaluate (yyP34->Unary.Expr);
/* line 1452 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,")"); }
  }
   return (t1 && t2);

  case kValof:
  if (yyP34->Valof.ExprList->Kind == kAnExpr) {
  if (yyP34->Valof.ExprList->AnExpr.Next->Kind == kNoExpr) {
/* line 1455 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return false;

  }
  }
  break;
  case kFuncCall:
/* line 1458 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1459 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: function calls are not supported", xxError, yyP34->FuncCall.Pos);
  }
   return false;

  case kConvert:
/* line 1462 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return false;

  case kRound:
/* line 1465 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1467 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: the ROUND operator is not supported", xxError, yyP34->Round.Pos);
  }
   return false;

  case kTrunc:
/* line 1470 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1472 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: the TRUNC operator is not supported", xxError, yyP34->Trunc.Pos);
  }
   return false;

  case kCountArray:
/* line 1475 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1476 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: the array count operation is not supported", xxError, yyP34->CountArray.Pos);
  }
   return false;

  case kElementalExpr:
  if (yyP34->ElementalExpr.Elemental->Kind == kUnmappedIdentifier) {
/* line 1479 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1480 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   id = SMV_GetSymbolID (yyP34->ElementalExpr.Elemental->UnmappedIdentifier.Sym);
/* line 1482 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ if (id>0) {
         t1=true;
         sprintf(cond,"int%d",id);
         strcat(strbuff,cond);
       } 
       else t1=false; }
  }
   return t1;

  }
/* line 1490 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1491 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with ElementalExpr node", xxError, yyP34->ElementalExpr.Pos);
  }
   return false;

  case kUnBound:
/* line 1494 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1495 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with UnBound node", xxError, yyP34->UnBound.Pos);
  }
   return false;

  case kConst:
/* line 1498 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1499 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: constant node contains non-constant value", xxError, yyP34->Const.Pos);
  }
   return false;

  case kString:
/* line 1502 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1503 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with String node", xxError, yyP34->String.Pos);
  }
   return false;

  case kDefault:
/* line 1506 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1507 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   strcat (strbuff, "1");
  }
   return true;

  }

 yyAbort ("SMV_Expr_Evaluate");
}

static bool SMV_EvaluateBinary
# if defined __STDC__ | defined __cplusplus
(register tTree yyP35)
# else
(yyP35)
 register tTree yyP35;
# endif
{

  switch (yyP35->Kind) {
  case kLess:
/* line 1514 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1515 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"<"); }
  }
   return true;

  case kGreater:
/* line 1518 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1519 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,">"); }
  }
   return true;

  case kLessEqual:
/* line 1522 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1523 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"<="); }
  }
   return true;

  case kGreaterEqual:
/* line 1526 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1527 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,">="); }
  }
   return true;

  case kEqual:
/* line 1530 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1531 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"="); }
  }
   return true;

  case kNotEqual:
/* line 1534 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1535 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"~="); }
  }
   return true;

  case kAdd:
/* line 1538 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1539 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"+"); }
  }
   return true;

  case kSubtract:
/* line 1542 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1543 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"-"); }
  }
   return true;

  case kMultiply:
/* line 1546 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1547 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"*"); }
  }
   return true;

  case kDivide:
/* line 1550 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1552 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"/"); }
  }
   return true;

  case kRem:
/* line 1555 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1558 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: the remainder operator is not supported", xxRestriction, NoPosition);
/* line 1559 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff," mod "); }
  }
   return true;

  case kBitAnd:
/* line 1562 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1563 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"&"); }
  }
   return true;

  case kBitOr:
/* line 1566 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1567 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"|"); }
  }
   return true;

  case kBitEor:
/* line 1570 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1571 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"^"); }
  }
   return true;

  case kLeftShift:
/* line 1574 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1575 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"<<"); }
  }
   return true;

  case kRightShift:
/* line 1578 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1579 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,">>"); }
  }
   return true;

  case kSize:
/* line 1582 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1583 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("not sure what to do with the Size node", xxError, NoPosition);
  }
   return false;

  case kPlus:
/* line 1586 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return false;

  case kMinus:
/* line 1591 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return false;

  case kTimes:
/* line 1596 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return false;

  case kAfterOp:
/* line 1601 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return false;

  case kAnd:
/* line 1604 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1607 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"&"); }
  }
   return true;

  case kOr:
/* line 1610 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1613 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"|"); }
  }
   return true;

  }

/* line 1616 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1617 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: unknown operator detected", xxError, NoPosition);
  }
   return false;

}

static bool SMV_EvaluateUnary
# if defined __STDC__ | defined __cplusplus
(register tTree yyP36)
# else
(yyP36)
 register tTree yyP36;
# endif
{
  if (yyP36->Kind == kBitNot) {
/* line 1624 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1626 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"~"); }
  }
   return true;

  }
  if (yyP36->Kind == kNot) {
/* line 1629 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1631 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"~"); }
  }
   return true;

  }
  if (yyP36->Kind == kSubtract) {
/* line 1634 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1636 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
{ strcat(strbuff,"-"); }
  }
   return true;

  }
 yyAbort ("SMV_EvaluateUnary");
}

static int SMV_Instance_SpecList
# if defined __STDC__ | defined __cplusplus
(register int yyP39, register int yyP38, register tTree yyP37)
# else
(yyP39, yyP38, yyP37)
 register int yyP39;
 register int yyP38;
 register tTree yyP37;
# endif
{
/* line 1645 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
int newval;
  if (yyP37->Kind == kASpec) {
/* line 1646 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1647 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   newval = SMV_Instance_Spec (yyP39, yyP38, yyP37->ASpec.Spec);
/* line 1648 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! (SMV_Instance_SpecList (yyP39, yyP38, yyP37->ASpec.Next))) goto yyL1;
  }
   return newval;
yyL1:;

  }
  if (yyP37->Kind == kNoSpec) {
/* line 1651 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return yyP38;

  }
 yyAbort ("SMV_Instance_SpecList");
}

static int SMV_Instance_Spec
# if defined __STDC__ | defined __cplusplus
(register int yyP42, register int yyP41, register tTree yyP40)
# else
(yyP42, yyP41, yyP40)
 register int yyP42;
 register int yyP41;
 register tTree yyP40;
# endif
{
  if (Tree_IsType (yyP40, kDefProc)) {
/* line 1656 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1657 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP40->DefProc.Process -> Process . SMV_pcid = yyP42;
/* line 1658 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   yyP40->DefProc.Process -> Process . SMV_pcval = yyP41;
  }
   return SMV_Proc (yyP40->DefProc.Process);

  }
  if (yyP40->Kind == kPragmaTrack) {
/* line 1661 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1663 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! (SymbolKind (yyP40->PragmaTrack.Sym) -> AnId . SMV_idnum = SMV_NewInt (yyP40->PragmaTrack.Min, yyP40->PragmaTrack.Max))) goto yyL2;
  }
   return yyP41;
yyL2:;

  }
/* line 1666 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return yyP41;

}

static void SMV_SpecList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP43)
# else
(yyP43)
 register tTree yyP43;
# endif
{
  if (yyP43->Kind == kASpec) {
/* line 1671 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1672 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Spec (yyP43->ASpec.Spec);
/* line 1673 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_SpecList (yyP43->ASpec.Next);
  }
   return;

  }
  if (yyP43->Kind == kNoSpec) {
/* line 1675 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  }
 yyAbort ("SMV_SpecList");
;
}

static void SMV_Spec
# if defined __STDC__ | defined __cplusplus
(register tTree yyP44)
# else
(yyP44)
 register tTree yyP44;
# endif
{

  switch (yyP44->Kind) {
  case kIs:
/* line 1679 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1680 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Is (yyP44->Is.Sym, yyP44->Is.Elemental);
  }
   return;

  case kRetypes:
  case kReshapes:
/* line 1682 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  case kVal:
/* line 1684 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  case kValRetypes:
  case kValReshapes:
/* line 1686 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  case kDefIsFunc:
  case kInlineDefIsFunc:
/* line 1688 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1689 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with DefIsFunc node", xxError, yyP44->DefIsFunc.Pos);
  }
   return;

  case kDefFunc:
  case kInlineDefFunc:
/* line 1691 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1692 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with DefFunc node", xxError, yyP44->DefFunc.Pos);
  }
   return;

  case kPrototypeProc:
/* line 1694 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1695 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with PrototypeProc node", xxError, yyP44->PrototypeProc.Pos);
  }
   return;

  case kPrototypeFunc:
/* line 1697 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1698 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with PrototypeFunc node", xxError, yyP44->PrototypeFunc.Pos);
  }
   return;

  case kDeclType:
/* line 1700 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1701 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with DeclType node", xxError, yyP44->DeclType.Pos);
  }
   return;

  case kDecl:
/* line 1703 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1704 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_Decl (yyP44->Decl.Type, yyP44->Decl.IdList);
  }
   return;

  case kDefSeqPro:
/* line 1706 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1707 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with DefSeqPro node", xxError, yyP44->DefSeqPro.Pos);
  }
   return;

  case kDefTagPro:
/* line 1709 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1710 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with DefTagPro node", xxError, yyP44->DefTagPro.Pos);
  }
   return;

  case kHCode:
/* line 1712 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1713 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: not sure what to do with HCode node", xxError, yyP44->HCode.Pos);
  }
   return;

  case kPlace:
/* line 1715 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1717 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((SymbolKind (yyP44->Place.Sym) -> AnId . Place != NULL))) goto yyL14;
  {
/* line 1718 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: identifier already has a PLACE", xxError, yyP44->Place.Pos);
  }
  }
   return;
yyL14:;

/* line 1720 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1721 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! ((yyP44->Place.Expr -> Expr . C == false))) goto yyL15;
  {
/* line 1722 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   Message ("SMV.trafo: PLACE expression is not constant", xxError, yyP44->Place.Pos);
  }
  }
   return;
yyL15:;

/* line 1724 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  case kPragmaTrack:
/* line 1726 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1728 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   if (! (SymbolKind (yyP44->PragmaTrack.Sym) -> AnId . SMV_idnum = SMV_NewInt (yyP44->PragmaTrack.Min, yyP44->PragmaTrack.Max))) goto yyL17;
  {
/* line 1729 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   fprintf (datfh, "int%d:", SymbolKind (yyP44->PragmaTrack.Sym) -> AnId . SMV_idnum);
/* line 1730 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   WriteIdent (datfh, Symbol (yyP44->PragmaTrack.Sym) . Id);
/* line 1731 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   fprintf (datfh, "\n", SymbolKind (yyP44->PragmaTrack.Sym) -> AnId . SMV_idnum);
  }
  }
   return;
yyL17:;

  break;
  case kNullSpec:
/* line 1733 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

  }

/* line 1735 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

;
}

static void SMV_Is
# if defined __STDC__ | defined __cplusplus
(tSymbol yyP46, register tTree yyP45)
# else
(yyP46, yyP45)
 tSymbol yyP46;
 register tTree yyP45;
# endif
{
  if (yyP45->Kind == kUnmappedIdentifier) {
/* line 1739 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1740 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SymbolKind (yyP46) -> AnId . SMV_idnum = SymbolKind (yyP45->UnmappedIdentifier.Sym) -> AnId . SMV_idnum;
  }
   return;

  }
/* line 1742 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

;
}

static void SMV_Decl
# if defined __STDC__ | defined __cplusplus
(register tTree yyP48, register tTree yyP47)
# else
(yyP48, yyP47)
 register tTree yyP48;
 register tTree yyP47;
# endif
{
  if (yyP48->Kind == kChan) {
/* line 1746 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1747 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AllocateChanIDs (yyP47);
  }
   return;

  }
/* line 1749 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

;
}

static void SMV_AllocateChanIDs
# if defined __STDC__ | defined __cplusplus
(register tTree yyP49)
# else
(yyP49)
 register tTree yyP49;
# endif
{
  if (yyP49->Kind == kAnId) {
/* line 1753 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
  {
/* line 1754 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SymbolKind (yyP49->AnId.Sym) -> AnId . SMV_idnum = SMV_NewChan ();
/* line 1755 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   SMV_AllocateChanIDs (yyP49->AnId.Next);
  }
   return;

  }
/* line 1757 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return;

;
}

static int SMV_GetSymbolID
# if defined __STDC__ | defined __cplusplus
(tSymbol yyP50)
# else
(yyP50)
 tSymbol yyP50;
# endif
{
/* line 1761 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return SymbolKind (yyP50) -> AnId . SMV_idnum;

}

static bool SMV_Type_IsTimer
# if defined __STDC__ | defined __cplusplus
(register tTree yyP51)
# else
(yyP51)
 register tTree yyP51;
# endif
{
  if (yyP51->Kind == kTimer) {
/* line 1767 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return true;

  }
/* line 1770 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */
   return false;

}

void BeginSMVout ()
{
/* line 730 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */

    NextPCId = 0;
    NextIntId = 1;
    NextResult = 0;
    Channels = (SMV_Channels)malloc(sizeof(struct channels));
    Channels->chanid = 0;
    Channels->cases = NULL;
    Channels->next = NULL;
  
}

void CloseSMVout ()
{
/* line 743 "/home/simon/spoc/source/spoc.1.3//source/gmd/smv.trafo" */

    SMV_PCs curctr,nextctr;
    SMV_Channels curchan,nextchan;
    SMV_Variables curvar,nextvar;
    SMV_PC_Cases curctrcase,nextctrcase;
    SMV_Channel_Cases curchancase,nextchancase;
    SMV_Variable_Cases curvarcase,nextvarcase;

    /* Free up the program counter list.*/
    for(curctr=nextctr=PCHead; nextctr; curctr=nextctr) {
      nextctr = curctr->next;
      for(curctrcase=nextctrcase=curctr->cases; nextctrcase; curctrcase=nextctrcase) {
        nextctrcase = curctrcase->next;
        free(curctrcase->condition);
        free(curctrcase->result);
        free(curctrcase);
      }
      free(curctr);
    }

    /* Free up the channels list.*/
    for(curchan=nextchan=Channels; nextchan; curchan=nextchan) {
      nextchan = curchan->next;
      for(curchancase=nextchancase=curchan->cases; nextchancase; curchancase=nextchancase) {
        nextchancase = curchancase->next;
        if(curchancase->what==from_tracked_expr)
          free((char*)curchancase->data);
        free(curchancase);
      }
      free(curchan);
    }

    /* Free up the variables list.*/
    for(curvar=nextvar=Variables; nextvar; curvar=nextvar) {
      nextvar = curvar->next;
      for(curvarcase=nextvarcase=curvar->cases; nextvarcase; curvarcase=nextvarcase) {
        nextvarcase = curvarcase->next;
        if(curvarcase->what==from_tracked_expr)
          free((char*)curvarcase->data);
        free(curvarcase);
      }
      free(curvar);
    }
  
}
