#!/usr/bin/perl

# Check the arguments
if ($#ARGV == 0 && $ARGV[0] =~ /^(.*)\.smv$/) {
  $filename = $1;
}
else {
  die "Syntax: $0 <filename>.smv\n";
}

# Open the files
open(DATAFH, "$filename.dat") or die "Can't open $filename.dat: $!\n";
open(SOURCE, "$filename.occ") or die "Can't open $filename.occ: $!\n";
open(SMVOUT, "smv $filename.smv |") or die "Can't open smv output: $!\n";

# Read in the data file
%data = ();

while ($line = <DATAFH>) {
  chomp ($line);
  ($x,$y) = split(":",$line);
  push @{ $data{$x}}, $y;
}

# Read in the source file
for ($i=1; $line = <SOURCE>; $i++) {
  $line =~ /^\s*(.*)$/;
  $source[$i] = $1;
}

# Loop over the SMV output
while ($line = <SMVOUT>) {
  if ($line =~ /^(--|$)/) { print $line;}
  elsif ($line =~ /^(termination \= 0|rangeerror \= 0)/) { }
  elsif ($line =~ /^(rangeerror|termination)/) { print $line; }
  elsif ($line =~ /^(int[0-9]+) = ([0-9]+)/) {
    printf "%s = %d\n", ${$data{$1}}[0], $2;
  }
  # Deal with a PC value.
  elsif ($line =~ /^(pc[0-9]+) = ([0-9]+)/) {
    foreach $linenum (@{$data{$1._.$2}}) {
      if ($linenum =~ /\#(\d*)/) {
        printf "[%2d] %s\n", $1, $source[$1];
      } else {
        print "[--] (", $linenum, ")\n";
      }
    }
  }
}
