# include "PICout.h"
# include "yyPICout.w"
# include "System.h"
# include <stdio.h>
# include "Tree.h"

# define yyInline
# ifndef NULL
# define NULL 0L
# endif
# ifndef false
# define false 0
# endif
# ifndef true
# define true 1
# endif

# ifdef yyInline
# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
  free += nodesize [kind]; \
  ptr->yyHead.yyMark = 0; \
  ptr->Kind = kind;
# else
# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
# endif

# define yyWrite(s) (void) fputs (s, yyf)
# define yyWriteNl (void) fputc ('\n', yyf)

/* line 28 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */

  #include <stdio.h>

  /* These flags allow error checking to be selectively enabled/disabled for each part of the Occam langauge. */
  /* To adhere to the Occam standard, all of the checks must be enabled. */
  /* There is a considerable overhead in both time and space to perform these checks. */
  bool PIC_CheckForInvalidShifts      = true;  /* a shift is invalid if the shift amount is negative or is greater than the size of the data type being shifted */
  bool PIC_CheckForInvalidReplicators = true;  /* a replicator is invalid if the FOR value is negative */
  bool PIC_CheckForArithmeticOverflow = true;  /* overflows can occur in additions and subtractions */
  bool PIC_CheckForFailedIFs          = true;  /* an IF fails when none of the guards evaluates to TRUE */
  bool PIC_CheckForFailedCASEs        = true;  /* a CASE fails when none of the expressions matches the selector */
  bool PIC_CheckForInvalidConversions = true;  /* a conversion is invalid if the source operand exceeds the size of the destination */

  /* Data about the PIC 16C84. */
  #define MAX_PIC_DATA (36)
  #define MAX_PIC_PROG (1024)
  #define REG_START (12)
  #define REG_END (REG_START + MAX_PIC_DATA - 1)

  static unsigned int instr_count;       /* the number of instructions which have been output so far */
  static unsigned int last_instr_count;  /* the value of instr_count when the last block was started */
  static unsigned int last_instr_branch; /* the instr_count of the last conditional branch instruction */

  #define InvalidW (0xFFFFFFFF)
  static int currentW;

  #define InvalidProcess (-1)
  static int start_process;
  static int current_process;

  static void PIC_Tree(tTree);
  static void PIC_SelectBank0(void);
  static void PIC_SelectBank1(void);
  static void PIC_SetSchedulingBit(int);
  static void PIC_ClearSchedulingBit(int);
  static void PIC_CallIfSchedulingBitSet(int);
  static void PIC_StoreConstantIntoMemory(int,int);
  static void PIC_ClearBitInMemory(int);
  static void PIC_SetBitInMemory(int);
  static void PIC_SkipIfBitSet(int);
  static void PIC_ReturnValueInW(int);
  static void PIC_IncrementMemory(int);
  static void PIC_AddWToMemory(int);
  static void PIC_DecrementMemoryAndSkipIfZero(int);
  static void PIC_LoadMemoryIntoW(int);
  static void PIC_SkipIfCarryFlagIsSet(void);
  static void PIC_SkipIfCarryFlagIsClear(void);
  static void PIC_SkipIfZeroFlagIsClear();
  static void PIC_RotateMemoryLeft(int);
  static void PIC_RotateMemoryRight(int);
  static void PIC_ComplementMemory(int);
  static void PIC_XORMemoryIntoW(int);
  static void PIC_StoreWIntoMemory(int);
  static void PIC_InvalidateEntireCache(void);

  /* This determines whether timer handling code will be generated. */
  /* By default it is set to FALSE, which means that no timer code will be produced. */
  /* It will automatically be set to true, when a TIMER definition is encountered. */
  static bool IsTimerUsedFlag;

  /* This stores the address of the timer counter. */
  /* The timer is a 16-bit INT, so we need two bytes. */
  static int TimerAddressMSB;
  static int TimerAddressLSB;

  /* This determines whether the code to perform 16-bit multiplication will be generated. */
  /* By default it is set to FALSE, which means that no code will be produced. */
  /* It will automatically be set to TRUE when a 16-bit multiply operator is encountered. */
  static bool IsMul16Needed;

  /* These store the address of the result and operands for 16-bit multiplications. */
  static int Mul16Address0;  /* this is used to store the result */
  static int Mul16Address1;  /* this is used to store the 1st operand */
  static int Mul16Address2;  /* this is used to store the 2nd operand */

  /* This is the filehandle, used for all assembler output. */
  static FILE* fh;

  typedef struct process_element
    {
    int                     id;
    struct process_element* next;
    }process_element;
  static process_element* process_list;

  /* This records which bits of memory are currently allocated. */
  /* 0=free, 1=allocated */
  static unsigned char* UsedMemory;

  static unsigned int*  MemoryLevels;
  static unsigned int   Level;

  /* This records whether we know the status of each bit of memory. */
  /* 0=unknown, 1=known */
  static unsigned char* MemoryValid;

  /* This records the status of each bit of memory. */
  /* The contents is accurate only when the corresponding MemoryValid bit is set. */
  static unsigned char* MemoryContents;

  /* This records the number of bits which have been shared. */
  /* It is used for statistical purposes only. */
  static unsigned int SharedMemory;

  /* This records the number of cache hits which result in code savings. */
  static unsigned int cache_hits;

  /* Checks whether all of the bits in the specified byte are cached. */
  /* Returns true if ALL of the bits are cached, and false otherwise. */
  static bool PIC_IsByteFullyCached(int address)
    {
    if (address<REG_START) return false;
    return (MemoryValid[address-REG_START]==0xFF) ? true : false;
    }

  /* Returns the contents of a cached memory location. */
  /* The result is undefined for uncached bits. */
  static unsigned char PIC_GetCachedByte(int address)
    {
    return MemoryContents[address-REG_START];
    }

  /* Returns the value of a cached bit. */
  /* The result is undefined if the bit is not cached. */
  static bool PIC_GetCachedBit(int address)
    {
    int byte = address / 8;
    int bit  = address % 8;
    return (MemoryContents[byte-REG_START] & (1<<bit)) ? true : false;
    }

  /* Sets the contents of a byte, and caches it. */
  /* If the address is invalid, then do nothing. */
  static void PIC_CacheByte(int address, unsigned char value)
    {
    if (address==0) {PIC_InvalidateEntireCache(); return;}
    if (address<REG_START) return;
    MemoryContents[address-REG_START] = value;
    MemoryValid[address-REG_START] = 0xFF;
    }

  /* Checks whether the specified bit is cached. */
  /* Returns TRUE is the bit is cached, and FALSE otherwise. */
  /* If the address is invalid, then do nothing. */
  static bool PIC_IsBitCached(int address)
    {
    int byte = address / 8;
    int bit  = address % 8;
    if (address<REG_START) return false;
    return (MemoryValid[byte-REG_START] & (1<<bit)) ? true : false;
    }

  /* Sets the contents of a bit, and caches it. */
  /* If the address is invalid, then do nothing. */
  static void PIC_CacheBit(int address, bool value)
    {
    int byte = address / 8;
    int bit  = address % 8;
    if (byte==0) {PIC_InvalidateEntireCache(); return;}
    if (byte<REG_START) return;
    MemoryContents[byte-REG_START] &= ~(1<<bit);
    MemoryContents[byte-REG_START] |= (((value==true) ? 1 : 0) << bit);
    MemoryValid[byte-REG_START] |= (1 << bit);
    }

  /* Invalidates a single bit of memory. */
  /* The address is in the form (byte*8)+bit. */
  /* If the address is invalid, then do nothing. */
  static void PIC_InvalidateCachedBit(int address)
    {
    int byte = address / 8;
    int bit  = address % 8;
    if (byte==0) {PIC_InvalidateEntireCache(); return;}
    if (byte<REG_START) return;
    MemoryValid[byte-REG_START] &= (~(1<<bit));
    }

  /* Invalidates a single byte of memory. */
  /* If the address is invalid, then do nothing. */
  static void PIC_InvalidateCachedByte(int address)
    {
    if (address==0) {PIC_InvalidateEntireCache(); return;}
    if (address<REG_START) return;
    MemoryValid[address-REG_START]=0;
    }

  /* Invalidates all cached memory entires. */
  static void PIC_InvalidateEntireCache(void)
    {
    unsigned int n;
    for(n=0; n<MAX_PIC_DATA; n++)
      MemoryValid[n]=0;
    }

  /* This will allocate a block of memory of the specified size. */
  /* The start address of the memory block will be returned. */
  static int PIC_AllocateBytes(unsigned int size)
    {
    unsigned int m;
    unsigned int count=0;
    for(m=0; m<MAX_PIC_DATA; m++)
      {
      if (UsedMemory[m]==0x00) count++;
      else count=0;

      if (count==size)
        {
        while(count!=0)
          {
          /* iterate through the bits we want to allocate */
          unsigned int b;
          for(b=0; b<8; b++)
            {
            /* update the SharedMemory variable */
            if (MemoryLevels[m*8+b]!=0) SharedMemory++;

            /* mark the bit as being used */
            MemoryLevels[m*8+b]=Level;
            }
          UsedMemory[m]=0xFF;
          m--;
          count--;
          }
/*        fprintf(fh,"allocating %d bytes at %d on level %d\n",size,m+REG_START+1,Level);*/
        Level++;
        return m + REG_START + 1;
        }
      }

    /* there is no free memory */
    /* generate an error */
    return InvalidW;
    }

  /* This will allocate a single bit of memory. */
  /* The address of the bit will be returned. */
  /* The address is (byte*8)+bit. */
  static int PIC_AllocateBit(void)
    {
    unsigned int m;
    for(m=0; m<MAX_PIC_DATA; m++)
      if (UsedMemory[m]!=0xFF)
        {
        unsigned int b;
        for(b=0; b<8; b++)
          if ((UsedMemory[m] & (1<<b))==0)
            {
            /* update the SharedMemory variable */
            if (MemoryLevels[m*8+b]!=0) SharedMemory++;

            /* mark the bit as being used */
            UsedMemory[m] |= 1<<b;
            MemoryLevels[m*8+b] = Level++;

            return ((REG_START + m) * 8) + b;
            }
        }

    /* there is no free memory */
    /* generate an error */
    return InvalidW;
    }

  /* This will allocate a single bit of memory. */
  /* The bit is guaranteed not to have been used previously, */
  /* and it will not be used in the future. */
  /* The address of the bit will be returned. */
  /* The address is (byte*8)+bit. */
  static int PIC_AllocateUniqueBit(void)
    {
    unsigned int m;
    for(m=0; m<MAX_PIC_DATA; m++)
      if (UsedMemory[m]!=0xFF)
        {
        unsigned int b;
        for(b=0; b<8; b++)
          if (((UsedMemory[m] & (1<<b))==0) && (MemoryLevels[m*8+b]==0))
            {
            /* mark the bit as being used */
            UsedMemory[m] |= 1<<b;
            MemoryLevels[m*8+b] = 0xFFFFFFFF;

            return ((REG_START + m) * 8) + b;
            }
        }

    /* there is no free memory */
    /* generate an error */
    return InvalidW;
    }

  /* This will deallocate a block of memory, and allow it to be reused. */
  static void PIC_DeallocateBytes(int address,int size)
    {
/*    fprintf(fh,"deallocating %d bytes at %d\n",size,address);*/
    address -= REG_START;
    while(size>0)
      UsedMemory[address+(--size)]=0x00;
    }

  /* This will deallocate a bit of memory, and allow it to be reused. */
  static void PIC_DeallocateBit(int address)
    {
    UsedMemory[(address/8) - REG_START] &= ~(1<<(address%8));
    }

  /* This will deallocate multiple bits, starting from the specified address. */
  static void PIC_DeallocateBits(int address, int size)
    {
    while(size>0)
      {
      UsedMemory[(address/8) - REG_START] &= ~(1<<(address%8));
      address++;
      size--;
      }
    }

  /* Sets the allocated bit for each memory location having an level >= s and < e. */
  /* This prevents the specified memory locations from being allocated. */
  static void PIC_RestrictMemory(unsigned int s, unsigned int e)
    {
    unsigned int m;
    for(m=0; m<MAX_PIC_DATA; m++)
      {
      unsigned int b;
      for(b=0; b<8; b++)
        if ((MemoryLevels[m*8+b]>=s) && (MemoryLevels[m*8+b]<e)) UsedMemory[m]|=1<<b;
      }
    }

  /* Clears the allocated bit for each memory location having a level >= s and < e. */
  /* This allows the specified memory locations to be allocated again. */
  static PIC_DerestrictMemory(unsigned int s, unsigned int e)
    {
    unsigned int m;
    for(m=0; m<MAX_PIC_DATA; m++)
      {
      unsigned int b;
      for(b=0; b<8; b++)
        if ((MemoryLevels[m*8+b]>=s) && (MemoryLevels[m*8+b]<e)) UsedMemory[m]&=~(1<<b);
      }
    }

  /* Returns a unique process number, and allocates memory for the scheduling bit. */
  /* The allocated process number will be added to process_list for future reference. */
  static int PIC_NewProcess(void)
    {
    int pr = PIC_AllocateUniqueBit();
    process_element* pe = (process_element*)malloc(sizeof(process_element));
    if (pe==NULL) Message("pic.trafo: malloc() failed in PIC_NewProcess()",xxFatal,NoPosition);
    pe->id = pr;
    pe->next = process_list;
    process_list = pe;
    return pr;
    }

  #define GetSchedulingBit(process) (process%8)
  #define GetSchedulingByte(process) (process/8)

  /* Starts a known process, and allows it to be given a comment. */
  /* The comment should not contain any newlines. */
  static void PIC_StartProcess2(int process, const char* comments)
    {
    current_process = process;
    if (current_process!=InvalidProcess)
      fprintf(fh,"\nproc%d:\t\t;%s\n",current_process,comments);
    currentW=InvalidW;
    PIC_InvalidateEntireCache();
    last_instr_count=instr_count;
    }

  /* Starts a new process, and allows it to be given a comment. */
  /* The comment should not contain any newlines. */
  static void PIC_StartProcess(const char* comments)
    {
    PIC_StartProcess2(PIC_NewProcess(),comments);
    }

  /* Returns true is we are in an InvalidProcess, since we can not be sure whether any code has been generated. */
  /* Returns true if we are in a known process, and the current block contains at least one instruction. */
  /* Otherwise false is returned. */
  static bool PIC_IsCurrentBlockUsed(void)
    {
    return ((current_process==InvalidProcess) ||
            ((current_process!=InvalidProcess) && (last_instr_count!=instr_count))) ? true : false;
    }

  static unsigned int current_label;

  static unsigned int PIC_GetNewLabel(void)
    {
    return current_label++;
    }

  static void PIC_OutputLabel(unsigned int label)
    {
    fprintf(fh,"\nlab%d:\n",label);
    currentW=InvalidW;
    PIC_InvalidateEntireCache();
    }

  static void PIC_GotoLabel(unsigned int label,const char* comments)
    {
    fprintf(fh,"  GOTO lab%d\t;%s\n",label,comments);
    instr_count++;
    }

  static int NextChannelId;
  typedef struct channel_element
    {
    int                     channel;
    int                     proc_ready;
    int                     proc_cont;
    int                     address;
    struct channel_element* next;
    }channel_element;
  static channel_element* channel_list;

  /* This is used to record the details about each channel output, so that we can generate the channel handler later on. */
  static void PIC_RegisterChannelOutput(int channel,int proc_ready,int proc_cont,int address)
    {
    channel_element* ce = (channel_element*)malloc(sizeof(channel_element));
    if (ce==NULL) Message("pic.trafo: malloc() failed in PIC_RegisterChannelOutput()",xxFatal,NoPosition);
    ce->channel    = channel;
    ce->proc_ready = proc_ready;
    ce->proc_cont  = proc_cont;
    ce->address    = address;
    ce->next       = channel_list;
    channel_list   = ce;
    }

  /* This is used to output some comments at the start of the assembly file. */
  /* The 1st instruction will be executed after a power-up/reset. */
  /* The 4th instruction will be executed after an interrupt has occured. */
  static void PIC_Header(void)
    {
    fprintf(fh,"; Code generated by occ2c.\n");
    fprintf(fh,"; Assembly code targetted at PIC 16C84.\n");
    fprintf(fh,"\n\n  GOTO start\t;reset handler (must be 1st instruction)\n");
    fprintf(fh,"  NOP\t\t;don't remove this instruction - it is essential!\n");
    fprintf(fh,"  NOP\t\t;don't remove this instruction - it is essential!\n");
    fprintf(fh,"  NOP\t\t;don't remove this instruction - it is essential!\n");
    fprintf(fh,"  GOTO int\t;interrupt handler (must be 4th instruction)\n");
    instr_count+=5;
    }

  /* This will generate the channel handler code for each of the channel outputs which we registered earlier. */
  static void PIC_Channels(void)
    {
    int chan;
    for(chan=0; chan<NextChannelId; chan++)
      {
      channel_element* ce = channel_list;
      channel_element* pe = NULL;
      fprintf(fh,"\nchan%d:\n",chan);
      while(ce!=NULL)
        {
        if (ce->channel==chan)
          {
          if (pe!=NULL)
            {
            int label = PIC_GetNewLabel();
            PIC_InvalidateEntireCache();
            PIC_SkipIfBitSet(pe->proc_ready);
            PIC_GotoLabel(label,"channel output flag is clear, so try next channel");
            PIC_ClearBitInMemory(pe->proc_ready);
            PIC_SetSchedulingBit(pe->proc_cont);
            PIC_ReturnValueInW(pe->address);
            PIC_OutputLabel(label);
            }
          pe=ce;
          }
        ce=ce->next;
        }
      if (pe!=NULL)
        PIC_SkipIfBitSet(pe->proc_ready);
      PIC_ReturnValueInW(0);
      if (pe!=NULL)
        {
        PIC_ClearBitInMemory(pe->proc_ready);
        PIC_SetSchedulingBit(pe->proc_cont);
        PIC_ReturnValueInW(pe->address);
        }
      }
    }

  /* Generates code to perform a 16-bit multiplication. */
  static void PIC_Multiply16(void)
    {
    int loop;
    int no_add;
    int no_neg1;
    int no_neg2;

    if (IsMul16Needed==false) return;

    PIC_InvalidateEntireCache();

    fprintf(fh,"\nmul16:\n");
    fprintf(fh,"  ;This code performs a signed multiplication on two\n");
    fprintf(fh,"  ;16-bit operands, and produces a 16-bit result.\n");
    fprintf(fh,"  ;Negative operands will be inverted, and the sign of\n");
    fprintf(fh,"  ;the result will be sorted out at the end.\n");
    fprintf(fh,"  ;In the worst case, 270 instructions will be executed.");

    /* calculate the sign of the result and store it in bit 7 of register 4 */
    PIC_LoadMemoryIntoW(Mul16Address1+1);
    PIC_XORMemoryIntoW(Mul16Address2+1);
    PIC_StoreWIntoMemory(4);

    /* negate operand 1 if it is negative */
    no_neg1=PIC_GetNewLabel();
    PIC_SkipIfBitSet((Mul16Address1*8)+7);
    PIC_GotoLabel(no_neg1,"1st operand is positive, so don't negate it");
    PIC_ComplementMemory(Mul16Address1+0);
    PIC_ComplementMemory(Mul16Address1+1);
    PIC_IncrementMemory(Mul16Address1+0);
    PIC_SkipIfZeroFlagIsClear();  /* must test for Zero, since INCF does not update the Carry flag */
    PIC_IncrementMemory(Mul16Address1+1);
    PIC_OutputLabel(no_neg1);

    /* negate operand 2 if it is negative */
    no_neg2=PIC_GetNewLabel();
    PIC_SkipIfBitSet((Mul16Address2*8)+7);
    PIC_GotoLabel(no_neg2,"2nd operand is positive, so don't negate it");
    PIC_ComplementMemory(Mul16Address2+0);
    PIC_ComplementMemory(Mul16Address2+1);
    PIC_IncrementMemory(Mul16Address2+0);
    PIC_SkipIfZeroFlagIsClear();  /* must test for Zero, since INCF does not update the Carry flag */
    PIC_IncrementMemory(Mul16Address2+1);
    PIC_OutputLabel(no_neg2);

    /* initialise the result to zero */
    PIC_StoreConstantIntoMemory(0,Mul16Address0+0);
    PIC_StoreConstantIntoMemory(0,Mul16Address0+1);

    /* place the overall sign into the Carry flag */
    PIC_RotateMemoryLeft(4);

    /* initialise the loop counter (register 4) */
    PIC_StoreConstantIntoMemory(16,4);

    /* start of loop */
    loop=PIC_GetNewLabel();
    PIC_OutputLabel(loop);

    /* rotate the 1st operand right by one bit */
    /* the 1st bit will be filled with the overall sign bit */
    /* other vacant bits will be filled with rubbish */
    /* when the loop ends, the sign bit will be in bit 0 of the least significant byte */
    PIC_RotateMemoryRight(Mul16Address1+1);
    PIC_RotateMemoryRight(Mul16Address1+0);

    /* if the Carry is set, add the 2nd operand to the result */
    no_add=PIC_GetNewLabel();
    PIC_SkipIfCarryFlagIsSet();
    PIC_GotoLabel(no_add,"skip the addition");
      PIC_LoadMemoryIntoW(Mul16Address2+0);
      PIC_AddWToMemory(Mul16Address0+0);
      PIC_SkipIfCarryFlagIsClear();
      PIC_IncrementMemory(Mul16Address0+1);  /* if this results in a Carry then an overflow has occured! */
      PIC_LoadMemoryIntoW(Mul16Address2+1);
      PIC_AddWToMemory(Mul16Address0+1);  /* if this results in a Carry then an overflow has occured! */
    PIC_OutputLabel(no_add);

    /* clear the Carry flag, and then rotate the 2nd operand left by 1 */
    PIC_ClearBitInMemory((3*8)+0);
    PIC_RotateMemoryLeft(Mul16Address2+0);
    PIC_RotateMemoryLeft(Mul16Address2+1);

    /* decrement counter (register 4), and goto start of loop if not Zero */
    PIC_DecrementMemoryAndSkipIfZero(4);
    PIC_GotoLabel(loop,"iterate again");

    /* negate the result if necessary */
    /* the sign is obtained from bit 0 of the 1st operand */
    PIC_SkipIfBitSet((Mul16Address1*8)+0);
    fprintf(fh,"  RETURN\t;return to the caller\n");
    instr_count+=1;
    PIC_ComplementMemory(Mul16Address0+0);
    PIC_ComplementMemory(Mul16Address0+1);
    PIC_IncrementMemory(Mul16Address0+0);
    PIC_SkipIfZeroFlagIsClear();
    PIC_IncrementMemory(Mul16Address0+1);

    /* return to the caller */
    fprintf(fh,"  RETURN\t;return to the caller\n");
    instr_count+=1;
    }

  /* This generates the scheduler for all of the processes. */
  static void PIC_Scheduler(void)
    {
    unsigned int m;
    process_element* pl;

    PIC_InvalidateEntireCache();

    /* initialise the scheduling bits */
    fprintf(fh,"\nstart:\n");
    for(m=0; m<MAX_PIC_DATA; m++)
      {
      pl=process_list;
      while(pl!=NULL)
        {
        if ((GetSchedulingByte(pl->id)==m) && (pl->id!=start_process))
          {
          PIC_StoreConstantIntoMemory(0,m);
          pl=NULL;
          }
        else
          pl = pl->next;
        }
     }

    /* initialise the timer if it is used wihtin the program */
    if (IsTimerUsedFlag==true)
      {
      /* select page 0 of the register map */
      PIC_SelectBank0();

      /* clear the TMR0 counter (register 1) */
      PIC_StoreConstantIntoMemory(0x00,1);

      /* select page 1 of the register map */
      PIC_SelectBank1();

      /* clear the watchdog timer */
      fprintf(fh,"  CLRWDT\t;clear the watchdog timer\n");
      instr_count=instr_count+1;

      /* initialise the timer using the OPTION register */
      /* use processor clock, do not use prescaler */
      PIC_StoreConstantIntoMemory(0x00,0x01);

      /* select page 0 of the register map */
      PIC_SelectBank0();

      /* set global interrupt enable and TMR0 interrupt enable in INTCON (register 11) */
      PIC_StoreConstantIntoMemory(0xA0,0x0B);
      }

    /* call the first process */
    fprintf(fh,"  CALL lab%d\n",start_process);
    instr_count=instr_count+1;

    PIC_InvalidateEntireCache();

    /* generate the scheduling bit checking code */
    fprintf(fh,"\nscheduler:\n");
    pl=process_list;
    while(pl!=NULL)
      {
      PIC_CallIfSchedulingBitSet(pl->id);
      pl = pl->next;
      }

    /* generate the loop */
    fprintf(fh,"  GOTO scheduler\n");
    instr_count++;
    }

  /* This generates code to handle interrupts. */
  static void PIC_Interrupts(void)
    {
    fprintf(fh,"\nint:\n");
    if (IsTimerUsedFlag==true)
      {
      fprintf(fh,"  ;This interrupt handler has been carefully designed\n");
      fprintf(fh,"  ;so that it doesn't modifiy either the status register\n");
      fprintf(fh,"  ;or the W register. This avoids having to save and\n");
      fprintf(fh,"  ;restore them, thus reducing memory consumption.\n");
      fprintf(fh,"  BCF 11,2\t;clear the TMR0 overflow bit in the INTCON register\n");
      fprintf(fh,"  INCFSZ %d,1\t;increment the LSB of the timer counter, and skip if Zero\n",TimerAddressLSB);
      fprintf(fh,"  RETFIE\t;resume the interrupted program\n");
      fprintf(fh,"  INCFSZ %d,1\t;increment the MSB of the timer counter, and skip if Zero\n",TimerAddressMSB);
      fprintf(fh,"  RETFIE\t;resume the interrupted program\n");
      instr_count=instr_count+5;
      }
    fprintf(fh,"  RETFIE\t;resume the interrupted program\n");
    instr_count++;
    }

  /* This outputs several statistics about the compiled program. */
  static void PIC_Footer(void)
    {
    unsigned int n;
    unsigned int bits;

    /* count the number of data bits used */
    bits=0;
    for(n=0; n<(MAX_PIC_DATA*8); n++)
      if (MemoryLevels[n]>0) bits++;

    /* output data and program memory usage statistics */
    fprintf(fh,"\n\n");
    fprintf(fh,";Memory usage statistics:\n");
    fprintf(fh,";  data memory total: %d bits\n",MAX_PIC_DATA*8);
    fprintf(fh,";  data memory used:  %d (%d%%) bits\n",bits,(unsigned int)(((bits*100.0)/(MAX_PIC_DATA*8))+0.9999));
    fprintf(fh,";  data memory reuse: %d bits\n",SharedMemory);
    fprintf(fh,";  data cache hits:   %d hits\n",cache_hits);
    fprintf(fh,";  prog memory total: %d instructions\n",MAX_PIC_PROG);
    fprintf(fh,";  prog memory used:  %d (%d%%) instructions\n",instr_count,(unsigned int)(((instr_count*100.0)/MAX_PIC_PROG)+0.9999));
    }

  /* This is the function which ParserDrv.c calls when it wants to output PIC assembler. */
  /* If the filename is '-', then the assembler will be printed to stdout. */
  /* Otherwise a new file will be opened, and the output will be writen to it. */
  void OutputPIC(const char* filename, tTree tree)
    {
    fh = (strcmp(filename,"-")==0) ? stdout : fopen(filename,"w");
    if (fh==NULL) MessageI("pic.trafo: cannot open file ",xxFatal,NoPosition,xxString,filename);

    PIC_Header();
    PIC_Tree(tree);
    PIC_Channels();
    PIC_Multiply16();
    PIC_Scheduler();
    PIC_Interrupts();
    PIC_Footer();

    fclose(fh);
    }
  

static void yyExit () { Exit (1); }

void (* PICout_Exit) () = yyExit;

extern FILE *yyf;

static void yyAbort
# ifdef __cplusplus
 (char * yyFunction)
# else
 (yyFunction) char * yyFunction;
# endif
{
 (void) fprintf (stderr, "Error: module PICout, routine %s failed\n", yyFunction);
 PICout_Exit ();
}

static void PIC_Tree ARGS((tTree yyP1));
static void PIC_OuterSpecList ARGS((tTree yyP2));
static void PIC_OuterSpec ARGS((tTree yyP3));
static void PIC_ProcList_Seq ARGS((tTree yyP4));
static int PIC_ProcList_Par1 ARGS((tTree yyP5));
static void PIC_ProcList_Par2 ARGS((int yyP7, tTree yyP6));
static void PIC_Proc ARGS((tTree yyP8));
static void PIC_SpecList_Allocate ARGS((tTree yyP9));
static void PIC_Spec_Allocate ARGS((tTree yyP10));
static void PIC_SpecList_Deallocate ARGS((tTree yyP11));
static void PIC_Spec_Deallocate ARGS((tTree yyP12));
static void PIC_IdList_AllocateMemory ARGS((tTree yyP14, tTree yyP13));
static int PIC_AllocateMemory ARGS((tTree yyP15));
static void PIC_IdList_DeallocateMemory ARGS((tTree yyP17, tTree yyP16));
static void PIC_DeallocateMemory ARGS((int yyP19, tTree yyP18));
static int PIC_Type_GetPrimitiveSize ARGS((tTree yyP20));
static int PIC_Type_GetSize ARGS((tTree yyP21));
static void PIC_Assign ARGS((tTree yyP23, tTree yyP22));
static void PIC_Expr_Evaluate ARGS((tTree yyP24));
static int PIC_Expr_Evaluate2 ARGS((tTree yyP25));
static int PIC_GetSymbolAddress ARGS((tTree yyP26));
static void PIC_EvaluateBinary ARGS((tTree yyP30, tTree yyP29, tTree yyP28, tTree yyP27));
static void PIC_AltList ARGS((int yyP33, int yyP32, tTree yyP31));
static void PIC_AltList_Guard ARGS((int yyP37, int yyP36, int yyP35, tTree yyP34));
static void PIC_CaseChoiceList ARGS((int yyP40, tTree yyP39, tTree yyP38));
static void PIC_CaseChoiceListCompare ARGS((int yyP44, int yyP43, tTree yyP42, tTree yyP41));
static void PIC_IfChoiceList ARGS((int yyP46, tTree yyP45));
static void PIC_Expr_Equal ARGS((int yyP49, tTree yyP48, tTree yyP47));
static void PIC_EvaluateUnary ARGS((tTree yyP52, tTree yyP51, tTree yyP50));
static void PIC_Constant ARGS((int yyP55, tValue yyP54, tTree yyP53));
static bool PIC_Type_IsTimer ARGS((tTree yyP56));
static int PIC_GetChannelNumber ARGS((tSymbol yyP57));
static void PIC_SelectBank0 ARGS(());
static void PIC_SelectBank1 ARGS(());
static void PIC_ClearSchedulingBit ARGS((int yyP58));
static void PIC_SetSchedulingBit ARGS((int yyP59));
static void PIC_SetSchedulingBit_Dynamic ARGS((int yyP61, int yyP60));
static void PIC_StoreDynamicProcessId ARGS((int yyP64, int yyP63, int yyP62));
static void PIC_ReturnToScheduler ARGS(());
static void PIC_ReturnValueInW ARGS((int yyP65));
static void PIC_CallIfSchedulingBitSet ARGS((int yyP66));
static void PIC_CallChannelHandler ARGS((int yyP67));
static void PIC_CallMul16 ARGS(());
static void PIC_StoreConstantIntoMemory ARGS((int yyP69, int yyP68));
static void PIC_SetBitInMemory ARGS((int yyP70));
static void PIC_ClearBitInMemory ARGS((int yyP71));
static void PIC_SkipIfBitSet ARGS((int yyP72));
static void PIC_SkipIfBitClear ARGS((int yyP73));
static void PIC_SkipIfZeroFlagIsSet ARGS(());
static void PIC_SkipIfZeroFlagIsClear ARGS(());
static void PIC_SkipIfCarryFlagIsSet ARGS(());
static void PIC_SkipIfCarryFlagIsClear ARGS(());
static void PIC_GotoLabelIfBitSet ARGS((int yyP75, int yyP74));
static void PIC_GotoLabelIfBitClear ARGS((int yyP77, int yyP76));
static void PIC_TestWForZero ARGS(());
static void PIC_TestMemoryForZero ARGS((int yyP78));
static void PIC_IncrementMemory ARGS((int yyP79));
static void PIC_IncrementMemoryToW ARGS((int yyP80));
static void PIC_IncrementMemoryAndSkipIfZero ARGS((int yyP81));
static void PIC_DecrementMemory ARGS((int yyP82));
static void PIC_DecrementMemoryAndSkipIfZero ARGS((int yyP83));
static void PIC_LoadMemoryIntoW ARGS((int yyP84));
static void PIC_LoadConstantIntoW ARGS((int yyP85));
static void PIC_StoreWIntoMemory ARGS((int yyP86));
static void PIC_CopyBit ARGS((int yyP88, int yyP87));
static void PIC_CopyByte ARGS((int yyP90, int yyP89));
static void PIC_XORMemoryIntoW ARGS((int yyP91));
static void PIC_XORWIntoMemory ARGS((int yyP92));
static void PIC_XORConstantIntoW ARGS((int yyP93));
static void PIC_ORMemoryIntoW ARGS((int yyP94));
static void PIC_ORWIntoMemory ARGS((int yyP95));
static void PIC_ANDMemoryIntoW ARGS((int yyP96));
static void PIC_ANDWIntoMemory ARGS((int yyP97));
static void PIC_ANDConstantIntoW ARGS((int yyP98));
static void PIC_AddConstantToW ARGS((int yyP99));
static void PIC_AddMemoryToW ARGS((int yyP100));
static void PIC_AddWToMemory ARGS((int yyP101));
static void PIC_ComplementMemory ARGS((int yyP102));
static void PIC_ComplementMemoryToW ARGS((int yyP103));
static void PIC_RotateMemoryLeft ARGS((int yyP104));
static void PIC_RotateMemoryRight ARGS((int yyP105));
static void PIC_SubtractWFromConstant ARGS((int yyP106));
static void PIC_SubtractWFromMemory ARGS((int yyP107));
static void PIC_SwapMemoryToW ARGS((int yyP108));

static void PIC_Tree
# if defined __STDC__ | defined __cplusplus
(register tTree yyP1)
# else
(yyP1)
 register tTree yyP1;
# endif
{
  if (yyP1->Kind == kStart) {
  if (yyP1->Start.Process->Kind == kNullProcess) {
/* line 818 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 819 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OuterSpecList (yyP1->Start.Process->NullProcess.SpecList);
  }
   return;

  }
  }
 yyAbort ("PIC_Tree");
;
}

static void PIC_OuterSpecList
# if defined __STDC__ | defined __cplusplus
(register tTree yyP2)
# else
(yyP2)
 register tTree yyP2;
# endif
{
  if (yyP2->Kind == kASpec) {
/* line 824 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 825 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OuterSpec (yyP2->ASpec.Spec);
/* line 826 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OuterSpecList (yyP2->ASpec.Next);
  }
   return;

  }
  if (yyP2->Kind == kNoSpec) {
/* line 828 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
 yyAbort ("PIC_OuterSpecList");
;
}

static void PIC_OuterSpec
# if defined __STDC__ | defined __cplusplus
(register tTree yyP3)
# else
(yyP3)
 register tTree yyP3;
# endif
{

  switch (yyP3->Kind) {
  case kDefProc:
  case kInlineDefProc:
/* line 833 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 836 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   start_process = PIC_GetNewLabel ();
/* line 837 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (start_process);
/* line 838 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StartProcess2 (InvalidProcess, "MAIN");
/* line 839 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP3->DefProc.Process);
/* line 840 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 841 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
   return;

  case kPrototypeProc:
/* line 843 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 844 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with PrototypeProc node", xxError, yyP3->PrototypeProc.Pos);
  }
   return;

  case kDefFunc:
  case kInlineDefFunc:
/* line 846 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 847 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefFunc node", xxError, yyP3->DefFunc.Pos);
  }
   return;

  case kPrototypeFunc:
/* line 849 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 850 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with PrototypeFunc node", xxError, yyP3->PrototypeFunc.Pos);
  }
   return;

  case kHCode:
/* line 852 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 853 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with HCode node", xxError, yyP3->HCode.Pos);
  }
   return;

  case kAVal:
  case kVal:
  case kValRetypes:
  case kValReshapes:
/* line 855 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kDefSeqPro:
/* line 857 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 858 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefSeqPro node", xxError, yyP3->DefSeqPro.Pos);
  }
   return;

  case kDefTagPro:
/* line 860 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 861 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefTagPro node", xxError, yyP3->DefTagPro.Pos);
  }
   return;

  case kDeclType:
/* line 863 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kNullSpec:
/* line 865 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }

 yyAbort ("PIC_OuterSpec");
;
}

static void PIC_ProcList_Seq
# if defined __STDC__ | defined __cplusplus
(register tTree yyP4)
# else
(yyP4)
 register tTree yyP4;
# endif
{
  if (yyP4->Kind == kAProcess) {
/* line 873 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 874 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP4->AProcess.Process);
/* line 875 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ProcList_Seq (yyP4->AProcess.Next);
  }
   return;

  }
  if (yyP4->Kind == kNoProcess) {
/* line 877 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
 yyAbort ("PIC_ProcList_Seq");
;
}

static int PIC_ProcList_Par1
# if defined __STDC__ | defined __cplusplus
(register tTree yyP5)
# else
(yyP5)
 register tTree yyP5;
# endif
{
  if (yyP5->Kind == kAProcess) {
/* line 885 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 886 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP5->AProcess.Process -> Process . PIC_PRC = PIC_NewProcess ();
/* line 887 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetSchedulingBit (yyP5->AProcess.Process -> Process . PIC_PRC);
  }
   return PIC_ProcList_Par1 (yyP5->AProcess.Next) + 1;

  }
  if (yyP5->Kind == kNoProcess) {
/* line 890 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 0;

  }
 yyAbort ("PIC_ProcList_Par1");
}

static void PIC_ProcList_Par2
# if defined __STDC__ | defined __cplusplus
(register int yyP7, register tTree yyP6)
# else
(yyP7, yyP6)
 register int yyP7;
 register tTree yyP6;
# endif
{
/* line 899 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
unsigned int s; unsigned int e;
  if (yyP6->Kind == kAProcess) {
/* line 900 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 901 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   s = Level;
/* line 902 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StartProcess2 (yyP6->AProcess.Process -> Process . PIC_PRC, "PAR - child process");
/* line 903 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP6->AProcess.Process);
/* line 904 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   e = Level;
/* line 905 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_RestrictMemory (s, e);
/* line 906 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 907 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DecrementMemory (yyP7);
/* line 908 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 909 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ProcList_Par2 (yyP7, yyP6->AProcess.Next);
/* line 910 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DerestrictMemory (s, e);
  }
   return;

  }
  if (yyP6->Kind == kNoProcess) {
/* line 912 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
 yyAbort ("PIC_ProcList_Par2");
;
}

static void PIC_Proc
# if defined __STDC__ | defined __cplusplus
(register tTree yyP8)
# else
(yyP8)
 register tTree yyP8;
# endif
{
/* line 917 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int np; tTree tr; int index; int count; int label1; int label2; bool count_const; int chan; int adr; int size; int n;

  switch (yyP8->Kind) {
  case kAssign:
/* line 918 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 923 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Assign.SpecList);
/* line 924 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Assign (yyP8->Assign.Lval, yyP8->Assign.Rval);
/* line 925 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Assign.SpecList);
  }
   return;

  case kAlt:
/* line 927 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 929 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int label=PIC_GetNewLabel();
    PIC_SpecList_Allocate(yyP8->Alt.SpecList);
    PIC_AltList(current_process,label,yyP8->Alt.AltList);
    PIC_SpecList_Deallocate(yyP8->Alt.SpecList);
    PIC_OutputLabel(label);
    }
  }
   return;

  case kAltRep:
/* line 937 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 940 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_CheckForInvalidReplicators == true))) goto yyL3;
  {
/* line 941 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->AltRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL3;
  {
/* line 942 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->AltRep.Replicator -> Replicator . For -> Expr . CV . Number < 0))) goto yyL3;
  {
/* line 943 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 944 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
  }
  }
  }
   return;
yyL3:;

/* line 946 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 950 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->AltRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL4;
  {
/* line 951 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->AltRep.Replicator -> Replicator . From -> Expr . CV . Number == 0))) goto yyL4;
  {
/* line 952 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->AltRep.SpecList);
/* line 953 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->AltRep.SpecList);
  }
  }
  }
   return;
yyL4:;

/* line 955 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 958 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: replicated ALT is not supported", xxRestriction, yyP8->AltRep.Pos);
/* line 959 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->AltRep.SpecList);
/* line 960 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->AltRep.SpecList);
  }
   return;

  case kInput:
  if (yyP8->Input.ExprList->Kind == kAnExpr) {
  if (yyP8->Input.ExprList->AnExpr.Next->Kind == kNoExpr) {
/* line 962 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 964 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_IsTimer (yyP8->Input.Element -> UnmappedIdentifier . SynType) == true))) goto yyL6;
  {
/* line 966 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Input.SpecList);
/* line 969 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory ((11 * 8) + 5);
/* line 972 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   adr = PIC_GetSymbolAddress (SymbolKind (yyP8->Input.ExprList->AnExpr.Expr -> ElementalExpr . Elemental -> UnmappedIdentifier . Sym));
/* line 975 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CopyByte (adr + 0, TimerAddressLSB);
/* line 976 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CopyByte (adr + 1, TimerAddressMSB);
/* line 979 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory ((11 * 8) + 5);
/* line 981 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Input.SpecList);
  }
  }
   return;
yyL6:;

/* line 983 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 985 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((SymbolKind (yyP8->Input.Element -> UnmappedIdentifier . Sym) -> AnId . Place != NoTree))) goto yyL7;
  {
/* line 990 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int adr1=SymbolKind(yyP8->Input.Element->UnmappedIdentifier.Sym)->AnId.Place->Place.Expr->Expr.CV.Number;
    int adr2=PIC_GetSymbolAddress(SymbolKind(yyP8->Input.ExprList->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym));
    int size=PIC_Type_GetSize(yyP8->Input.Element->UnmappedIdentifier.SynType->Chan.Protocol->Simple.Type);
    int n;

    if (size==1)
      {
      if (adr1>=0x400) PIC_SelectBank1();
      PIC_CopyBit(adr2,adr1&0x3FF);
      if (adr1>=0x400) PIC_SelectBank0();
      }
    else
      {
      
      if (adr1>=0x80)
        PIC_SelectBank1();

      
      for(n=0; n<(size>>3); n++)
        PIC_CopyByte(adr2+n,(adr1+n)&0x7F);

      
      if (adr1>=0x80)
        PIC_SelectBank0();
      }
    }
  }
  }
   return;
yyL7:;

/* line 1018 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1020 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Input.SpecList);
/* line 1023 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   chan = PIC_GetChannelNumber (yyP8->Input.Element -> UnmappedIdentifier . Sym);
/* line 1024 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   adr = PIC_GetSymbolAddress (SymbolKind (yyP8->Input.ExprList->AnExpr.Expr -> ElementalExpr . Elemental -> UnmappedIdentifier . Sym));
/* line 1025 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   size = PIC_Type_GetSize (yyP8->Input.Element -> UnmappedIdentifier . SynType -> Chan . Protocol -> Simple . Type) >> 3;
/* line 1027 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (PIC_IsCurrentBlockUsed()==true)
      {
      PIC_ClearSchedulingBit(current_process);
      np=PIC_NewProcess();
      PIC_SetSchedulingBit(np);
      PIC_StartProcess2(np,"Input continuation");
      }
    }
/* line 1037 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CallChannelHandler (chan);
/* line 1038 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_TestWForZero ();
/* line 1039 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsClear ();
/* line 1040 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 1041 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreWIntoMemory (4);
/* line 1043 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;
    for(n=0; n<size; n++)
      {
      PIC_CopyByte(adr+n,0);
      if (n<(size-1))
        PIC_IncrementMemory(4);
      }
    }
/* line 1053 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Input.SpecList);
  }
   return;

  }
  }
  break;
  case kOutput:
  if (yyP8->Output.ExprList->Kind == kAnExpr) {
  if (yyP8->Output.ExprList->AnExpr.Next->Kind == kNoExpr) {
/* line 1055 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1057 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((SymbolKind (yyP8->Output.Element -> UnmappedIdentifier . Sym) -> AnId . Place != NoTree))) goto yyL9;
  {
/* line 1062 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int adr=SymbolKind(yyP8->Output.Element->UnmappedIdentifier.Sym)->AnId.Place->Place.Expr->Expr.CV.Number;
    int size=PIC_Type_GetSize(yyP8->Output.Element->UnmappedIdentifier.SynType->Chan.Protocol->Simple.Type);
    int n;

    if (size==1)
      {
      if (adr>=0x400) PIC_SelectBank1();
      yyP8->Output.ExprList->AnExpr.Expr->Expr.PIC_REG = adr & 0x3FF;
      PIC_Expr_Evaluate(yyP8->Output.ExprList->AnExpr.Expr);
      if (adr>=0x400) PIC_SelectBank0();
      }
    else
      {
      
      if (adr>=0x80)
        PIC_SelectBank1();

      
      yyP8->Output.ExprList->AnExpr.Expr->Expr.PIC_REG = adr & 0x7F;
      PIC_Expr_Evaluate(yyP8->Output.ExprList->AnExpr.Expr);

      
      if (adr>=0x80)
        PIC_SelectBank0();
      }
    }
  }
  }
   return;
yyL9:;

/* line 1090 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1092 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Output.SpecList);
/* line 1095 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   chan = PIC_GetChannelNumber (yyP8->Output.Element -> UnmappedIdentifier . Sym);
/* line 1096 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   size = PIC_Type_GetSize (yyP8->Output.ExprList->AnExpr.Expr -> Expr . SynType) >> 3;
/* line 1099 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   adr = PIC_AllocateBytes (size);
/* line 1100 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP8->Output.ExprList->AnExpr.Expr -> Expr . PIC_REG = adr;
/* line 1101 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP8->Output.ExprList->AnExpr.Expr);
/* line 1103 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 1104 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   np = PIC_AllocateUniqueBit ();
/* line 1105 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetSchedulingBit (np);
/* line 1106 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 1108 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StartProcess ("Output continuation");
/* line 1109 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_RegisterChannelOutput (chan, np, current_process, adr);
/* line 1110 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBytes (adr, size);
/* line 1112 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Output.SpecList);
  }
   return;

  }
  }
  break;
  case kCaseIn:
/* line 1114 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1115 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with CaseIn node", xxError, yyP8->CaseIn.Pos);
/* line 1116 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->CaseIn.SpecList);
/* line 1117 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->CaseIn.SpecList);
  }
   return;

  case kAfter:
/* line 1119 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1124 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->After.SpecList);
/* line 1126 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int adr;
    int process;

    
    adr = PIC_AllocateBytes(2);
    yyP8->After.Expr->Expr.PIC_REG = adr;
    PIC_Expr_Evaluate(yyP8->After.Expr);

    
    process=PIC_NewProcess();
    PIC_ClearSchedulingBit(current_process);
    PIC_SetSchedulingBit(process);

    
    PIC_StartProcess2(process,"check for expiration of timer AFTER");

    
    PIC_ClearBitInMemory((11*8)+5);

    
    PIC_ClearBitInMemory((4*8)+0);

    
    PIC_LoadMemoryIntoW(adr+0);
    PIC_SubtractWFromMemory(TimerAddressLSB);
    PIC_SkipIfZeroFlagIsSet();
    PIC_SetBitInMemory((4*8)+0);
    PIC_RotateMemoryLeft(4);

    
    PIC_LoadMemoryIntoW(adr+1);
    PIC_SubtractWFromMemory(TimerAddressMSB);
    PIC_SkipIfBitSet((4*8)+0);
    PIC_AddConstantToW(255);
    PIC_SkipIfZeroFlagIsSet();
    PIC_SetBitInMemory((4*8)+1);

    
    PIC_SetBitInMemory((11*8)+5);

    
    PIC_SkipIfBitSet((4*8)+1);
    PIC_ReturnToScheduler();

    
    PIC_ANDConstantIntoW(0x80);
    PIC_SkipIfZeroFlagIsSet();
    PIC_ReturnToScheduler();

    
    PIC_DeallocateBytes(adr,2);
    }
/* line 1180 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->After.SpecList);
  }
   return;

  case kSeq:
/* line 1182 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1183 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Seq.SpecList);
/* line 1184 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ProcList_Seq (yyP8->Seq.ProcList);
/* line 1185 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Seq.SpecList);
  }
   return;

  case kSeqRep:
/* line 1187 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1190 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_CheckForInvalidReplicators == true))) goto yyL14;
  {
/* line 1191 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->SeqRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL14;
  {
/* line 1192 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->SeqRep.Replicator -> Replicator . For -> Expr . CV . Number < 0))) goto yyL14;
  {
/* line 1193 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 1194 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
  }
  }
  }
   return;
yyL14:;

/* line 1196 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1200 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->SeqRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL15;
  {
/* line 1201 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->SeqRep.Replicator -> Replicator . From -> Expr . CV . Number == 0))) goto yyL15;
  {
/* line 1202 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->SeqRep.SpecList);
/* line 1203 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->SeqRep.SpecList);
  }
  }
  }
   return;
yyL15:;

/* line 1205 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1207 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    count_const = ((yyP8->SeqRep.Replicator->Replicator.For->Expr.C==true) && (yyP8->SeqRep.Replicator->Replicator.For->Expr.CV.Number<256)) ? true : false;

    PIC_SpecList_Allocate(yyP8->SeqRep.SpecList);

    
    index=PIC_AllocateBytes(2);

    
    
    
    count=PIC_AllocateBytes((count_const==true) ? 1 : 2);

    
    yyP8->SeqRep.Replicator->Replicator.For->Expr.PIC_REG=count;
    if (yyP8->SeqRep.Replicator->Replicator.For->Expr.C==true)
      PIC_Constant(count,yyP8->SeqRep.Replicator->Replicator.For->Expr.CV,yyP8->SeqRep.Replicator->Replicator.For->Expr.SynType);
    else
      PIC_Expr_Evaluate(yyP8->SeqRep.Replicator->Replicator.For);

    
    
    if ((PIC_CheckForInvalidReplicators==true) && (yyP8->SeqRep.Replicator->Replicator.For->Expr.C==false))
      {
      label1 = PIC_GetNewLabel();
      PIC_GotoLabelIfBitClear(((count+1)*8)+7,label1);
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      PIC_OutputLabel(label1);
      }

    
    
    label1=PIC_GetNewLabel();
    if (yyP8->SeqRep.Replicator->Replicator.For->Expr.C==false)
      {
      PIC_LoadMemoryIntoW(count+0);
      PIC_ORMemoryIntoW(count+1);
      PIC_SkipIfZeroFlagIsClear();
      PIC_GotoLabel(label1,"loop count is zero, so skip over loop body");
      }

    
    yyP8->SeqRep.Replicator->Replicator.From->Expr.PIC_REG=index;
    if (yyP8->SeqRep.Replicator->Replicator.From->Expr.C==true)
      PIC_Constant(index,yyP8->SeqRep.Replicator->Replicator.From->Expr.CV,yyP8->SeqRep.Replicator->Replicator.From->Expr.SynType);
    else
      PIC_Expr_Evaluate(yyP8->SeqRep.Replicator->Replicator.From);

    
    label2=PIC_GetNewLabel();
    PIC_OutputLabel(label2);

    
    PIC_Proc(yyP8->SeqRep.Process);

    
    PIC_IncrementMemory(index);
    PIC_SkipIfZeroFlagIsClear();
    PIC_IncrementMemory(index+1);

    
    
    if (count_const==true)
      {
      PIC_DecrementMemoryAndSkipIfZero(count);
      PIC_GotoLabel(label2,"loop count not Zero, so loop again");
      }
    else
      {
      PIC_DecrementMemory(count);
      PIC_IncrementMemoryToW(count);
      PIC_SkipIfZeroFlagIsClear();
      PIC_DecrementMemory(count+1);
      PIC_ORMemoryIntoW(count+1);
      PIC_SkipIfZeroFlagIsSet();
      PIC_GotoLabel(label2,"loop again");
      }

    
    PIC_OutputLabel(label1);

    
    PIC_DeallocateBytes(count,(count_const==true) ? 1 : 2);
    PIC_DeallocateBytes(index,2);

    PIC_SpecList_Deallocate(yyP8->SeqRep.SpecList);
    }
  }
   return;

  case kPar:
/* line 1297 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1299 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Par.SpecList);
/* line 1302 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   adr = PIC_AllocateBytes (1);
/* line 1303 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory (PIC_ProcList_Par1 (yyP8->Par.ProcList), adr);
/* line 1305 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 1306 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   np = PIC_NewProcess ();
/* line 1307 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetSchedulingBit (np);
/* line 1308 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 1310 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ProcList_Par2 (adr, yyP8->Par.ProcList);
/* line 1313 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StartProcess2 (np, "Par - wait for children to finish");
/* line 1314 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_LoadMemoryIntoW (adr);
/* line 1315 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsSet ();
/* line 1316 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 1318 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBytes (adr, 1);
/* line 1319 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Par.SpecList);
  }
   return;

  case kParRep:
/* line 1321 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1324 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_CheckForInvalidReplicators == true))) goto yyL18;
  {
/* line 1325 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->ParRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL18;
  {
/* line 1326 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->ParRep.Replicator -> Replicator . For -> Expr . CV . Number < 0))) goto yyL18;
  {
/* line 1327 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 1328 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
  }
  }
  }
   return;
yyL18:;

/* line 1330 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1334 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->ParRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL19;
  {
/* line 1335 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->ParRep.Replicator -> Replicator . From -> Expr . CV . Number == 0))) goto yyL19;
  {
/* line 1336 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->ParRep.SpecList);
/* line 1337 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->ParRep.SpecList);
  }
  }
  }
   return;
yyL19:;

/* line 1339 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1342 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: replicated PAR is not supported", xxRestriction, yyP8->ParRep.Pos);
/* line 1343 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->ParRep.SpecList);
/* line 1344 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->ParRep.SpecList);
  }
   return;

  case kIf:
/* line 1346 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1347 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->If.SpecList);
/* line 1348 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   label1 = PIC_GetNewLabel ();
/* line 1349 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IfChoiceList (label1, yyP8->If.ChoiceList);
/* line 1350 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->If.SpecList);
/* line 1353 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (label1);
  }
   return;

  case kIfRep:
/* line 1355 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1358 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_CheckForInvalidReplicators == true))) goto yyL22;
  {
/* line 1359 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->IfRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL22;
  {
/* line 1360 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->IfRep.Replicator -> Replicator . For -> Expr . CV . Number < 0))) goto yyL22;
  {
/* line 1361 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 1362 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
  }
  }
  }
   return;
yyL22:;

/* line 1364 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1368 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->IfRep.Replicator -> Replicator . For -> Expr . C == true))) goto yyL23;
  {
/* line 1369 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->IfRep.Replicator -> Replicator . From -> Expr . CV . Number == 0))) goto yyL23;
  {
/* line 1370 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->IfRep.SpecList);
/* line 1371 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->IfRep.SpecList);
  }
  }
  }
   return;
yyL23:;

/* line 1373 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1375 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: replicated IF is not supported", xxRestriction, yyP8->IfRep.Pos);
/* line 1376 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->IfRep.SpecList);
/* line 1377 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->IfRep.SpecList);
  }
   return;

  case kCase:
/* line 1379 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1380 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Case.SpecList);
/* line 1381 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   label1 = PIC_GetNewLabel ();
/* line 1382 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   adr = PIC_AllocateMemory (yyP8->Case.Selector -> Expr . SynType);
/* line 1383 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP8->Case.Selector -> Expr . PIC_REG = adr;
/* line 1384 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP8->Case.Selector);
/* line 1385 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CaseChoiceList (label1, yyP8->Case.Selector, yyP8->Case.ChoiceList);
/* line 1386 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateMemory (adr, yyP8->Case.Selector -> Expr . SynType);
/* line 1387 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Case.SpecList);
/* line 1390 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (label1);
  }
   return;

  case kWhile:
/* line 1392 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1394 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->While.Expr -> Expr . C == true))) goto yyL26;
  {
/* line 1395 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->While.Expr -> Expr . CV . Number == true))) goto yyL26;
  {
/* line 1397 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->While.SpecList);
/* line 1398 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (PIC_IsCurrentBlockUsed()==true)
      {
      np=PIC_NewProcess();
      PIC_ClearSchedulingBit(current_process);
      PIC_SetSchedulingBit(np);
      PIC_StartProcess2(np,"WHILE - start of loop");
      }
    }
/* line 1408 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   np = current_process;
/* line 1409 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP8->While.Process);
/* line 1410 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (current_process!=np)
      {
      PIC_ClearSchedulingBit(current_process);
      PIC_SetSchedulingBit(np);
      }
    }
/* line 1417 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->While.SpecList);
/* line 1418 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
  }
  }
   return;
yyL26:;

/* line 1420 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1424 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->While.Expr -> Expr . C == true))) goto yyL27;
  {
/* line 1425 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP8->While.Expr -> Expr . CV . Number == false))) goto yyL27;
  {
/* line 1426 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->While.SpecList);
/* line 1427 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->While.SpecList);
  }
  }
  }
   return;
yyL27:;

/* line 1429 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1431 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->While.SpecList);
/* line 1433 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (PIC_IsCurrentBlockUsed()==true)
      {
      np=PIC_NewProcess();
      PIC_ClearSchedulingBit(current_process);
      PIC_SetSchedulingBit(np);
      PIC_StartProcess2(np,"WHILE - start of loop");
      }
    }
/* line 1444 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP8->While.Expr -> Expr . PIC_REG = PIC_AllocateBit ();
/* line 1445 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP8->While.Expr);
/* line 1448 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   label1 = PIC_GetNewLabel ();
/* line 1449 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet (yyP8->While.Expr -> Expr . PIC_REG);
/* line 1450 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (label1, "exit from the loop");
/* line 1451 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBit (yyP8->While.Expr -> Expr . PIC_REG);
/* line 1454 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   np = current_process;
/* line 1455 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP8->While.Process);
/* line 1456 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (current_process!=np)
      {
      PIC_ClearSchedulingBit(current_process);
      PIC_SetSchedulingBit(np);
      }
    }
/* line 1463 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 1466 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   current_process = np;
/* line 1467 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (label1);
/* line 1468 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->While.SpecList);
  }
   return;

  case kSkip:
/* line 1470 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1472 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Skip.SpecList);
/* line 1473 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Skip.SpecList);
  }
   return;

  case kStop:
/* line 1475 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1477 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Stop.SpecList);
/* line 1478 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Stop.SpecList);
/* line 1479 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 1480 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
   return;

  case kCCode:
/* line 1482 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1483 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with CCode node", xxError, yyP8->CCode.Pos);
/* line 1484 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->CCode.SpecList);
/* line 1485 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->CCode.SpecList);
  }
   return;

  case kInstance:
/* line 1487 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1488 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->Instance.SpecList);
/* line 1489 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (PIC_IsCurrentBlockUsed()==true)
      {
      np=PIC_NewProcess();
      PIC_ClearSchedulingBit(current_process);
      PIC_SetSchedulingBit(np);
      PIC_StartProcess2(np,"Instance - wait");
      }
    }
/* line 1499 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   tr = SymbolKind (yyP8->Instance.Sym);
/* line 1500 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   np = PIC_NewProcess ();
/* line 1501 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_LoadMemoryIntoW (tr -> DefProc . PIC_R_BIT);
/* line 1502 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsSet ();
/* line 1503 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 1504 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreDynamicProcessId (np, tr -> DefProc . PIC_R_BYTE, tr -> DefProc . PIC_R_BIT);
/* line 1505 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetSchedulingBit (tr -> DefProc . PIC_PRC);
/* line 1506 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 1507 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 1509 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StartProcess2 (np, "Instance - continuation");
/* line 1510 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->Instance.SpecList);
  }
   return;

  case kNullProcess:
/* line 1512 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1514 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP8->NullProcess.SpecList);
/* line 1515 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP8->NullProcess.SpecList);
  }
   return;

  }

 yyAbort ("PIC_Proc");
;
}

static void PIC_SpecList_Allocate
# if defined __STDC__ | defined __cplusplus
(register tTree yyP9)
# else
(yyP9)
 register tTree yyP9;
# endif
{
  if (yyP9->Kind == kASpec) {
/* line 1521 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1522 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Spec_Allocate (yyP9->ASpec.Spec);
/* line 1523 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP9->ASpec.Next);
  }
   return;

  }
  if (yyP9->Kind == kNoSpec) {
/* line 1525 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
 yyAbort ("PIC_SpecList_Allocate");
;
}

static void PIC_Spec_Allocate
# if defined __STDC__ | defined __cplusplus
(register tTree yyP10)
# else
(yyP10)
 register tTree yyP10;
# endif
{
/* line 1531 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int pr1; int pr2; int np; int label; unsigned int old_process; int oldW; unsigned int instr_count_delta;

  switch (yyP10->Kind) {
  case kIs:
/* line 1532 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kRetypes:
  case kReshapes:
/* line 1534 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kVal:
/* line 1536 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kValRetypes:
  case kValReshapes:
/* line 1538 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kDefProc:
  case kInlineDefProc:
/* line 1540 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1542 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP10 -> DefProc . PIC_R_BYTE = PIC_AllocateBytes (1);
/* line 1543 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP10 -> DefProc . PIC_R_BIT = PIC_AllocateBytes (1);
/* line 1544 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory (0, yyP10 -> DefProc . PIC_R_BIT);
/* line 1547 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   old_process = current_process;
/* line 1548 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   oldW = currentW;
/* line 1549 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count_delta = instr_count - last_instr_count;
/* line 1550 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   label = PIC_GetNewLabel ();
/* line 1551 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (label, "continue with the current block");
/* line 1554 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StartProcess ("DefProc - contents");
/* line 1555 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP10 -> DefProc . PIC_PRC = current_process;
/* line 1556 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP10->DefProc.Process);
/* line 1557 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 1558 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetSchedulingBit_Dynamic (yyP10 -> DefProc . PIC_R_BYTE, yyP10 -> DefProc . PIC_R_BIT);
/* line 1559 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory (0, yyP10 -> DefProc . PIC_R_BIT);
/* line 1560 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
/* line 1563 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   current_process = old_process;
/* line 1564 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (label);
/* line 1565 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = oldW;
/* line 1566 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_count = instr_count - instr_count_delta;
  }
   return;

  case kDefIsFunc:
  case kInlineDefIsFunc:
/* line 1568 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1569 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefIsFunc node", xxError, yyP10->DefIsFunc.Pos);
  }
   return;

  case kDefFunc:
  case kInlineDefFunc:
/* line 1571 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1572 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefFunc node", xxError, yyP10->DefFunc.Pos);
  }
   return;

  case kPrototypeProc:
/* line 1574 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1575 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with PrototypeProc node", xxError, yyP10->PrototypeProc.Pos);
  }
   return;

  case kPrototypeFunc:
/* line 1577 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1578 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with PrototypeFunc node", xxError, yyP10->PrototypeFunc.Pos);
  }
   return;

  case kDeclType:
/* line 1580 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1581 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DeclType node", xxError, yyP10->DeclType.Pos);
  }
   return;

  case kDecl:
/* line 1583 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1584 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IdList_AllocateMemory (yyP10->Decl.IdList, yyP10->Decl.Type);
  }
   return;

  case kDefSeqPro:
/* line 1586 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1587 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefSeqPro node", xxError, yyP10->DefSeqPro.Pos);
  }
   return;

  case kDefTagPro:
/* line 1589 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1590 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefTagPro node", xxError, yyP10->DefTagPro.Pos);
  }
   return;

  case kHCode:
/* line 1592 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1593 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with HCode node", xxError, yyP10->HCode.Pos);
  }
   return;

  case kPlace:
/* line 1595 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1597 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((SymbolKind (yyP10->Place.Sym) -> AnId . Place != NULL))) goto yyL15;
  {
/* line 1598 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: identifier already has a PLACE", xxError, yyP10->Place.Pos);
  }
  }
   return;
yyL15:;

/* line 1600 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1601 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP10->Place.Expr -> Expr . C == false))) goto yyL16;
  {
/* line 1602 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: PLACE expression is not constant", xxError, yyP10->Place.Pos);
  }
  }
   return;
yyL16:;

/* line 1604 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1606 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetPrimitiveSize (SymbolType (yyP10->Place.Sym)) == 1))) goto yyL17;
  {
/* line 1607 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (((yyP10->Place.Expr -> Expr . CV . Number < 0) || (yyP10->Place.Expr -> Expr . CV . Number > 2047)))) goto yyL17;
  {
/* line 1608 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: PLACE expression is out of range (0x00-0x7FF) for a bit address", xxError, yyP10->Place.Pos);
  }
  }
  }
   return;
yyL17:;

/* line 1610 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1612 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetPrimitiveSize (SymbolType (yyP10->Place.Sym)) != 1))) goto yyL18;
  {
/* line 1613 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (((yyP10->Place.Expr -> Expr . CV . Number < 0) || (yyP10->Place.Expr -> Expr . CV . Number > 255)))) goto yyL18;
  {
/* line 1614 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: PLACE expression is out of range (0x00-0xFF) for a byte address", xxError, yyP10->Place.Pos);
  }
  }
  }
   return;
yyL18:;

/* line 1616 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1618 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (SymbolKind (yyP10->Place.Sym) -> AnId . Place = yyP10)) goto yyL19;
  }
   return;
yyL19:;

  break;
  case kNullSpec:
/* line 1620 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }

/* line 1622 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

;
}

static void PIC_SpecList_Deallocate
# if defined __STDC__ | defined __cplusplus
(register tTree yyP11)
# else
(yyP11)
 register tTree yyP11;
# endif
{
  if (yyP11->Kind == kASpec) {
/* line 1628 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1629 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Spec_Deallocate (yyP11->ASpec.Spec);
/* line 1630 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP11->ASpec.Next);
  }
   return;

  }
  if (yyP11->Kind == kNoSpec) {
/* line 1632 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
 yyAbort ("PIC_SpecList_Deallocate");
;
}

static void PIC_Spec_Deallocate
# if defined __STDC__ | defined __cplusplus
(register tTree yyP12)
# else
(yyP12)
 register tTree yyP12;
# endif
{
/* line 1638 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int pr1; int pr2;

  switch (yyP12->Kind) {
  case kIs:
/* line 1639 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kRetypes:
  case kReshapes:
/* line 1641 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kVal:
/* line 1643 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kValRetypes:
  case kValReshapes:
/* line 1645 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kDefProc:
  case kInlineDefProc:
/* line 1647 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1648 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBytes (1, yyP12 -> DefProc . PIC_R_BYTE);
/* line 1649 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBytes (1, yyP12 -> DefProc . PIC_R_BIT);
  }
   return;

  case kDefIsFunc:
  case kInlineDefIsFunc:
/* line 1651 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1652 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefIsFunc node", xxError, yyP12->DefIsFunc.Pos);
  }
   return;

  case kDefFunc:
  case kInlineDefFunc:
/* line 1654 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1655 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefFunc node", xxError, yyP12->DefFunc.Pos);
  }
   return;

  case kPrototypeProc:
/* line 1657 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1658 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with PrototypeProc node", xxError, yyP12->PrototypeProc.Pos);
  }
   return;

  case kPrototypeFunc:
/* line 1660 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1661 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with PrototypeFunc node", xxError, yyP12->PrototypeFunc.Pos);
  }
   return;

  case kDeclType:
/* line 1663 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kDecl:
/* line 1665 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1666 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IdList_DeallocateMemory (yyP12->Decl.IdList, yyP12->Decl.Type);
  }
   return;

  case kNullSpec:
/* line 1668 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  case kDefSeqPro:
/* line 1670 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1671 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefSeqPro node", xxError, yyP12->DefSeqPro.Pos);
  }
   return;

  case kDefTagPro:
/* line 1673 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1674 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with DefTagPro node", xxError, yyP12->DefTagPro.Pos);
  }
   return;

  case kHCode:
/* line 1676 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1677 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with HCode node", xxError, yyP12->HCode.Pos);
  }
   return;

  }

/* line 1679 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

;
}

static void PIC_IdList_AllocateMemory
# if defined __STDC__ | defined __cplusplus
(register tTree yyP14, register tTree yyP13)
# else
(yyP14, yyP13)
 register tTree yyP14;
 register tTree yyP13;
# endif
{
  if (yyP14->Kind == kAnId) {
  if (yyP13->Kind == kChan) {
/* line 1689 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1691 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IdList_AllocateMemory (yyP14->AnId.Next, yyP13);
  }
   return;

  }
  if (yyP13->Kind == kTimer) {
/* line 1693 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1694 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((IsTimerUsedFlag == true))) goto yyL2;
  }
   return;
yyL2:;

/* line 1697 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1698 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   IsTimerUsedFlag = true;
/* line 1699 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   TimerAddressLSB = PIC_AllocateBytes (1);
/* line 1700 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   TimerAddressMSB = PIC_AllocateBytes (1);
  }
   return;

  }
/* line 1702 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1703 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP14 -> AnId . PIC_REG = PIC_AllocateMemory (yyP13);
/* line 1704 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IdList_AllocateMemory (yyP14->AnId.Next, yyP13);
  }
   return;

  }
  if (yyP14->Kind == kNoId) {
/* line 1706 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
 yyAbort ("PIC_IdList_AllocateMemory");
;
}

static int PIC_AllocateMemory
# if defined __STDC__ | defined __cplusplus
(register tTree yyP15)
# else
(yyP15)
 register tTree yyP15;
# endif
{
  if (Tree_IsType (yyP15, kBool)) {
/* line 1714 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return PIC_AllocateBit ();

  }
/* line 1715 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return PIC_AllocateBytes (PIC_Type_GetSize (yyP15) >> 3);

}

static void PIC_IdList_DeallocateMemory
# if defined __STDC__ | defined __cplusplus
(register tTree yyP17, register tTree yyP16)
# else
(yyP17, yyP16)
 register tTree yyP17;
 register tTree yyP16;
# endif
{
  if (yyP17->Kind == kAnId) {
  if (yyP16->Kind == kChan) {
/* line 1720 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1722 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IdList_DeallocateMemory (yyP17->AnId.Next, yyP16);
  }
   return;

  }
  if (yyP16->Kind == kTimer) {
/* line 1724 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
/* line 1727 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1728 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IdList_DeallocateMemory (yyP17->AnId.Next, yyP16);
/* line 1729 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateMemory (yyP17 -> AnId . PIC_REG, yyP16);
  }
   return;

  }
  if (yyP17->Kind == kNoId) {
/* line 1731 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
 yyAbort ("PIC_IdList_DeallocateMemory");
;
}

static void PIC_DeallocateMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP19, register tTree yyP18)
# else
(yyP19, yyP18)
 register int yyP19;
 register tTree yyP18;
# endif
{
  if (Tree_IsType (yyP18, kBool)) {
/* line 1737 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1737 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBit (yyP19);
  }
   return;

  }
/* line 1738 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1738 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBytes (yyP19, PIC_Type_GetSize (yyP18) >> 3);
  }
   return;

;
}

static int PIC_Type_GetPrimitiveSize
# if defined __STDC__ | defined __cplusplus
(register tTree yyP20)
# else
(yyP20)
 register tTree yyP20;
# endif
{

  switch (yyP20->Kind) {
  case kBool:
  case kUserBool:
/* line 1745 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 1;

  case kByte:
  case kUserByte:
/* line 1746 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 8;

  case kInt:
  case kUserInt:
/* line 1747 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 16;

  case kInt16:
  case kUserInt16:
/* line 1748 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 16;

  case kInt32:
  case kUserInt32:
/* line 1749 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 32;

  case kInt64:
  case kUserInt64:
/* line 1750 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 64;

  case kUntypedInt:
/* line 1751 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 16;

  case kReal32:
  case kUserReal32:
/* line 1752 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 32;

  case kReal64:
  case kUserReal64:
/* line 1753 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return 64;

  case kChan:
/* line 1754 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return PIC_Type_GetPrimitiveSize (yyP20->Chan.Protocol -> Simple . Type);

  case kArray:
  case kUserArray:
/* line 1755 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return PIC_Type_GetPrimitiveSize (yyP20->Array.Type);

  }

/* line 1756 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1756 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   WriteTree (stdout, yyP20);
/* line 1756 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: unknown type detected", xxError, NoPosition);
/* line 1756 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   goto yyL12;
  }
yyL12:;

 yyAbort ("PIC_Type_GetPrimitiveSize");
}

static int PIC_Type_GetSize
# if defined __STDC__ | defined __cplusplus
(register tTree yyP21)
# else
(yyP21)
 register tTree yyP21;
# endif
{
  if (Tree_IsType (yyP21, kArray)) {
/* line 1762 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1763 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP21->Array.Expr -> Expr . C == true))) goto yyL1;
  }
   return (yyP21->Array.Expr -> Expr . CV . Number * PIC_Type_GetSize (yyP21->Array.Type));
yyL1:;

  }
/* line 1766 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return PIC_Type_GetPrimitiveSize (yyP21);

}

static void PIC_Assign
# if defined __STDC__ | defined __cplusplus
(register tTree yyP23, register tTree yyP22)
# else
(yyP23, yyP22)
 register tTree yyP23;
 register tTree yyP22;
# endif
{
  if (yyP22->Kind == kAnExpr) {
  if (yyP22->AnExpr.Expr->Kind == kValof) {
/* line 1774 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1777 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP22->AnExpr.Expr->Valof.SpecList);
/* line 1778 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP22->AnExpr.Expr->Valof.Process);
/* line 1779 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Assign (yyP23, yyP22->AnExpr.Expr->Valof.ExprList);
/* line 1780 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP22->AnExpr.Expr->Valof.SpecList);
  }
   return;

  }
  }
  if (yyP23->Kind == kAnExpr) {
  if (yyP23->AnExpr.Next->Kind == kNoExpr) {
  if (yyP23->AnExpr.Expr->Kind == kElementalExpr) {
  if (yyP23->AnExpr.Expr->ElementalExpr.Elemental->Kind == kUnmappedIdentifier) {
  if (yyP22->Kind == kAnExpr) {
  if (yyP22->AnExpr.Next->Kind == kNoExpr) {
/* line 1782 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1785 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP22->AnExpr.Expr -> Expr . PIC_REG = PIC_GetSymbolAddress (SymbolKind (yyP23->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym));
/* line 1786 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP22->AnExpr.Expr);
  }
   return;

  }
  }
  }
  }
  }
  if (yyP23->AnExpr.Expr->Kind == kElementalExpr) {
  if (yyP23->AnExpr.Expr->ElementalExpr.Elemental->Kind == kUnmappedIdentifier) {
  if (yyP22->Kind == kAnExpr) {
/* line 1788 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1791 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP22->AnExpr.Expr -> Expr . C == true))) goto yyL3;
  {
/* line 1794 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Assign (yyP23->AnExpr.Next, yyP22->AnExpr.Next);
/* line 1797 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP22->AnExpr.Expr -> Expr . PIC_REG = PIC_GetSymbolAddress (SymbolKind (yyP23->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym));
/* line 1798 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP22->AnExpr.Expr);
  }
  }
   return;
yyL3:;

/* line 1800 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1802 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP22->AnExpr.Expr -> Expr . SynType) == 1))) goto yyL4;
  {
/* line 1805 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP22->AnExpr.Expr -> Expr . PIC_REG = PIC_AllocateBit ();
/* line 1806 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP22->AnExpr.Expr);
/* line 1809 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Assign (yyP23->AnExpr.Next, yyP22->AnExpr.Next);
/* line 1812 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CopyBit (PIC_GetSymbolAddress (SymbolKind (yyP23->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym)), yyP22->AnExpr.Expr -> Expr . PIC_REG);
/* line 1815 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBit (yyP22->AnExpr.Expr -> Expr . PIC_REG);
  }
  }
   return;
yyL4:;

/* line 1817 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1819 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP22->AnExpr.Expr->Expr.SynType) >> 3;
    int adr = PIC_GetSymbolAddress(SymbolKind(yyP23->AnExpr.Expr->ElementalExpr.Elemental->UnmappedIdentifier.Sym));
    int n;

    
    yyP22->AnExpr.Expr->Expr.PIC_REG=PIC_AllocateBytes(size);
    PIC_Expr_Evaluate(yyP22->AnExpr.Expr);

    
    PIC_Assign(yyP23->AnExpr.Next,yyP22->AnExpr.Next);

    
    for(n=0; n<size; n++)
      PIC_CopyByte(adr+n,yyP22->AnExpr.Expr->Expr.PIC_REG+n);

    
    PIC_DeallocateBytes(yyP22->AnExpr.Expr->Expr.PIC_REG,size);
    }
  }
   return;

  }
  }
  }
  }
  if (yyP23->Kind == kNoExpr) {
  if (yyP22->Kind == kNoExpr) {
/* line 1839 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
  }
 yyAbort ("PIC_Assign");
;
}

static void PIC_Expr_Evaluate
# if defined __STDC__ | defined __cplusplus
(register tTree yyP24)
# else
(yyP24)
 register tTree yyP24;
# endif
{
/* line 1844 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1845 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP24 -> Expr . PIC_REG != InvalidProcess))) goto yyL1;
  {
/* line 1846 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Expr_Evaluate2(yyP24);
    if (size!=0) PIC_DeallocateBits(yyP24->Expr.PIC_REG,size);
    }
  }
  }
   return;
yyL1:;

/* line 1851 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1852 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: no place to store result during expression evaluation", xxError, yyP24 -> Expr . Pos);
  }
   return;

;
}

static int PIC_Expr_Evaluate2
# if defined __STDC__ | defined __cplusplus
(register tTree yyP25)
# else
(yyP25)
 register tTree yyP25;
# endif
{
/* line 1861 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int result;
/* line 1862 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1864 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP25 -> Expr . C == true))) goto yyL1;
  {
/* line 1865 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    result=0;
    if (yyP25->Expr.PIC_REG==InvalidProcess)
      {
      result=PIC_Type_GetSize(yyP25->Expr.SynType);
      yyP25->Expr.PIC_REG=PIC_AllocateMemory(yyP25->Expr.SynType);
      }

    
    PIC_Constant(yyP25->Expr.PIC_REG,yyP25->Expr.CV,yyP25->Expr.SynType);
    }
  }
  }
   return result;
yyL1:;


  switch (yyP25->Kind) {
  case kBinary:
/* line 1879 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1880 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    int size1=PIC_Expr_Evaluate2(yyP25->Binary.Lop);
    int size2=PIC_Expr_Evaluate2(yyP25->Binary.Rop);

    
    result = 0;
    if (yyP25->Expr.PIC_REG==InvalidProcess)
      {
      result=PIC_Type_GetSize(yyP25->Binary.SynType);
      yyP25->Expr.PIC_REG=PIC_AllocateMemory(yyP25->Binary.SynType);
      }

    
    PIC_EvaluateBinary(yyP25,yyP25->Binary.Lop,yyP25->Binary.Rop,yyP25->Binary.Op);

    
    if (size1!=0) PIC_DeallocateBits(yyP25->Binary.Lop->Expr.PIC_REG,size1);
    if (size2!=0) PIC_DeallocateBits(yyP25->Binary.Rop->Expr.PIC_REG,size2);
    }
  }
   return result;

  case kUnary:
/* line 1902 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1903 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    int size=PIC_Expr_Evaluate2(yyP25->Unary.Expr);

    
    result = 0;
    if (yyP25->Expr.PIC_REG==InvalidProcess)
      {
      result=PIC_Type_GetSize(yyP25->Unary.SynType);
      yyP25->Expr.PIC_REG=PIC_AllocateMemory(yyP25->Unary.SynType);
      }

    
    PIC_EvaluateUnary(yyP25,yyP25->Unary.Expr,yyP25->Unary.Op);

    
    if (size!=0) PIC_DeallocateBits(yyP25->Unary.Expr->Expr.PIC_REG,size);
    }
  }
   return result;

  case kValof:
  if (yyP25->Valof.ExprList->Kind == kAnExpr) {
  if (yyP25->Valof.ExprList->AnExpr.Next->Kind == kNoExpr) {
/* line 1923 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1925 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP25->Valof.SpecList);
/* line 1926 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP25->Valof.Process);
/* line 1927 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP25->Valof.ExprList->AnExpr.Expr -> Expr . PIC_REG = yyP25 -> Valof . PIC_REG;
/* line 1928 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   result = PIC_Expr_Evaluate2 (yyP25->Valof.ExprList->AnExpr.Expr);
/* line 1929 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP25->Valof.SpecList);
  }
   return result;

  }
  }
  break;
  case kFuncCall:
/* line 1932 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1933 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: function calls are not supported", xxError, yyP25->FuncCall.Pos);
  }
   return 0;

  case kConvert:
/* line 1936 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1938 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP25->Convert.Type) == PIC_Type_GetSize (yyP25->Convert.Expr -> Expr . SynType)))) goto yyL6;
  {
/* line 1939 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP25->Convert.Expr -> Expr . PIC_REG = yyP25 -> Convert . PIC_REG;
/* line 1940 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   result = PIC_Expr_Evaluate2 (yyP25->Convert.Expr);
  }
  }
   return result;
yyL6:;

/* line 1943 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1946 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP25->Convert.Type) > 1))) goto yyL7;
  {
/* line 1947 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP25->Convert.Expr -> Expr . SynType) == 1))) goto yyL7;
  {
/* line 1948 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;

    
    int size=PIC_Expr_Evaluate2(yyP25->Convert.Expr);

    
    result=0;
    if (yyP25->Convert.PIC_REG==InvalidProcess)
      {
      result=PIC_Type_GetSize(yyP25->Convert.SynType);
      yyP25->Convert.PIC_REG=PIC_AllocateMemory(yyP25->Convert.SynType);
      }

    
    for(n=0; n<(PIC_Type_GetSize(yyP25->Expr.SynType)>>3); n++)
      PIC_StoreConstantIntoMemory(0x00,yyP25->Expr.PIC_REG+n);

    
    PIC_SkipIfBitClear(yyP25->Convert.Expr->Expr.PIC_REG);
    PIC_SetBitInMemory(yyP25->Expr.PIC_REG*8);

    
    if (size!=0) PIC_DeallocateBits(yyP25->Convert.Expr->Expr.PIC_REG,size);
    }
  }
  }
  }
   return result;
yyL7:;

/* line 1975 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 1978 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP25->Convert.Type) == 1))) goto yyL8;
  {
/* line 1979 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP25->Convert.Expr -> Expr . SynType) > 1))) goto yyL8;
  {
/* line 1980 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    int size=PIC_Expr_Evaluate2(yyP25->Convert.Expr);

    
    result=0;
    if (yyP25->Convert.PIC_REG==InvalidProcess)
      {
      result=PIC_Type_GetSize(yyP25->Convert.SynType);
      yyP25->Convert.PIC_REG=PIC_AllocateMemory(yyP25->Convert.SynType);
      }

    
    if (PIC_CheckForInvalidConversions==true)
      {
      int label = PIC_GetNewLabel();
      int size = PIC_Type_GetSize(yyP25->Convert.Expr->Expr.SynType) >> 3;
      int n;

      
      PIC_LoadMemoryIntoW(yyP25->Convert.Expr->Expr.PIC_REG);
      PIC_ANDConstantIntoW(0xFE);

      
      for(n=1; n<size; n++)
        PIC_ORMemoryIntoW(yyP25->Convert.Expr->Expr.PIC_REG+n);

      
      
      PIC_SkipIfZeroFlagIsSet();
      PIC_GotoLabel(label,"the conversion is valid, so continue");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      PIC_OutputLabel(label);
      }

    
    PIC_CopyBit(yyP25->Expr.PIC_REG,yyP25->Convert.Expr->Expr.PIC_REG*8);

    
    if (size!=0) PIC_DeallocateBits(yyP25->Convert.Expr->Expr.PIC_REG,size);
    }
  }
  }
  }
   return result;
yyL8:;

/* line 2024 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2027 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP25->Convert.Type) < PIC_Type_GetSize (yyP25->Convert.Expr -> Expr . SynType)))) goto yyL9;
  {
/* line 2028 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;

    
    int size=PIC_Expr_Evaluate2(yyP25->Convert.Expr);

    
    result=0;
    if (yyP25->Convert.PIC_REG==InvalidProcess)
      {
      result=PIC_Type_GetSize(yyP25->Convert.SynType);
      yyP25->Convert.PIC_REG=PIC_AllocateMemory(yyP25->Convert.SynType);
      }

    if (PIC_CheckForInvalidConversions==true)
      {
      int size_s = PIC_Type_GetSize(yyP25->Convert.Expr->Expr.SynType) >> 3;
      int size_d = PIC_Type_GetSize(yyP25->Convert.Type) >> 3;
      int neg = PIC_GetNewLabel();
      int cont = PIC_GetNewLabel();
      int n;

      
      PIC_SkipIfBitClear(((yyP25->Convert.Expr->Expr.PIC_REG+size_d-1)*8)+7);
      PIC_GotoLabel(neg,"check the sign bits are all set");

      
      if ((size_s-size_d)==1)
        PIC_TestMemoryForZero(yyP25->Convert.Expr->Expr.PIC_REG+size_d);
      else
        {
        PIC_LoadMemoryIntoW(yyP25->Convert.Expr->Expr.PIC_REG+size_d);
        for(n=1; n<(size_s-size_d); n++)
          PIC_ORMemoryIntoW(yyP25->Convert.Expr->Expr.PIC_REG+size_d+n);
        }
      PIC_SkipIfZeroFlagIsClear();
      PIC_GotoLabel(cont,"sign bits are all zero, so conversion is valid");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();

      
      PIC_OutputLabel(neg);
      PIC_LoadMemoryIntoW(yyP25->Convert.Expr->Expr.PIC_REG+size_d);
      for(n=1; n<(size_s-size_d); n++)
        PIC_ANDMemoryIntoW(yyP25->Convert.Expr->Expr.PIC_REG+size_d+n);
      PIC_XORConstantIntoW(0xFF);
      PIC_SkipIfZeroFlagIsClear();
      PIC_GotoLabel(cont,"sign bits are all ones, so conversion is valid");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();

      PIC_OutputLabel(cont);
      }

    
    for(n=0; n<(PIC_Type_GetSize(yyP25->Convert.Type)>>3); n++)
      PIC_CopyByte(yyP25->Expr.PIC_REG+n,yyP25->Convert.Expr->Expr.PIC_REG+n);

    
    if (size!=0) PIC_DeallocateBits(yyP25->Convert.Expr->Expr.PIC_REG,size);
    }
  }
  }
   return result;
yyL9:;

/* line 2091 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2095 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP25->Convert.Type) > PIC_Type_GetSize (yyP25->Convert.Expr -> Expr . SynType)))) goto yyL10;
  {
/* line 2096 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;

    
    int size=PIC_Expr_Evaluate2(yyP25->Convert.Expr);

    
    result=0;
    if (yyP25->Convert.PIC_REG==InvalidProcess)
      {
      result=PIC_Type_GetSize(yyP25->Convert.SynType);
      yyP25->Convert.PIC_REG=PIC_AllocateMemory(yyP25->Convert.SynType);
      }

    
    for(n=0; n<(PIC_Type_GetSize(yyP25->Convert.Expr->Expr.SynType)>>3); n++)
      PIC_CopyByte(yyP25->Expr.PIC_REG+n,yyP25->Convert.Expr->Expr.PIC_REG+n);

    
    PIC_LoadConstantIntoW(0x00);
    PIC_SkipIfBitClear((yyP25->Expr.PIC_REG*8)+PIC_Type_GetSize(yyP25->Convert.Expr->Expr.SynType)-1);
    PIC_LoadConstantIntoW(0xFF);

    
    for(n=PIC_Type_GetSize(yyP25->Convert.Expr->Expr.SynType)>>3; n<PIC_Type_GetSize(yyP25->Convert.Type)>>3; n++)
      PIC_StoreWIntoMemory(yyP25->Expr.PIC_REG+n);

    
    if (size!=0) PIC_DeallocateBits(yyP25->Convert.Expr->Expr.PIC_REG,size);
    }
  }
  }
   return result;
yyL10:;

  break;
  case kRound:
/* line 2128 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2130 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: the ROUND operator is not supported", xxError, yyP25->Round.Pos);
  }
   return 0;

  case kTrunc:
/* line 2133 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2135 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: the TRUNC operator is not supported", xxError, yyP25->Trunc.Pos);
  }
   return 0;

  case kCountArray:
/* line 2138 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2139 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: the array count operation is not supported", xxError, yyP25->CountArray.Pos);
  }
   return 0;

  case kElementalExpr:
  if (yyP25->ElementalExpr.Elemental->Kind == kUnmappedIdentifier) {
/* line 2142 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2145 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP25 -> Expr . SynType) == 1))) goto yyL14;
  {
/* line 2146 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP25 -> Expr . PIC_REG != InvalidW))) goto yyL14;
  {
/* line 2147 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetSymbolAddress (SymbolKind (yyP25->ElementalExpr.Elemental->UnmappedIdentifier.Sym)) != InvalidW))) goto yyL14;
  {
/* line 2148 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CopyBit (yyP25 -> Expr . PIC_REG, PIC_GetSymbolAddress (SymbolKind (yyP25->ElementalExpr.Elemental->UnmappedIdentifier.Sym)));
  }
  }
  }
  }
   return 0;
yyL14:;

/* line 2151 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2154 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP25 -> Expr . PIC_REG != InvalidW))) goto yyL15;
  {
/* line 2155 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetSymbolAddress (SymbolKind (yyP25->ElementalExpr.Elemental->UnmappedIdentifier.Sym)) != InvalidW))) goto yyL15;
  {
/* line 2156 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;
    for(n=0; n<PIC_Type_GetSize(yyP25->Expr.SynType)>>3; n++)
      PIC_CopyByte(yyP25->Expr.PIC_REG+n,PIC_GetSymbolAddress(SymbolKind(yyP25->ElementalExpr.Elemental->UnmappedIdentifier.Sym))+n);
    }
  }
  }
  }
   return 0;
yyL15:;

/* line 2163 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2164 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP25 -> Expr . PIC_REG = PIC_GetSymbolAddress (SymbolKind (yyP25->ElementalExpr.Elemental->UnmappedIdentifier.Sym));
  }
   return 0;

  }
/* line 2167 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2168 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with ElementalExpr node", xxError, yyP25->ElementalExpr.Pos);
  }
   return 0;

  case kUnBound:
/* line 2171 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2172 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with UnBound node", xxError, yyP25->UnBound.Pos);
  }
   return 0;

  case kConst:
/* line 2175 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2176 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: constant node contains non-constant value", xxError, yyP25->Const.Pos);
  }
   return 0;

  case kString:
/* line 2179 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2180 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with String node", xxError, yyP25->String.Pos);
  }
   return 0;

  case kDefault:
/* line 2183 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2184 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with Default node", xxError, yyP25->Default.Pos);
  }
   return 0;

  }

 yyAbort ("PIC_Expr_Evaluate2");
}

static int PIC_GetSymbolAddress
# if defined __STDC__ | defined __cplusplus
(register tTree yyP26)
# else
(yyP26)
 register tTree yyP26;
# endif
{
  if (yyP26->Kind == kIs) {
/* line 2190 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return PIC_GetSymbolAddress (SymbolKind (yyP26->Is.Elemental -> UnmappedIdentifier . Sym));

  }
  if (yyP26->Kind == kReplicator) {
/* line 2191 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return yyP26 -> Replicator . From -> Expr . PIC_REG;

  }
  if (yyP26->Kind == kAnId) {
/* line 2192 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return yyP26 -> AnId . PIC_REG;

  }
/* line 2193 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2193 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   WriteTree (stdout, yyP26);
/* line 2193 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   goto yyL4;
  }
yyL4:;

 yyAbort ("PIC_GetSymbolAddress");
}

static void PIC_EvaluateBinary
# if defined __STDC__ | defined __cplusplus
(register tTree yyP30, register tTree yyP29, register tTree yyP28, register tTree yyP27)
# else
(yyP30, yyP29, yyP28, yyP27)
 register tTree yyP30;
 register tTree yyP29;
 register tTree yyP28;
 register tTree yyP27;
# endif
{
/* line 2197 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int a1; int a2; int a3;

  switch (yyP27->Kind) {
  case kLess:
/* line 2198 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2201 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP29 -> Expr . SynType) == 8))) goto yyL1;
  {
/* line 2202 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP28 -> Expr . SynType) == 8))) goto yyL1;
  {
/* line 2203 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    if (yyP28->Expr.C==true)
      PIC_LoadConstantIntoW(yyP28->Expr.CV.Number);
    else
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG);

    
    if (yyP29->Expr.C==true)
      PIC_SubtractWFromConstant(yyP29->Expr.CV.Number);
    else
      PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG);

    
    PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
    PIC_SkipIfCarryFlagIsClear();
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
    PIC_SkipIfZeroFlagIsClear();
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
    }
  }
  }
  }
   return;
yyL1:;

/* line 2224 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2228 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;

    
    int size = PIC_Type_GetSize(yyP29->Expr.SynType) >> 3;

    
    for(n=0; n<size; n++)
      {
      
      if (n!=0)
        PIC_RotateMemoryLeft(4);

      
      if (yyP28->Expr.C==true)
        PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);

      
      if (yyP29->Expr.C==true)
        PIC_SubtractWFromConstant((yyP29->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      
      if (n!=0)
        {
        PIC_SkipIfBitSet((4*8)+0);
        PIC_AddConstantToW(255);
        }
      }

    
    PIC_StoreWIntoMemory(4);

    
    PIC_RotateMemoryLeft(4);

    
    if (yyP29->Expr.C==true)
      PIC_LoadConstantIntoW((yyP29->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_LoadMemoryIntoW(yyP29->Expr.PIC_REG+size-1);
    if (yyP28->Expr.C==true)
      PIC_XORConstantIntoW((yyP28->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_XORMemoryIntoW(yyP28->Expr.PIC_REG+size-1);
    PIC_StoreWIntoMemory(4);

    
    
    PIC_RotateMemoryRight(4);

    
    if (yyP28->Expr.C==true)
      PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+size-1);
    PIC_XORMemoryIntoW(4);
    PIC_ANDConstantIntoW(0x80);
    

    
    PIC_SkipIfZeroFlagIsSet();
    PIC_ClearBitInMemory((4*8)+6);

    
    
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
    PIC_LoadConstantIntoW(1<<(yyP30->Expr.PIC_REG%8));
    PIC_SkipIfBitClear((4*8)+7);
    PIC_XORWIntoMemory(yyP30->Expr.PIC_REG/8);
    PIC_SkipIfBitClear((4*8)+6);
    PIC_XORWIntoMemory(yyP30->Expr.PIC_REG/8);
    }
  }
   return;

  case kGreater:
/* line 2305 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2308 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP29 -> Expr . SynType) == 8))) goto yyL3;
  {
/* line 2309 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP28 -> Expr . SynType) == 8))) goto yyL3;
  {
/* line 2310 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    if (yyP28->Expr.C==true)
      PIC_LoadConstantIntoW(yyP28->Expr.CV.Number);
    else
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG);

    
    if (yyP29->Expr.C==true)
      PIC_SubtractWFromConstant(yyP29->Expr.CV.Number);
    else
      PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG);

    
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
    PIC_SkipIfCarryFlagIsClear();
    PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
    }
  }
  }
  }
   return;
yyL3:;

/* line 2329 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2333 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int label = PIC_GetNewLabel();
    int n;

    
    int size = PIC_Type_GetSize(yyP29->Expr.SynType) >> 3;

    
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);

    
    for(n=0; n<size; n++)
      {
      
      if (n!=0)
        PIC_RotateMemoryLeft(4);

      
      if (yyP28->Expr.C==true)
        PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);

      
      if (yyP29->Expr.C==true)
        PIC_SubtractWFromConstant((yyP29->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      
      if (n!=0)
        {
        PIC_SkipIfBitSet((4*8)+0);
        PIC_AddConstantToW(255);
        }

      
      PIC_SkipIfZeroFlagIsSet();
      PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
      }

    
    PIC_SkipIfBitSet(yyP30->Expr.PIC_REG);
    PIC_GotoLabel(label,"the result is Zero, so the condition is FALSE");

    
    PIC_StoreWIntoMemory(4);

    
    PIC_RotateMemoryLeft(4);

    
    if (yyP29->Expr.C==true)
      PIC_LoadConstantIntoW((yyP29->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_LoadMemoryIntoW(yyP29->Expr.PIC_REG+size-1);
    if (yyP28->Expr.C==true)
      PIC_XORConstantIntoW((yyP28->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_XORMemoryIntoW(yyP28->Expr.PIC_REG+size-1);
    PIC_StoreWIntoMemory(4);

    
    
    PIC_RotateMemoryRight(4);

    
    if (yyP28->Expr.C==true)
      PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+size-1);
    PIC_XORMemoryIntoW(4);
    PIC_ANDConstantIntoW(0x80);
    

    
    PIC_SkipIfZeroFlagIsSet();
    PIC_ClearBitInMemory((4*8)+6);

    
    
    
    PIC_LoadConstantIntoW(1<<(yyP30->Expr.PIC_REG%8));
    PIC_SkipIfBitClear((4*8)+7);
    PIC_XORWIntoMemory(yyP30->Expr.PIC_REG/8);
    PIC_SkipIfBitClear((4*8)+6);
    PIC_XORWIntoMemory(yyP30->Expr.PIC_REG/8);

    
    PIC_OutputLabel(label);
    }
  }
   return;

  case kLessEqual:
/* line 2425 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2428 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP29 -> Expr . SynType) == 8))) goto yyL5;
  {
/* line 2429 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP28 -> Expr . SynType) == 8))) goto yyL5;
  {
/* line 2430 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    if (yyP28->Expr.C==true)
      PIC_LoadConstantIntoW(yyP28->Expr.CV.Number);
    else
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG);

    
    if (yyP29->Expr.C==true)
      PIC_SubtractWFromConstant(yyP29->Expr.CV.Number);
    else
      PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG);

    
    PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
    PIC_SkipIfCarryFlagIsClear();
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
    }
  }
  }
  }
   return;
yyL5:;

/* line 2449 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2453 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int label = PIC_GetNewLabel();
    int n;

    
    int size = PIC_Type_GetSize(yyP29->Expr.SynType) >> 3;

    
    PIC_SetBitInMemory(yyP30->Expr.PIC_REG);

    
    for(n=0; n<size; n++)
      {
      
      if (n!=0)
        PIC_RotateMemoryLeft(4);

      
      if (yyP28->Expr.C==true)
        PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);

      
      if (yyP29->Expr.C==true)
        PIC_SubtractWFromConstant((yyP29->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      
      if (n!=0)
        {
        PIC_SkipIfBitSet((4*8)+0);
        PIC_AddConstantToW(255);
        }

      
      PIC_SkipIfZeroFlagIsSet();
      PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
      }

    
    PIC_SkipIfBitClear(yyP30->Expr.PIC_REG);
    PIC_GotoLabel(label,"result was Zero, so exit comparison early");

    
    PIC_StoreWIntoMemory(4);

    
    PIC_RotateMemoryLeft(4);

    
    if (yyP29->Expr.C==true)
      PIC_LoadConstantIntoW((yyP29->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_LoadMemoryIntoW(yyP29->Expr.PIC_REG+size-1);
    if (yyP28->Expr.C==true)
      PIC_XORConstantIntoW((yyP28->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_XORMemoryIntoW(yyP28->Expr.PIC_REG+size-1);
    PIC_StoreWIntoMemory(4);

    
    
    PIC_RotateMemoryRight(4);

    
    if (yyP28->Expr.C==true)
      PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+size-1);
    PIC_XORMemoryIntoW(4);
    PIC_ANDConstantIntoW(0x80);
    

    
    PIC_SkipIfZeroFlagIsSet();
    PIC_ClearBitInMemory((4*8)+6);

    
    
    
    PIC_LoadConstantIntoW(1<<(yyP30->Expr.PIC_REG%8));
    PIC_SkipIfBitClear((4*8)+7);
    PIC_XORWIntoMemory(yyP30->Expr.PIC_REG/8);
    PIC_SkipIfBitClear((4*8)+6);
    PIC_XORWIntoMemory(yyP30->Expr.PIC_REG/8);

    
    PIC_OutputLabel(label);
    }
  }
   return;

  case kGreaterEqual:
/* line 2545 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2548 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP29 -> Expr . SynType) == 8))) goto yyL7;
  {
/* line 2549 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP28 -> Expr . SynType) == 8))) goto yyL7;
  {
/* line 2550 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    if (yyP28->Expr.C==true)
      PIC_LoadConstantIntoW(yyP28->Expr.CV.Number);
    else
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG);

    
    if (yyP29->Expr.C==true)
      PIC_SubtractWFromConstant(yyP29->Expr.CV.Number);
    else
      PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG);

    
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
    PIC_SkipIfCarryFlagIsClear();
    PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
    PIC_SkipIfZeroFlagIsClear();
    PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
    }
  }
  }
  }
   return;
yyL7:;

/* line 2571 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2575 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;

    
    int size = PIC_Type_GetSize(yyP29->Expr.SynType) >> 3;

    
    for(n=0; n<size; n++)
      {
      
      if (n!=0)
        PIC_RotateMemoryLeft(4);

      
      if (yyP28->Expr.C==true)
        PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);

      
      if (yyP29->Expr.C==true)
        PIC_SubtractWFromConstant((yyP29->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      
      if (n!=0)
        {
        PIC_SkipIfBitSet((4*8)+0);
        PIC_AddConstantToW(255);
        }
      }

    
    PIC_StoreWIntoMemory(4);

    
    PIC_RotateMemoryLeft(4);

    
    if (yyP29->Expr.C==true)
      PIC_LoadConstantIntoW((yyP29->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_LoadMemoryIntoW(yyP29->Expr.PIC_REG+size-1);
    if (yyP28->Expr.C==true)
      PIC_XORConstantIntoW((yyP28->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_XORMemoryIntoW(yyP28->Expr.PIC_REG+size-1);
    PIC_StoreWIntoMemory(4);

    
    
    PIC_RotateMemoryRight(4);

    
    if (yyP28->Expr.C==true)
      PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>((size-1)<<3))&0xFF);
    else
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+size-1);
    PIC_XORMemoryIntoW(4);
    PIC_ANDConstantIntoW(0x80);
    

    
    PIC_SkipIfZeroFlagIsSet();
    PIC_ClearBitInMemory((4*8)+6);

    
    
    PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
    PIC_LoadConstantIntoW(1<<(yyP30->Expr.PIC_REG%8));
    PIC_SkipIfBitClear((4*8)+7);
    PIC_XORWIntoMemory(yyP30->Expr.PIC_REG/8);
    PIC_SkipIfBitClear((4*8)+6);
    PIC_XORWIntoMemory(yyP30->Expr.PIC_REG/8);
    }
  }
   return;

  case kEqual:
/* line 2652 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2653 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Equal (yyP30 -> Expr . PIC_REG, yyP29, yyP28);
  }
   return;

  case kNotEqual:
/* line 2655 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2658 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP29 -> Expr . SynType) == 1))) goto yyL10;
  {
/* line 2659 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP28 -> Expr . SynType) == 1))) goto yyL10;
  {
/* line 2660 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    a1=yyP30->Expr.PIC_REG;
    a2=yyP29->Expr.PIC_REG;
    a3=yyP28->Expr.PIC_REG;

    
    if ((yyP29->Expr.C==true) && (yyP28->Expr.C==false))
      {
      PIC_SetBitInMemory(a1);
      if (yyP29->Expr.CV.Number==true)
        PIC_SkipIfBitClear(a3);
      else
        PIC_SkipIfBitSet(a3);
      PIC_ClearBitInMemory(a1);
      }
    if ((yyP29->Expr.C==false) && (yyP28->Expr.C==true))
      {
      PIC_SetBitInMemory(a1);
      if (yyP28->Expr.CV.Number==true)
        PIC_SkipIfBitClear(a2);
      else
        PIC_SkipIfBitSet(a2);
      PIC_ClearBitInMemory(a1);
      }
    if ((yyP29->Expr.C==false) && (yyP28->Expr.C==false))
      {
      PIC_LoadConstantIntoW(1<<(a1%8));
      PIC_ClearBitInMemory(a1);
      PIC_SkipIfBitClear(a2);
      PIC_XORWIntoMemory(a1/8);
      PIC_SkipIfBitClear(a3);
      PIC_XORWIntoMemory(a1/8);
      }
    }
  }
  }
  }
   return;
yyL10:;

/* line 2696 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2699 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    int size = PIC_Type_GetSize(yyP29->Expr.SynType) >> 3;
    int n;

    
    int label = PIC_GetNewLabel();

    
    a1=yyP30->Expr.PIC_REG;
    a2=yyP29->Expr.PIC_REG;
    a3=yyP28->Expr.PIC_REG;

    
    PIC_SetBitInMemory(a1);

    
    for(n=0; n<size; n++)
      {
      if ((yyP29->Expr.C==true) && (yyP28->Expr.C==false))
        {
        PIC_LoadConstantIntoW((yyP29->Expr.CV.Number>>(n<<3)) & 0xFF);
        PIC_XORMemoryIntoW(a3+n);
        }
      if ((yyP29->Expr.C==false) && (yyP28->Expr.C==true))
        {
        PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>(n<<3)) & 0xFF);
        PIC_XORMemoryIntoW(a2+n);
        }
      if ((yyP29->Expr.C==false) && (yyP28->Expr.C==false))
        {
        PIC_LoadMemoryIntoW(a2+n);
        PIC_XORMemoryIntoW(a3+n);
        }
      PIC_SkipIfZeroFlagIsSet();
      PIC_GotoLabel(label,"operands are not equal, so abort comparison");
      }

    
    PIC_ClearBitInMemory(a1);

    
    PIC_OutputLabel(label);
    }
  }
   return;

  case kAdd:
/* line 2744 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2749 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;
    int n;

    
    a1=yyP30->Expr.PIC_REG;
    a2=yyP29->Expr.PIC_REG;
    a3=yyP28->Expr.PIC_REG;

    
    for(n=0; n<size; n++)
      {
      
      
      
      if ((PIC_CheckForArithmeticOverflow==true) && (n==(size-1)))
        {
        
        if (yyP29->Expr.C==true)
          PIC_LoadConstantIntoW((yyP29->Expr.CV.Number>>(n*8))&255);
        else
          PIC_LoadMemoryIntoW(a2+n);

        
        if (yyP28->Expr.C==true)
          PIC_XORConstantIntoW((yyP28->Expr.CV.Number>>(n*8))&255);
        else
          PIC_XORMemoryIntoW(a3+n);

        
        PIC_StoreWIntoMemory(4);

        
        if ((yyP28->Expr.C==true) && (((yyP28->Expr.CV.Number>>(n*8))&0x80)!=0)) PIC_SetBitInMemory((4*8)+6);
        if ((yyP28->Expr.C==true) && (((yyP28->Expr.CV.Number>>(n*8))&0x80)==0)) PIC_ClearBitInMemory((4*8)+6);
        if (yyP28->Expr.C==false) PIC_CopyBit((4*8)+6,((a3+n)*8)+7);
        }

      
      if (yyP29->Expr.C==true)
        PIC_LoadConstantIntoW((yyP29->Expr.CV.Number>>(n*8))&255);
      else
        PIC_LoadMemoryIntoW(a2+n);

      
      if (n!=0)
        {
        PIC_SkipIfCarryFlagIsClear();
        PIC_AddConstantToW(1);
        }

      
      
      if ((n!=0) && (n!=(size-1)))
        if (!((yyP28->Expr.C==true) && (((yyP28->Expr.CV.Number>>(n*8))&255)==0)))
          PIC_RotateMemoryLeft(4);

      
      if (yyP28->Expr.C==true)
        PIC_AddConstantToW((yyP28->Expr.CV.Number>>(n*8))&255);
      else
        PIC_AddMemoryToW(a3+n);

      
      PIC_StoreWIntoMemory(a1+n);

      
      
      if ((n!=0) && (n!=(size-1)))
        if (!((yyP28->Expr.C==true) && (((yyP28->Expr.CV.Number>>(n*8))&255)==0)))
          {
          PIC_SkipIfCarryFlagIsSet();
          PIC_RotateMemoryRight(4);
          }
      }

    if (PIC_CheckForArithmeticOverflow==true)
      {
      
      int cont = PIC_GetNewLabel();
      PIC_SkipIfBitClear((4*8)+7);
      PIC_GotoLabel(cont,"no overflow, so continue");
      PIC_RotateMemoryLeft(4);
      PIC_XORWIntoMemory(4);
      PIC_SkipIfBitSet((4*8)+7);
      PIC_GotoLabel(cont,"no overflow, so continue");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      PIC_OutputLabel(cont);
      }
    }
  }
   return;

  case kSubtract:
/* line 2841 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2846 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP29 -> Expr . C == true))) goto yyL13;
  {
/* line 2847 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == false))) goto yyL13;
  {
/* line 2848 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int label;

    
    
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;

    
    int n;
    for(n=0; n<size; n++)
      {
      

      
      
      
      if ((PIC_CheckForArithmeticOverflow==true) && (n==(size-1)))
        {
        
        PIC_LoadMemoryIntoW(a3+n);

        
        if (((yyP29->Expr.CV.Number>>(n*8))&0x80)!=0)
          PIC_XORConstantIntoW(0x80);

        
        PIC_StoreWIntoMemory(4);
        }

      if (n!=0)
        {
        
        
        PIC_RotateMemoryRight(4);
        }

      
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);
      PIC_SubtractWFromConstant((yyP29->Expr.CV.Number >> (n*8)) & 0xFF);

      if (n!=0)
        {
        
        
        PIC_RotateMemoryRight(4);

        
        
        label=PIC_GetNewLabel();
        PIC_SkipIfBitClear((4*8)+6);
        PIC_GotoLabel(label,"no Carry from previous byte");
        PIC_AddConstantToW(255);
        PIC_SkipIfCarryFlagIsSet();
        PIC_ClearBitInMemory((4*8)+7);
        PIC_OutputLabel(label);

        
        
        
        PIC_RotateMemoryLeft(4);
        }

      
      PIC_StoreWIntoMemory(yyP30->Expr.PIC_REG+n);
      }

    if (PIC_CheckForArithmeticOverflow==true)
      {
      
      
      int cont = PIC_GetNewLabel();
      PIC_SkipIfBitSet((4*8)+6);
      PIC_GotoLabel(cont,"no overflow, so continue");

      PIC_ANDConstantIntoW(0x80);
      if (((yyP29->Expr.CV.Number>>((size-1)*8))&0x80)!=0)
        PIC_SkipIfZeroFlagIsSet();
      else
        PIC_SkipIfZeroFlagIsClear();
      PIC_GotoLabel(cont,"no overflow, so continue");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      PIC_OutputLabel(cont);
      }
    }
  }
  }
  }
   return;
yyL13:;

/* line 2934 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 2939 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP29 -> Expr . C == false))) goto yyL14;
  {
/* line 2940 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL14;
  {
/* line 2941 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int label;

    
    
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;

    
    int n;
    for(n=0; n<size; n++)
      {
      

      
      
      
      if ((PIC_CheckForArithmeticOverflow==true) && (n==(size-1)))
        {
        
        PIC_LoadMemoryIntoW(a3+n);

        
        if (((yyP28->Expr.CV.Number>>(n*8))&0x80)!=0)
          PIC_XORConstantIntoW(0x80);

        
        PIC_StoreWIntoMemory(4);
        }

      if (n!=0)
        {
        
        
        PIC_RotateMemoryRight(4);
        }

      
      PIC_LoadConstantIntoW((yyP28->Expr.CV.Number >> (n*8)) & 0xFF);
      PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      if (n!=0)
        {
        
        
        PIC_RotateMemoryRight(4);

        
        
        label=PIC_GetNewLabel();
        PIC_SkipIfBitClear((4*8)+6);
        PIC_GotoLabel(label,"no Carry from previous byte");
        PIC_AddConstantToW(255);
        PIC_SkipIfCarryFlagIsSet();
        PIC_ClearBitInMemory((4*8)+7);
        PIC_OutputLabel(label);

        
        
        
        PIC_RotateMemoryLeft(4);
        }

      
      PIC_StoreWIntoMemory(yyP30->Expr.PIC_REG+n);
      }

    if (PIC_CheckForArithmeticOverflow==true)
      {
      
      
      int cont = PIC_GetNewLabel();
      PIC_SkipIfBitSet((4*8)+6);
      PIC_GotoLabel(cont,"no overflow, so continue");

      PIC_ANDConstantIntoW(0x80);
      if (((yyP28->Expr.CV.Number>>((size-1)*8))&0x80)!=0)
        PIC_SkipIfZeroFlagIsClear();
      else
        PIC_SkipIfZeroFlagIsSet();
      PIC_GotoLabel(cont,"no overflow, so continue");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      PIC_OutputLabel(cont);
      }
    }
  }
  }
  }
   return;
yyL14:;

/* line 3027 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3032 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int label;

    
    
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;

    
    int n;
    for(n=0; n<size; n++)
      {
      

      
      
      
      if ((PIC_CheckForArithmeticOverflow==true) && (n==(size-1)))
        {
        
        PIC_LoadMemoryIntoW(a2+n);

        
        PIC_XORMemoryIntoW(a3+n);

        
        PIC_StoreWIntoMemory(4);

        
        PIC_CopyBit((4*8)+6,((a3+n)*8)+7);
        }

      if (n!=0)
        {
        
        
        PIC_RotateMemoryRight(4);
        }

      
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);
      PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      if (n!=0)
        {
        
        
        PIC_RotateMemoryRight(4);

        
        
        label=PIC_GetNewLabel();
        PIC_SkipIfBitClear((4*8)+6);
        PIC_GotoLabel(label,"no Carry from previous byte");
        PIC_AddConstantToW(255);
        PIC_SkipIfCarryFlagIsSet();
        PIC_ClearBitInMemory((4*8)+7);
        PIC_OutputLabel(label);

        
        
        
        PIC_RotateMemoryLeft(4);
        }

      
      PIC_StoreWIntoMemory(yyP30->Expr.PIC_REG+n);
      }

    if (PIC_CheckForArithmeticOverflow==true)
      {
      
      
      
      int cont = PIC_GetNewLabel();
      PIC_SkipIfBitSet((4*8)+6);
      PIC_GotoLabel(cont,"no overflow, so continue");

      PIC_RotateMemoryLeft(4);
      PIC_RotateMemoryLeft(4);
      PIC_XORMemoryIntoW(4);
      PIC_ANDConstantIntoW(0x80);
      PIC_SkipIfZeroFlagIsSet();
      PIC_GotoLabel(cont,"no overflow, so continue");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      PIC_OutputLabel(cont);
      }
    }
  }
   return;

  case kMultiply:
/* line 3121 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3124 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP30 -> Expr . SynType) == 16))) goto yyL16;
  {
/* line 3125 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (IsMul16Needed==false)
      {
      Mul16Address0=PIC_AllocateBytes(2);  
      Mul16Address1=PIC_AllocateBytes(2);  
      Mul16Address2=PIC_AllocateBytes(2);  
      IsMul16Needed=true;
      }

    
    PIC_CopyByte(Mul16Address1+0,yyP29->Expr.PIC_REG+0);
    PIC_CopyByte(Mul16Address1+1,yyP29->Expr.PIC_REG+1);
    PIC_CopyByte(Mul16Address2+0,yyP28->Expr.PIC_REG+0);
    PIC_CopyByte(Mul16Address2+1,yyP28->Expr.PIC_REG+1);

    
    PIC_CallMul16();

    
    PIC_CopyByte(yyP30->Expr.PIC_REG+0,Mul16Address0+0);
    PIC_CopyByte(yyP30->Expr.PIC_REG+1,Mul16Address0+1);
    }
  }
  }
   return;
yyL16:;

  break;
  case kDivide:
/* line 3148 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3151 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: the division operator is not supported", xxRestriction, NoPosition);
  }
   return;

  case kRem:
/* line 3153 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3156 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: the remainder operator is not supported", xxRestriction, NoPosition);
  }
   return;

  case kBitAnd:
/* line 3158 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3160 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP30 -> Expr . SynType) == PIC_Type_GetSize (yyP29 -> Expr . SynType)))) goto yyL19;
  {
/* line 3161 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP29 -> Expr . SynType) == PIC_Type_GetSize (yyP28 -> Expr . SynType)))) goto yyL19;
  {
/* line 3162 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int byte;

    
    if (yyP30->Expr.PIC_REG==yyP29->Expr.PIC_REG) {tTree E=yyP29; yyP29=yyP28; yyP28=E;}

    
    if (yyP28->Expr.C==true) {tTree E=yyP29; yyP29=yyP28; yyP28=E;}

    
    a1=yyP30->Expr.PIC_REG;
    a2=yyP29->Expr.PIC_REG;
    a3=yyP28->Expr.PIC_REG;

    
    for(byte=0; byte<(PIC_Type_GetSize(yyP30->Expr.SynType)>>3); byte++)
      {
      if (yyP29->Expr.C==true)
        {
        
        
        
        int num = (yyP29->Expr.CV.Number >> (byte<<3)) & 255;
        switch(num)
          {
          case 0x00:
            
            PIC_StoreConstantIntoMemory(0,a1+byte);
            break;
          case 0xFF:
            
            PIC_CopyByte(a1+byte,a3+byte);
            break;
          default:
            PIC_LoadConstantIntoW(num);
            if (a1!=a3)
              {
              PIC_ANDMemoryIntoW(a3+byte);
              PIC_StoreWIntoMemory(a1+byte);
              }
            else
              PIC_ANDWIntoMemory(a1+byte);
            break;
          }
        }
      else
        {
        PIC_LoadMemoryIntoW(a2+byte);
        PIC_ANDMemoryIntoW(a3+byte);
        PIC_StoreWIntoMemory(a1+byte);
        }
      }
    }
  }
  }
  }
   return;
yyL19:;

  break;
  case kBitOr:
/* line 3216 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3218 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP30 -> Expr . SynType) == PIC_Type_GetSize (yyP29 -> Expr . SynType)))) goto yyL20;
  {
/* line 3219 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP29 -> Expr . SynType) == PIC_Type_GetSize (yyP28 -> Expr . SynType)))) goto yyL20;
  {
/* line 3220 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int byte;

    
    if (yyP30->Expr.PIC_REG==yyP29->Expr.PIC_REG) {tTree E=yyP29; yyP29=yyP28; yyP28=E;}

    
    if (yyP28->Expr.C==true) {tTree E=yyP29; yyP29=yyP28; yyP28=E;}

    
    a1=yyP30->Expr.PIC_REG;
    a2=yyP29->Expr.PIC_REG;
    a3=yyP28->Expr.PIC_REG;

    
    for(byte=0; byte<(PIC_Type_GetSize(yyP30->Expr.SynType)>>3); byte++)
      {
      if (yyP29->Expr.C==true)
        {
        
        
        
        int num = (yyP29->Expr.CV.Number >> (byte<<3)) & 255;
        switch(num)
          {
          case 0x00:
            
            PIC_CopyByte(a1+byte,a3+byte);
            break;
          case 0xFF:
            
            PIC_StoreConstantIntoMemory(0xFF,a1+byte);
            break;
          default:
            PIC_LoadConstantIntoW(num);
            if (a1!=a3)
              {
              PIC_ORMemoryIntoW(a3+byte);
              PIC_StoreWIntoMemory(a1+byte);
              }
            else
              PIC_ORWIntoMemory(a1+byte);
            break;
          }
        }
      else
        {
        PIC_LoadMemoryIntoW(a2+byte);
        PIC_ORMemoryIntoW(a3+byte);
        PIC_StoreWIntoMemory(a1+byte);
        }
      }
    }
  }
  }
  }
   return;
yyL20:;

  break;
  case kBitEor:
/* line 3274 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3276 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP30 -> Expr . SynType) == PIC_Type_GetSize (yyP29 -> Expr . SynType)))) goto yyL21;
  {
/* line 3277 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP29 -> Expr . SynType) == PIC_Type_GetSize (yyP28 -> Expr . SynType)))) goto yyL21;
  {
/* line 3278 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int byte;

    
    if (yyP30->Expr.PIC_REG==yyP29->Expr.PIC_REG) {tTree E=yyP29; yyP29=yyP28; yyP28=E;}

    
    if (yyP28->Expr.C==true) {tTree E=yyP29; yyP29=yyP28; yyP28=E;}

    
    a1=yyP30->Expr.PIC_REG;
    a2=yyP29->Expr.PIC_REG;
    a3=yyP28->Expr.PIC_REG;

    
    for(byte=0; byte<(PIC_Type_GetSize(yyP30->Expr.SynType)>>3); byte++)
      {
      if (yyP29->Expr.C==true)
        {
        
        
        
        int num = (yyP29->Expr.CV.Number >> (byte<<3)) & 255;
        switch(num)
          {
          case 0x00:
            
            PIC_CopyByte(a1+byte,a3+byte);
            break;
          case 0xFF:
            
            if (a1!=a3)
              {
              PIC_ComplementMemoryToW(a3+byte);
              PIC_StoreWIntoMemory(a1+byte);
              }
            else
              PIC_ComplementMemory(a1+byte);
            break;
          default:
            PIC_LoadConstantIntoW(num);
            if (a1!=a3)
              {
              PIC_XORMemoryIntoW(a3+byte);
              PIC_StoreWIntoMemory(a1+byte);
              }
            else
              PIC_XORWIntoMemory(a1+byte);
            break;
          }
        }
      else
        {
        PIC_LoadMemoryIntoW(a2+byte);
        PIC_XORMemoryIntoW(a3+byte);
        PIC_StoreWIntoMemory(a1+byte);
        }
      }
    }
  }
  }
  }
   return;
yyL21:;

  break;
  case kLeftShift:
/* line 3338 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3342 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_CheckForInvalidShifts == true))) goto yyL22;
  {
/* line 3343 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL22;
  {
/* line 3344 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (((yyP28 -> Expr . CV . Number < 0) || (yyP28 -> Expr . CV . Number > PIC_Type_GetSize (yyP30 -> Expr . SynType))))) goto yyL22;
  {
/* line 3345 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 3346 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
  }
  }
  }
   return;
yyL22:;

/* line 3348 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3352 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL23;
  {
/* line 3353 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . CV . Number % 8 == 0))) goto yyL23;
  {
/* line 3354 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int shift = yyP28->Expr.CV.Number/8;
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;
    int n;

    
    for(n=(size-1); n>=0; n--)
      {
      if (n-shift>=0)
        {
        
        PIC_CopyByte(yyP30->Expr.PIC_REG+n,yyP29->Expr.PIC_REG+n-shift);
        }
      else
        {
        
        PIC_StoreConstantIntoMemory(0,yyP30->Expr.PIC_REG+n);
        }
      }
    }
  }
  }
  }
   return;
yyL23:;

/* line 3375 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3380 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL24;
  {
/* line 3381 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a1 = yyP30 -> Expr . PIC_REG;
/* line 3382 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a2 = yyP29 -> Expr . PIC_REG;
/* line 3383 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;
    int n;
    int shift = yyP28->Expr.CV.Number;
    int label;

    
    for(n=0; n<size; n++)
      PIC_CopyByte(a1+n,a2+n);

    
    if (shift>1)
      PIC_StoreConstantIntoMemory(shift,4);

    
    if (shift>1)
      {
      label=PIC_GetNewLabel();
      PIC_OutputLabel(label);
      }

    
    if (shift>0)
      PIC_ClearBitInMemory((3*8) + 0);

    
    if (shift>0)
      for(n=0; n<size; n++)
        {
        
        PIC_RotateMemoryLeft(a1+n);
        }

    
    if (shift>1)
      {
      PIC_DecrementMemoryAndSkipIfZero(4);
      PIC_GotoLabel(label,"do another shift");
      }
    }
  }
  }
   return;
yyL24:;

/* line 3424 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3434 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a1 = yyP30 -> Expr . PIC_REG;
/* line 3435 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a2 = yyP29 -> Expr . PIC_REG;
/* line 3436 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a3 = yyP28 -> Expr . PIC_REG;
/* line 3437 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;
    int n;
    int label1;
    int label2;

    
    PIC_LoadMemoryIntoW(a3);
    PIC_StoreWIntoMemory(4);

    if (PIC_CheckForInvalidShifts==true)
      {
      
      label1 = PIC_GetNewLabel();
      PIC_SubtractWFromConstant(size<<3);  
      PIC_ANDConstantIntoW(0x80);  
      PIC_ORMemoryIntoW(a3+1);  
      PIC_SkipIfZeroFlagIsClear();
      PIC_GotoLabel(label1,"shift amount is valid, so do the shift");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      PIC_OutputLabel(label1);
      }

    label1 = PIC_GetNewLabel();
    label2 = PIC_GetNewLabel();

    
    if (yyP29->Expr.C==true)
      {
      
      PIC_Constant(a1,yyP29->Expr.CV,yyP29->Expr.SynType);
      }
    else
      {
      
      for(n=0; n<size; n++)
        PIC_CopyByte(a1+n,a2+n);
      }

    
    PIC_TestMemoryForZero(4);
    PIC_SkipIfZeroFlagIsClear();
    PIC_GotoLabel(label2,"shift is zero, so skip rotatation");

    
    PIC_OutputLabel(label1);

    
    PIC_ClearBitInMemory((3*8)+0);

    
    for(n=0; n<size; n++)
      {
      
      PIC_RotateMemoryLeft(a1+n);
      }

    
    PIC_DecrementMemoryAndSkipIfZero(4);
    PIC_GotoLabel(label1,"do another shift");

    
    PIC_OutputLabel(label2);
    }
  }
   return;

  case kRightShift:
/* line 3503 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3507 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_CheckForInvalidShifts == true))) goto yyL26;
  {
/* line 3508 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL26;
  {
/* line 3509 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (((yyP28 -> Expr . CV . Number < 0) || (yyP28 -> Expr . CV . Number > PIC_Type_GetSize (yyP30 -> Expr . SynType))))) goto yyL26;
  {
/* line 3510 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearSchedulingBit (current_process);
/* line 3511 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
  }
  }
  }
   return;
yyL26:;

/* line 3513 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3517 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL27;
  {
/* line 3518 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . CV . Number % 8 == 0))) goto yyL27;
  {
/* line 3519 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int shift = yyP28->Expr.CV.Number/8;
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;
    int n;

    
    for(n=0; n<size; n++)
      {
      if ((n+shift)<size)
        {
        
        PIC_CopyByte(yyP30->Expr.PIC_REG+n,yyP29->Expr.PIC_REG+n+shift);
        }
      else
        {
        
        PIC_StoreConstantIntoMemory(0,yyP30->Expr.PIC_REG+n);
        }
      }
    }
  }
  }
  }
   return;
yyL27:;

/* line 3540 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3545 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL28;
  {
/* line 3546 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a1 = yyP30 -> Expr . PIC_REG;
/* line 3547 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a2 = yyP29 -> Expr . PIC_REG;
/* line 3548 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;
    int n;
    int shift = yyP28->Expr.CV.Number;
    int label;

    
    for(n=0; n<size; n++)
      PIC_CopyByte(a1+n,a2+n);

    
    if (shift>1)
      PIC_StoreConstantIntoMemory(shift,4);

    
    if (shift>1)
      {
      label=PIC_GetNewLabel();
      PIC_OutputLabel(label);
      }

    
    if (shift>0)
      PIC_ClearBitInMemory((3*8) + 0);

    
    if (shift>0)
      for(n=(size-1); n>=0; n--)
        {
        
        PIC_RotateMemoryRight(a1+n);
        }

    
    if (shift>1)
      {
      PIC_DecrementMemoryAndSkipIfZero(4);
      PIC_GotoLabel(label,"do another shift");
      }
    }
  }
  }
   return;
yyL28:;

/* line 3589 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3599 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a1 = yyP30 -> Expr . PIC_REG;
/* line 3600 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a2 = yyP29 -> Expr . PIC_REG;
/* line 3601 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   a3 = yyP28 -> Expr . PIC_REG;
/* line 3602 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;
    int n;
    int label1;
    int label2;

    
    PIC_LoadMemoryIntoW(a3);
    PIC_StoreWIntoMemory(4);

    if (PIC_CheckForInvalidShifts==true)
      {
      
      label1 = PIC_GetNewLabel();
      PIC_SubtractWFromConstant(size<<3);  
      PIC_ANDConstantIntoW(0x80);  
      PIC_ORMemoryIntoW(a3+1);  
      PIC_SkipIfZeroFlagIsClear();
      PIC_GotoLabel(label1,"shift amount is valid, so do the shift");
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      PIC_OutputLabel(label1);
      }

    label1 = PIC_GetNewLabel();
    label2 = PIC_GetNewLabel();

    
    if (yyP29->Expr.C==true)
      {
      
      PIC_Constant(a1,yyP29->Expr.CV,yyP29->Expr.SynType);
      }
    else
      {
      
      for(n=0; n<size; n++)
        PIC_CopyByte(a1+n,a2+n);
      }

    
    PIC_TestMemoryForZero(4);
    PIC_SkipIfZeroFlagIsClear();
    PIC_GotoLabel(label2,"shift is zero, so skip rotatation");

    
    PIC_OutputLabel(label1);

    
    PIC_ClearBitInMemory((3*8)+0);

    
    for(n=(size-1); n>=0; n--)
      {
      
      PIC_RotateMemoryRight(a1+n);
      }

    
    PIC_DecrementMemoryAndSkipIfZero(4);
    PIC_GotoLabel(label1,"do another shift");

    
    PIC_OutputLabel(label2);
    }
  }
   return;

  case kSize:
/* line 3668 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3669 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("not sure what to do with the Size node", xxError, NoPosition);
  }
   return;

  case kPlus:
/* line 3671 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3676 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;
    int n;

    
    a1=yyP30->Expr.PIC_REG;
    a2=yyP29->Expr.PIC_REG;
    a3=yyP28->Expr.PIC_REG;

    
    for(n=0; n<size; n++)
      {
      
      if (yyP29->Expr.C==true)
        PIC_LoadConstantIntoW((yyP29->Expr.CV.Number>>(n*8))&255);
      else
        PIC_LoadMemoryIntoW(a2+n);

      
      if (n!=0)
        {
        PIC_SkipIfCarryFlagIsClear();
        PIC_AddConstantToW(1);
        }

      
      
      if ((n!=0) && (n!=(size-1)))
        if (!((yyP28->Expr.C==true) && (((yyP28->Expr.CV.Number>>(n*8))&255)==0)))
          PIC_RotateMemoryLeft(4);

      
      if (yyP28->Expr.C==true)
        PIC_AddConstantToW((yyP28->Expr.CV.Number>>(n*8))&255);
      else
        PIC_AddMemoryToW(a3+n);

      
      PIC_StoreWIntoMemory(a1+n);

      
      
      if ((n!=0) && (n!=(size-1)))
        if (!((yyP28->Expr.C==true) && (((yyP28->Expr.CV.Number>>(n*8))&255)==0)))
          {
          PIC_SkipIfCarryFlagIsSet();
          PIC_RotateMemoryRight(4);
          }
      }
    }
  }
   return;

  case kMinus:
/* line 3727 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3732 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP29 -> Expr . C == true))) goto yyL32;
  {
/* line 3733 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;

    
    int n;
    for(n=0; n<size; n++)
      {
      
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);
      PIC_SubtractWFromConstant((yyP29->Expr.CV.Number >> (n*8)) & 0xFF);

      
      
      if (n!=(size-1))
        PIC_RotateMemoryLeft(4);

      
      
      if (n!=0)
        {
        if (n!=(size-1))
          {
          int label = PIC_GetNewLabel();
          PIC_SkipIfBitClear((4*8)+1);
          PIC_GotoLabel(label,"no Carry, so do next byte");
          PIC_AddConstantToW(255);
          PIC_SkipIfCarryFlagIsSet();
          PIC_RotateMemoryLeft(4);
          PIC_OutputLabel(label);
          }
        else
          {
          PIC_SkipIfCarryFlagIsSet();
          PIC_AddConstantToW(255);
          }
        }

      
      PIC_StoreWIntoMemory(yyP30->Expr.PIC_REG+n);
      }
    }
  }
  }
   return;
yyL32:;

/* line 3777 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3782 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP28 -> Expr . C == true))) goto yyL33;
  {
/* line 3783 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;

    
    int n;
    for(n=0; n<size; n++)
      {
      
      PIC_LoadConstantIntoW((yyP28->Expr.CV.Number >> (n*8)) & 0xFF);
      PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      
      
      if (n!=(size-1))
        PIC_RotateMemoryLeft(4);

      
      
      if (n!=0)
        {
        if (n!=(size-1))
          {
          int label = PIC_GetNewLabel();
          PIC_SkipIfBitClear((4*8)+1);
          PIC_GotoLabel(label,"no Carry, so do next byte");
          PIC_AddConstantToW(255);
          PIC_SkipIfCarryFlagIsSet();
          PIC_RotateMemoryLeft(4);
          PIC_OutputLabel(label);
          }
        else
          {
          PIC_SkipIfCarryFlagIsSet();
          PIC_AddConstantToW(255);
          }
        }

      
      PIC_StoreWIntoMemory(yyP30->Expr.PIC_REG+n);
      }
    }
  }
  }
   return;
yyL33:;

/* line 3827 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3832 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    
    int size = PIC_Type_GetSize(yyP30->Expr.SynType) >> 3;

    
    int n;
    for(n=0; n<size; n++)
      {
      
      PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);
      PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      
      
      if (n!=(size-1))
        PIC_RotateMemoryLeft(4);

      
      
      if (n!=0)
        {
        if (n!=(size-1))
          {
          int label = PIC_GetNewLabel();
          PIC_SkipIfBitClear((4*8)+1);
          PIC_GotoLabel(label,"no Carry, so do next byte");
          PIC_AddConstantToW(255);
          PIC_SkipIfCarryFlagIsSet();
          PIC_RotateMemoryLeft(4);
          PIC_OutputLabel(label);
          }
        else
          {
          PIC_SkipIfCarryFlagIsSet();
          PIC_AddConstantToW(255);
          }
        }

      
      PIC_StoreWIntoMemory(yyP30->Expr.PIC_REG+n);
      }
    }
  }
   return;

  case kTimes:
/* line 3876 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3879 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP30 -> Expr . SynType) == 16))) goto yyL35;
  {
/* line 3880 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (IsMul16Needed==false)
      {
      Mul16Address0=PIC_AllocateBytes(2);  
      Mul16Address1=PIC_AllocateBytes(2);  
      Mul16Address2=PIC_AllocateBytes(2);  
      IsMul16Needed=true;
      }

    
    PIC_CopyByte(Mul16Address1+0,yyP29->Expr.PIC_REG+0);
    PIC_CopyByte(Mul16Address1+1,yyP29->Expr.PIC_REG+1);
    PIC_CopyByte(Mul16Address2+0,yyP28->Expr.PIC_REG+0);
    PIC_CopyByte(Mul16Address2+1,yyP28->Expr.PIC_REG+1);

    
    PIC_CallMul16();

    
    PIC_CopyByte(yyP30->Expr.PIC_REG+0,Mul16Address0+0);
    PIC_CopyByte(yyP30->Expr.PIC_REG+1,Mul16Address0+1);
    }
  }
  }
   return;
yyL35:;

  break;
  case kAfterOp:
/* line 3903 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3905 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP29->Expr.SynType) >> 3;
    int n;

    
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);

    
    for(n=0; n<size; n++)
      {
      
      if (n!=0)
        PIC_RotateMemoryLeft(4);

      
      if (yyP28->Expr.C==true)
        PIC_LoadConstantIntoW((yyP28->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_LoadMemoryIntoW(yyP28->Expr.PIC_REG+n);

      
      if (yyP29->Expr.C==true)
        PIC_SubtractWFromConstant((yyP29->Expr.CV.Number>>(n<<3))&0xFF);
      else
        PIC_SubtractWFromMemory(yyP29->Expr.PIC_REG+n);

      
      
      
      if ((n!=0) && (n!=(size-1)))
        {
        
        int no_carry=PIC_GetNewLabel();
        PIC_SkipIfBitClear((4*8)+0);
        PIC_GotoLabel(no_carry,"there is no carry to propogate, so skip the next section of code");

        
        PIC_RotateMemoryLeft(4);
        PIC_AddConstantToW(0xFF);

        
        PIC_SkipIfBitSet((4*8)+0);
        PIC_ClearBitInMemory((3*8)+0);

        PIC_OutputLabel(no_carry);
        }

      
      if (n==(size-1))
        {
        PIC_SkipIfBitSet((4*8)+0);
        PIC_AddConstantToW(0xFF);
        }

      
      PIC_SkipIfZeroFlagIsSet();
      PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
      }

    
    PIC_ANDConstantIntoW(0x80);
    PIC_SkipIfZeroFlagIsSet();
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
    }
  }
   return;

  case kAnd:
/* line 3970 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3973 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((((yyP29 -> Expr . C == true) && (yyP29 -> Expr . CV . Number == false)) || ((yyP28 -> Expr . C == true) && (yyP28 -> Expr . CV . Number == false))))) goto yyL37;
  {
/* line 3974 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory (yyP30 -> Expr . PIC_REG);
  }
  }
   return;
yyL37:;

/* line 3976 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3979 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (((yyP29 -> Expr . C == true) || (yyP28 -> Expr . C == true)))) goto yyL38;
  {
/* line 3980 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
    if (yyP29->Expr.C==false)
      {
      PIC_SkipIfBitSet(yyP29->Expr.PIC_REG);
      PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
      }
    if (yyP28->Expr.C==false)
      {
      PIC_SkipIfBitSet(yyP28->Expr.PIC_REG);
      PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
      }
    }
  }
  }
   return;
yyL38:;

/* line 3994 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 3997 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (((yyP29 -> Expr . C == false) && (yyP28 -> Expr . C == false)))) goto yyL39;
  {
/* line 3998 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory (yyP30 -> Expr . PIC_REG);
/* line 3999 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitClear (yyP29 -> Expr . PIC_REG);
/* line 4000 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet (yyP28 -> Expr . PIC_REG);
/* line 4001 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory (yyP30 -> Expr . PIC_REG);
  }
  }
   return;
yyL39:;

  break;
  case kOr:
/* line 4003 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4006 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((((yyP29 -> Expr . C == true) && (yyP29 -> Expr . CV . Number == true)) || ((yyP28 -> Expr . C == true) && (yyP28 -> Expr . CV . Number == true))))) goto yyL40;
  {
/* line 4007 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory (yyP30 -> Expr . PIC_REG);
  }
  }
   return;
yyL40:;

/* line 4009 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4012 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (((yyP29 -> Expr . C == true) || (yyP28 -> Expr . C == true)))) goto yyL41;
  {
/* line 4013 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    PIC_ClearBitInMemory(yyP30->Expr.PIC_REG);
    if (yyP29->Expr.C==false)
      {
      PIC_SkipIfBitClear(yyP29->Expr.PIC_REG);
      PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
      }
    if (yyP28->Expr.C==false)
      {
      PIC_SkipIfBitClear(yyP28->Expr.PIC_REG);
      PIC_SetBitInMemory(yyP30->Expr.PIC_REG);
      }
    }
  }
  }
   return;
yyL41:;

/* line 4027 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4030 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (((yyP29 -> Expr . C == false) && (yyP28 -> Expr . C == false)))) goto yyL42;
  {
/* line 4031 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory (yyP30 -> Expr . PIC_REG);
/* line 4032 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet (yyP29 -> Expr . PIC_REG);
/* line 4033 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitClear (yyP28 -> Expr . PIC_REG);
/* line 4034 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory (yyP30 -> Expr . PIC_REG);
  }
  }
   return;
yyL42:;

  break;
  }

/* line 4036 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4037 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: unknown operator detected", xxError, NoPosition);
  }
   return;

;
}

static void PIC_AltList
# if defined __STDC__ | defined __cplusplus
(register int yyP33, register int yyP32, register tTree yyP31)
# else
(yyP33, yyP32, yyP31)
 register int yyP33;
 register int yyP32;
 register tTree yyP31;
# endif
{
/* line 4042 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int next;
  if (yyP31->Kind == kAnAlt) {
/* line 4043 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4044 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   next = PIC_GetNewLabel ();
/* line 4045 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP31->AnAlt.SpecList);
/* line 4046 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_AltList_Guard (yyP33, next, yyP32, yyP31->AnAlt.Guard);
/* line 4047 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP31->AnAlt.SpecList);
/* line 4048 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (next);
/* line 4049 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_AltList (yyP33, yyP32, yyP31->AnAlt.Next);
  }
   return;

  }
  if (yyP31->Kind == kANewAlt) {
/* line 4051 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4052 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP31->ANewAlt.SpecList);
/* line 4053 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP31->ANewAlt.AltType);
/* line 4054 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP31->ANewAlt.SpecList);
/* line 4055 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_AltList (yyP33, yyP32, yyP31->ANewAlt.Next);
  }
   return;

  }
  if (yyP31->Kind == kNoAlt) {
/* line 4057 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4059 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ReturnToScheduler ();
  }
   return;

  }
 yyAbort ("PIC_AltList");
;
}

static void PIC_AltList_Guard
# if defined __STDC__ | defined __cplusplus
(register int yyP37, register int yyP36, register int yyP35, register tTree yyP34)
# else
(yyP37, yyP36, yyP35, yyP34)
 register int yyP37;
 register int yyP36;
 register int yyP35;
 register tTree yyP34;
# endif
{
/* line 4064 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int chan; int adr; int size;
  if (yyP34->Kind == kAltInput) {
  if (yyP34->AltInput.Input->Input.ExprList->Kind == kAnExpr) {
  if (yyP34->AltInput.Input->Input.ExprList->AnExpr.Next->Kind == kNoExpr) {
/* line 4065 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4067 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP34->AltInput.Expr -> Expr . C == true))) goto yyL1;
  {
/* line 4068 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP34->AltInput.Expr -> Expr . CV . Number == true))) goto yyL1;
  {
/* line 4071 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP34->AltInput.Input->Input.SpecList);
/* line 4072 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   chan = PIC_GetChannelNumber (yyP34->AltInput.Input->Input.Element -> UnmappedIdentifier . Sym);
/* line 4073 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CallChannelHandler (chan);
/* line 4074 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_TestWForZero ();
/* line 4075 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsClear ();
/* line 4076 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "no outputs ready, so try next guard");
/* line 4077 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreWIntoMemory (4);
/* line 4078 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP34->AltInput.Input->Input.SpecList);
/* line 4081 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   adr = PIC_GetSymbolAddress (SymbolKind (yyP34->AltInput.Input->Input.ExprList->AnExpr.Expr -> ElementalExpr . Elemental -> UnmappedIdentifier . Sym));
/* line 4082 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   size = PIC_Type_GetSize (yyP34->AltInput.Input->Input.Element -> UnmappedIdentifier . SynType -> Chan . Protocol -> Simple . Type) >> 3;
/* line 4085 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;
    for(n=0; n<size; n++)
      {
      PIC_CopyByte(adr+n,0);
      if (n<(size-1))
        PIC_IncrementMemory(4);
      }
    }
/* line 4096 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP34->AltInput.Process);
/* line 4097 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (current_process!=yyP37)
      {
      PIC_ClearSchedulingBit(current_process);
      current_process=yyP37;
      }
    }
/* line 4104 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP35, "process has finished, so goto end of ALT");
  }
  }
  }
   return;
yyL1:;

/* line 4106 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4110 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP34->AltInput.Expr -> Expr . PIC_REG = PIC_AllocateBit ();
/* line 4111 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP34->AltInput.Expr);
/* line 4112 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet (yyP34->AltInput.Expr -> Expr . PIC_REG);
/* line 4113 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "expression is FALSE, so try next guard");
/* line 4114 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBit (yyP34->AltInput.Expr -> Expr . PIC_REG);
/* line 4117 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP34->AltInput.Input->Input.SpecList);
/* line 4118 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   chan = PIC_GetChannelNumber (yyP34->AltInput.Input->Input.Element -> UnmappedIdentifier . Sym);
/* line 4119 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CallChannelHandler (chan);
/* line 4120 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_TestWForZero ();
/* line 4121 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsClear ();
/* line 4122 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "no outputs ready, so try next guard");
/* line 4123 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreWIntoMemory (4);
/* line 4124 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP34->AltInput.Input->Input.SpecList);
/* line 4127 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   adr = PIC_GetSymbolAddress (SymbolKind (yyP34->AltInput.Input->Input.ExprList->AnExpr.Expr -> ElementalExpr . Elemental -> UnmappedIdentifier . Sym));
/* line 4128 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   size = PIC_Type_GetSize (yyP34->AltInput.Input->Input.Element -> UnmappedIdentifier . SynType -> Chan . Protocol -> Simple . Type) >> 3;
/* line 4131 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int n;
    for(n=0; n<size; n++)
      {
      PIC_CopyByte(adr+n,0);
      if (n<(size-1))
        PIC_IncrementMemory(4);
      }
    }
/* line 4142 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP34->AltInput.Process);
/* line 4143 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (current_process!=yyP37)
      {
      PIC_ClearSchedulingBit(current_process);
      current_process=yyP37;
      }
    }
/* line 4150 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP35, "process has finished, so goto end of ALT");
  }
   return;

  }
  }
  }
  if (yyP34->Kind == kAltTimer) {
/* line 4152 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4154 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP34->AltTimer.Expr -> Expr . C == true))) goto yyL3;
  {
/* line 4155 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP34->AltTimer.Expr -> Expr . CV . Number == true))) goto yyL3;
  {
/* line 4158 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP34->AltTimer.After->After.SpecList);
/* line 4159 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (adr = PIC_AllocateBytes (2))) goto yyL3;
  {
/* line 4160 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (yyP34->AltTimer.After->After.Expr -> Expr . PIC_REG = adr)) goto yyL3;
  {
/* line 4161 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP34->AltTimer.After->After.Expr);
/* line 4164 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory ((11 * 8) + 5);
/* line 4167 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory ((4 * 8) + 0);
/* line 4170 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_LoadMemoryIntoW (adr + 0);
/* line 4171 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SubtractWFromMemory (TimerAddressLSB);
/* line 4172 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsSet ();
/* line 4173 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory ((4 * 8) + 0);
/* line 4174 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_RotateMemoryLeft (4);
/* line 4177 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_LoadMemoryIntoW (adr + 1);
/* line 4178 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SubtractWFromMemory (TimerAddressMSB);
/* line 4179 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet ((4 * 8) + 0);
/* line 4180 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_AddConstantToW (255);
/* line 4181 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsSet ();
/* line 4182 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory ((4 * 8) + 1);
/* line 4185 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory ((11 * 8) + 5);
/* line 4188 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet ((4 * 8) + 1);
/* line 4189 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "timer hasn't expired, so try next guard");
/* line 4192 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ANDConstantIntoW (128);
/* line 4193 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsSet ();
/* line 4194 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "timer hasn't expired, so try next guard");
/* line 4197 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBytes (adr, 2);
/* line 4198 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP34->AltTimer.After->After.SpecList);
/* line 4201 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP34->AltTimer.Process);
/* line 4202 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (current_process!=yyP37)
      {
      PIC_ClearSchedulingBit(current_process);
      current_process=yyP37;
      }
    }
/* line 4209 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP35, "process has finished, so goto end of ALT");
  }
  }
  }
  }
  }
   return;
yyL3:;

/* line 4211 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4215 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP34->AltTimer.Expr -> Expr . PIC_REG = PIC_AllocateBit ();
/* line 4216 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP34->AltTimer.Expr);
/* line 4217 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet (yyP34->AltTimer.Expr -> Expr . PIC_REG);
/* line 4218 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "expression is FALSE, so try next guard");
/* line 4219 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBit (yyP34->AltTimer.Expr -> Expr . PIC_REG);
/* line 4222 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP34->AltTimer.After->After.SpecList);
/* line 4223 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (adr = PIC_AllocateBytes (2))) goto yyL4;
  {
/* line 4224 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! (yyP34->AltTimer.After->After.Expr -> Expr . PIC_REG = adr)) goto yyL4;
  {
/* line 4225 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP34->AltTimer.After->After.Expr);
/* line 4228 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory ((11 * 8) + 5);
/* line 4231 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory ((4 * 8) + 0);
/* line 4234 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_LoadMemoryIntoW (adr + 0);
/* line 4235 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SubtractWFromMemory (TimerAddressLSB);
/* line 4236 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsSet ();
/* line 4237 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory ((4 * 8) + 0);
/* line 4238 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_RotateMemoryLeft (4);
/* line 4241 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_LoadMemoryIntoW (adr + 1);
/* line 4242 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SubtractWFromMemory (TimerAddressMSB);
/* line 4243 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet ((4 * 8) + 0);
/* line 4244 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_AddConstantToW (255);
/* line 4245 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsSet ();
/* line 4246 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory ((4 * 8) + 1);
/* line 4249 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory ((11 * 8) + 5);
/* line 4252 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet ((4 * 8) + 1);
/* line 4253 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "timer hasn't expired, so try next guard");
/* line 4256 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ANDConstantIntoW (128);
/* line 4257 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfZeroFlagIsSet ();
/* line 4258 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "timer hasn't expired, so try next guard");
/* line 4261 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBytes (adr, 2);
/* line 4262 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP34->AltTimer.After->After.SpecList);
/* line 4265 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP34->AltTimer.Process);
/* line 4266 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (current_process!=yyP37)
      {
      PIC_ClearSchedulingBit(current_process);
      current_process=yyP37;
      }
    }
/* line 4273 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP35, "process has finished, so goto end of ALT");
  }
  }
  }
   return;
yyL4:;

  }
  if (yyP34->Kind == kAltCaseIn) {
/* line 4275 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4276 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   Message ("pic.trafo: not sure what to do with CaseIn node", xxError, yyP34->AltCaseIn.Pos);
  }
   return;

  }
  if (yyP34->Kind == kNoInput) {
/* line 4278 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4280 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP34->NoInput.Expr -> Expr . PIC_REG = PIC_AllocateBit ();
/* line 4281 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP34->NoInput.Expr);
/* line 4282 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet (yyP34->NoInput.Expr -> Expr . PIC_REG);
/* line 4283 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP36, "expression is FALSE, so try next guard");
/* line 4284 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBit (yyP34->NoInput.Expr -> Expr . PIC_REG);
/* line 4287 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP34->NoInput.Process);
/* line 4288 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */

    if (current_process!=yyP37)
      {
      PIC_ClearSchedulingBit(current_process);
      current_process=yyP37;
      }
    
/* line 4295 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP35, "process has finished, so goto end of ALT");
  }
   return;

  }
 yyAbort ("PIC_AltList_Guard");
;
}

static void PIC_CaseChoiceList
# if defined __STDC__ | defined __cplusplus
(register int yyP40, register tTree yyP39, register tTree yyP38)
# else
(yyP40, yyP39, yyP38)
 register int yyP40;
 register tTree yyP39;
 register tTree yyP38;
# endif
{
/* line 4300 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int label1; int label2;
  if (yyP38->Kind == kAChoice) {
/* line 4301 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4302 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP38->AChoice.SpecList);
/* line 4303 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   label1 = PIC_GetNewLabel ();
/* line 4304 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   label2 = PIC_GetNewLabel ();
/* line 4305 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CaseChoiceListCompare (label1, label2, yyP39, yyP38->AChoice.ExprList);
/* line 4306 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (label1);
/* line 4307 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP38->AChoice.Process);
/* line 4308 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP40, "process has finished, so goto end of CASE");
/* line 4309 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP38->AChoice.SpecList);
/* line 4310 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (label2);
/* line 4311 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CaseChoiceList (yyP40, yyP39, yyP38->AChoice.Next);
  }
   return;

  }
  if (yyP38->Kind == kNoChoice) {
/* line 4313 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4314 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    if (PIC_CheckForFailedCASEs==true)
      {
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      }
    }
  }
   return;

  }
 yyAbort ("PIC_CaseChoiceList");
;
}

static void PIC_CaseChoiceListCompare
# if defined __STDC__ | defined __cplusplus
(register int yyP44, register int yyP43, register tTree yyP42, register tTree yyP41)
# else
(yyP44, yyP43, yyP42, yyP41)
 register int yyP44;
 register int yyP43;
 register tTree yyP42;
 register tTree yyP41;
# endif
{
/* line 4327 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int res;
  if (yyP41->Kind == kAnExpr) {
  if (yyP41->AnExpr.Expr->Kind == kDefault) {
/* line 4328 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4330 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP44, "reached ELSE guard, so execute body");
  }
   return;

  }
/* line 4332 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4334 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   res = PIC_AllocateBit ();
/* line 4335 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Equal (res, yyP42, yyP41->AnExpr.Expr);
/* line 4336 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabelIfBitSet (res, yyP44);
/* line 4337 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBit (res);
/* line 4338 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CaseChoiceListCompare (yyP44, yyP43, yyP42, yyP41->AnExpr.Next);
  }
   return;

  }
  if (yyP41->Kind == kNoExpr) {
/* line 4340 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4341 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP43, "guard failed, so try next guard");
  }
   return;

  }
 yyAbort ("PIC_CaseChoiceListCompare");
;
}

static void PIC_IfChoiceList
# if defined __STDC__ | defined __cplusplus
(register int yyP46, register tTree yyP45)
# else
(yyP46, yyP45)
 register int yyP46;
 register tTree yyP45;
# endif
{
/* line 4346 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int adr; int label;
  if (yyP45->Kind == kAChoice) {
/* line 4347 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4348 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP45->AChoice.SpecList);
/* line 4349 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   label = PIC_GetNewLabel ();
/* line 4350 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   adr = PIC_AllocateBit ();
/* line 4351 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   yyP45->AChoice.ExprList -> AnExpr . Expr -> Expr . PIC_REG = adr;
/* line 4352 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Expr_Evaluate (yyP45->AChoice.ExprList -> AnExpr . Expr);
/* line 4353 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabelIfBitClear (adr, label);
/* line 4354 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_DeallocateBit (adr);
/* line 4355 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP45->AChoice.Process);
/* line 4356 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_GotoLabel (yyP46, "process has finished, so goto end of IF");
/* line 4357 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP45->AChoice.SpecList);
/* line 4358 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_OutputLabel (label);
/* line 4359 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IfChoiceList (yyP46, yyP45->AChoice.Next);
  }
   return;

  }
  if (yyP45->Kind == kANewIf) {
/* line 4361 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4362 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Allocate (yyP45->ANewIf.SpecList);
/* line 4363 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_Proc (yyP45->ANewIf.IfType);
/* line 4364 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SpecList_Deallocate (yyP45->ANewIf.SpecList);
/* line 4365 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_IfChoiceList (yyP46, yyP45->ANewIf.Next);
  }
   return;

  }
  if (yyP45->Kind == kNoChoice) {
/* line 4367 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4368 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    if (PIC_CheckForFailedIFs==true)
      {
      PIC_ClearSchedulingBit(current_process);
      PIC_ReturnToScheduler();
      }
    }
  }
   return;

  }
 yyAbort ("PIC_IfChoiceList");
;
}

static void PIC_Expr_Equal
# if defined __STDC__ | defined __cplusplus
(register int yyP49, register tTree yyP48, register tTree yyP47)
# else
(yyP49, yyP48, yyP47)
 register int yyP49;
 register tTree yyP48;
 register tTree yyP47;
# endif
{
/* line 4382 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int a1; int a2;
/* line 4383 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4388 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP48 -> Expr . C == true))) goto yyL1;
  {
/* line 4389 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP47 -> Expr . C == true))) goto yyL1;
  {
/* line 4390 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    if (yyP48->Expr.CV.Number==yyP47->Expr.CV.Number)
      PIC_SetBitInMemory(yyP49);
    else
      PIC_ClearBitInMemory(yyP49);
    }
  }
  }
  }
   return;
yyL1:;

/* line 4397 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4399 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP48 -> Expr . SynType) == 1))) goto yyL2;
  {
/* line 4400 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP47 -> Expr . SynType) == 1))) goto yyL2;
  {
/* line 4402 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    a1=yyP48->Expr.PIC_REG;
    a2=yyP47->Expr.PIC_REG;

    
    if ((yyP48->Expr.C==true) && (yyP47->Expr.C==false))
      {
      PIC_SetBitInMemory(yyP49);
      if (yyP48->Expr.CV.Number==true)
        PIC_SkipIfBitSet(a2);
      else
        PIC_SkipIfBitClear(a2);
      PIC_ClearBitInMemory(yyP49);
      }
    if ((yyP48->Expr.C==false) && (yyP47->Expr.C==true))
      {
      PIC_SetBitInMemory(yyP49);
      if (yyP47->Expr.CV.Number==true)
        PIC_SkipIfBitSet(a1);
      else
        PIC_SkipIfBitClear(a1);
      PIC_ClearBitInMemory(yyP49);
      }
    if ((yyP48->Expr.C==false) && (yyP47->Expr.C==false))
      {
      PIC_LoadConstantIntoW(1<<(yyP49%8));
      PIC_SetBitInMemory(yyP49);
      PIC_SkipIfBitClear(a1);
      PIC_XORWIntoMemory(yyP49/8);
      PIC_SkipIfBitClear(a2);
      PIC_XORWIntoMemory(yyP49/8);
      }
    }
  }
  }
  }
   return;
yyL2:;

/* line 4437 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4439 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_Type_GetSize (yyP48 -> Expr . SynType) == PIC_Type_GetSize (yyP47 -> Expr . SynType)))) goto yyL3;
  {
/* line 4441 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    int size = PIC_Type_GetSize(yyP48->Expr.SynType) >> 3;
    int n;

    
    int label = PIC_GetNewLabel();

    
    a1=yyP48->Expr.PIC_REG;
    a2=yyP47->Expr.PIC_REG;

    
    PIC_ClearBitInMemory(yyP49);

    
    for(n=0; n<size; n++)
      {
      if ((yyP48->Expr.C==true) && (yyP47->Expr.C==false))
        {
        PIC_LoadConstantIntoW((yyP48->Expr.CV.Number>>(n<<3)) & 0xFF);
        PIC_XORMemoryIntoW(a2+n);
        }
      if ((yyP48->Expr.C==false) && (yyP47->Expr.C==true))
        {
        PIC_LoadConstantIntoW((yyP47->Expr.CV.Number>>(n<<3)) & 0xFF);
        PIC_XORMemoryIntoW(a1+n);
        }
      if ((yyP48->Expr.C==false) && (yyP47->Expr.C==false))
        {
        PIC_LoadMemoryIntoW(a1+n);
        PIC_XORMemoryIntoW(a2+n);
        }
      PIC_SkipIfZeroFlagIsSet();
      PIC_GotoLabel(label,"operands are not equal, so abort comparison");
      }

    
    PIC_SetBitInMemory(yyP49);

    
    PIC_OutputLabel(label);
    }
  }
  }
   return;
yyL3:;

 yyAbort ("PIC_Expr_Equal");
;
}

static void PIC_EvaluateUnary
# if defined __STDC__ | defined __cplusplus
(register tTree yyP52, register tTree yyP51, register tTree yyP50)
# else
(yyP52, yyP51, yyP50)
 register tTree yyP52;
 register tTree yyP51;
 register tTree yyP50;
# endif
{
/* line 4488 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
int a1; int a2;
  if (yyP50->Kind == kBitNot) {
/* line 4489 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4491 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP52 -> Expr . PIC_REG == yyP51 -> Expr . PIC_REG))) goto yyL1;
  {
/* line 4492 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */

    int size = PIC_Type_GetSize(yyP52->Expr.SynType) >> 3;
    int n;
    a1=yyP52->Expr.PIC_REG;
    for(n=0; n<size; n++)
      PIC_ComplementMemory(a1+n);
    
  }
  }
   return;
yyL1:;

/* line 4500 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4502 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP52 -> Expr . PIC_REG != yyP51 -> Expr . PIC_REG))) goto yyL2;
  {
/* line 4503 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */

    int size = PIC_Type_GetSize(yyP52->Expr.SynType) >> 3;
    int n;
    a1=yyP52->Expr.PIC_REG;
    a2=yyP51->Expr.PIC_REG;
    for(n=0; n<size; n++)
      {
      PIC_ComplementMemoryToW(a2+n);
      PIC_StoreWIntoMemory(a1+n);
      }
    
  }
  }
   return;
yyL2:;

  }
  if (yyP50->Kind == kNot) {
/* line 4515 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4517 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP52 -> Expr . PIC_REG == yyP51 -> Expr . PIC_REG))) goto yyL3;
  {
/* line 4518 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_LoadConstantIntoW (1 << (yyP52 -> Expr . PIC_REG % 8));
/* line 4519 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_XORWIntoMemory (yyP52 -> Expr . PIC_REG / 8);
  }
  }
   return;
yyL3:;

/* line 4521 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4523 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP52 -> Expr . PIC_REG != yyP51 -> Expr . PIC_REG))) goto yyL4;
  {
/* line 4524 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory (yyP52 -> Expr . PIC_REG);
/* line 4525 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitSet (yyP51 -> Expr . PIC_REG);
/* line 4526 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory (yyP52 -> Expr . PIC_REG);
  }
  }
   return;
yyL4:;

  }
  if (yyP50->Kind == kSubtract) {
/* line 4528 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4530 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP52 -> Expr . PIC_REG == yyP51 -> Expr . PIC_REG))) goto yyL5;
  {
/* line 4531 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP52->Expr.SynType) >> 3;
    int n;
    int label = PIC_GetNewLabel();

    
    for(n=0; n<size; n++)
      PIC_ComplementMemory(yyP52->Expr.PIC_REG+n);

    
    for(n=0; n<size; n++)
      {
      if (n==(size-1))
        PIC_IncrementMemory(yyP52->Expr.PIC_REG+n);
      else
        {
        PIC_IncrementMemoryAndSkipIfZero(yyP52->Expr.PIC_REG+n);
        PIC_GotoLabel(label,"no more additions are needed");
        }
      }

    
    PIC_OutputLabel(label);
    }
  }
  }
   return;
yyL5:;

/* line 4556 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4558 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP52 -> Expr . PIC_REG != yyP51 -> Expr . PIC_REG))) goto yyL6;
  {
/* line 4559 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    int size = PIC_Type_GetSize(yyP52->Expr.SynType) >> 3;
    int n;
    int label = PIC_GetNewLabel();

    
    for(n=0; n<size; n++)
      {
      PIC_ComplementMemoryToW(yyP51->Expr.PIC_REG+n);
      PIC_StoreWIntoMemory(yyP52->Expr.PIC_REG+n);
      }

    
    for(n=0; n<size; n++)
      {
      if (n==(size-1))
        PIC_IncrementMemory(yyP52->Expr.PIC_REG+n);
      else
        {
        PIC_IncrementMemoryAndSkipIfZero(yyP52->Expr.PIC_REG+n);
        PIC_GotoLabel(label,"no more additions are needed");
        }
      }

    
    PIC_OutputLabel(label);
    }
  }
  }
   return;
yyL6:;

  }
 yyAbort ("PIC_EvaluateUnary");
;
}

static void PIC_Constant
# if defined __STDC__ | defined __cplusplus
(register int yyP55, tValue yyP54, register tTree yyP53)
# else
(yyP55, yyP54, yyP53)
 register int yyP55;
 tValue yyP54;
 register tTree yyP53;
# endif
{

  switch (yyP53->Kind) {
  case kBool:
  case kUserBool:
/* line 4594 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4595 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP54 . Number == true))) goto yyL1;
  {
/* line 4596 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory (yyP55);
  }
  }
   return;
yyL1:;

/* line 4598 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4599 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP54 . Number == false))) goto yyL2;
  {
/* line 4600 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory (yyP55);
  }
  }
   return;
yyL2:;

  break;
  case kByte:
  case kUserByte:
/* line 4602 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4603 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory (yyP54 . Number, yyP55);
  }
   return;

  case kInt:
  case kUserInt:
/* line 4605 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4606 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 0) & 255, yyP55 + 0);
/* line 4607 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 8) & 255, yyP55 + 1);
  }
   return;

  case kInt16:
  case kUserInt16:
/* line 4609 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4610 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 0) & 255, yyP55 + 0);
/* line 4611 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 8) & 255, yyP55 + 1);
  }
   return;

  case kInt32:
  case kUserInt32:
/* line 4613 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4614 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 0) & 255, yyP55 + 0);
/* line 4615 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 8) & 255, yyP55 + 1);
/* line 4616 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 16) & 255, yyP55 + 2);
/* line 4617 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 24) & 255, yyP55 + 3);
  }
   return;

  case kInt64:
  case kUserInt64:
/* line 4619 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4620 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 0) & 255, yyP55 + 0);
/* line 4621 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 8) & 255, yyP55 + 1);
/* line 4622 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 16) & 255, yyP55 + 2);
/* line 4623 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 24) & 255, yyP55 + 3);
/* line 4624 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 32) & 255, yyP55 + 4);
/* line 4625 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 40) & 255, yyP55 + 5);
/* line 4626 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 48) & 255, yyP55 + 6);
/* line 4627 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 56) & 255, yyP55 + 7);
  }
   return;

  case kUntypedInt:
/* line 4629 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4632 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 0) & 255, yyP55 + 0);
/* line 4633 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory ((yyP54 . Number >> 8) & 255, yyP55 + 1);
  }
   return;

  }

 yyAbort ("PIC_Constant");
;
}

static bool PIC_Type_IsTimer
# if defined __STDC__ | defined __cplusplus
(register tTree yyP56)
# else
(yyP56)
 register tTree yyP56;
# endif
{
  if (yyP56->Kind == kTimer) {
/* line 4639 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return true;

  }
/* line 4642 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return false;

}

static int PIC_GetChannelNumber
# if defined __STDC__ | defined __cplusplus
(tSymbol yyP57)
# else
(yyP57)
 tSymbol yyP57;
# endif
{
/* line 4650 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4651 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
{
    
    if (SymbolKind(yyP57)->AnId.PIC_REG==-1)
      SymbolKind(yyP57)->AnId.PIC_REG=NextChannelId++;
    }
  }
   return SymbolKind (yyP57) -> AnId . PIC_REG;

}

static void PIC_SelectBank0
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4666 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4667 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory ((3 * 8) + 5);
  }
   return;

;
}

static void PIC_SelectBank1
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4674 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4675 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory ((3 * 8) + 5);
  }
   return;

;
}

static void PIC_ClearSchedulingBit
# if defined __STDC__ | defined __cplusplus
(register int yyP58)
# else
(yyP58)
 register int yyP58;
# endif
{
  if (equalint (yyP58, InvalidProcess)) {
/* line 4682 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
/* line 4684 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4685 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BCF %d,%d\t;clear the scheduling bit for proc%d\n", GetSchedulingByte (yyP58), GetSchedulingBit (yyP58), yyP58);
/* line 4686 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CacheBit (yyP58, false);
/* line 4687 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
if (last_instr_branch==instr_count) PIC_InvalidateEntireCache();
/* line 4688 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
  }
   return;

;
}

static void PIC_SetSchedulingBit
# if defined __STDC__ | defined __cplusplus
(register int yyP59)
# else
(yyP59)
 register int yyP59;
# endif
{
  if (equalint (yyP59, InvalidProcess)) {
/* line 4695 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
/* line 4697 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4698 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BSF %d,%d\t;set the scheduling bit for proc%d\n", GetSchedulingByte (yyP59), GetSchedulingBit (yyP59), yyP59);
/* line 4699 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CacheBit (yyP59, true);
/* line 4700 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
if (last_instr_branch==instr_count) PIC_InvalidateEntireCache();
/* line 4701 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
  }
   return;

;
}

static void PIC_SetSchedulingBit_Dynamic
# if defined __STDC__ | defined __cplusplus
(register int yyP61, register int yyP60)
# else
(yyP61, yyP60)
 register int yyP61;
 register int yyP60;
# endif
{
/* line 4706 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4707 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVF %d,0\t;load the scheduling byte into W\n", yyP61);
/* line 4708 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVWF 4\t;set the indirect address\n");
/* line 4709 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVF %d,0\t;load the scheduling bit into W\n", yyP60);
/* line 4710 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  IORWF 0,1\t;set the scheduling bit\n");
/* line 4711 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 4;
/* line 4712 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
/* line 4713 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateEntireCache ();
  }
   return;

;
}

static void PIC_StoreDynamicProcessId
# if defined __STDC__ | defined __cplusplus
(register int yyP64, register int yyP63, register int yyP62)
# else
(yyP64, yyP63, yyP62)
 register int yyP64;
 register int yyP63;
 register int yyP62;
# endif
{
/* line 4719 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4720 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory (GetSchedulingByte (yyP64), yyP63);
/* line 4721 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory (1 << GetSchedulingBit (yyP64), yyP62);
  }
   return;

;
}

static void PIC_ReturnToScheduler
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4727 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4728 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  RETURN\t;return to the scheduler\n");
/* line 4729 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
  }
   return;

;
}

static void PIC_ReturnValueInW
# if defined __STDC__ | defined __cplusplus
(register int yyP65)
# else
(yyP65)
 register int yyP65;
# endif
{
/* line 4735 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4736 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  RETLW %d\t;return with %d in W\n", yyP65, yyP65);
/* line 4737 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
  }
   return;

;
}

static void PIC_CallIfSchedulingBitSet
# if defined __STDC__ | defined __cplusplus
(register int yyP66)
# else
(yyP66)
 register int yyP66;
# endif
{
  if (equalint (yyP66, InvalidProcess)) {
/* line 4744 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
/* line 4746 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4747 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSC %d,%d\t;skip next instruction if proc%d is not scheduled\n", GetSchedulingByte (yyP66), GetSchedulingBit (yyP66), yyP66);
/* line 4748 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  CALL proc%d\t;execute proc%d\n", yyP66, yyP66);
/* line 4749 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 2;
/* line 4750 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateEntireCache ();
/* line 4751 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_CallChannelHandler
# if defined __STDC__ | defined __cplusplus
(register int yyP67)
# else
(yyP67)
 register int yyP67;
# endif
{
/* line 4759 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4760 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  CALL chan%d\t;call the handler for channel %d\n", yyP67, yyP67);
/* line 4761 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4762 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
/* line 4763 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateEntireCache ();
  }
   return;

;
}

static void PIC_CallMul16
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4771 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4772 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  CALL mul16\t;perform a 16-bit multiplication\n");
/* line 4773 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4774 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
/* line 4775 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateEntireCache ();
  }
   return;

;
}

static void PIC_StoreConstantIntoMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP69, register int yyP68)
# else
(yyP69, yyP68)
 register int yyP69;
 register int yyP68;
# endif
{
/* line 4786 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4787 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP68) == true))) goto yyL1;
  {
/* line 4788 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedByte (yyP68) == yyP69))) goto yyL1;
  {
/* line 4790 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
   return;
yyL1:;

/* line 4792 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4793 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP69 == 0))) goto yyL2;
  {
/* line 4794 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  CLRF %d\t;store 0\n", yyP68);
/* line 4795 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4796 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CacheByte (yyP68, 0);
  }
  }
   return;
yyL2:;

/* line 4798 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4799 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP69 == currentW))) goto yyL3;
  {
/* line 4800 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreWIntoMemory (yyP68);
  }
  }
   return;
yyL3:;

/* line 4802 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4803 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_LoadConstantIntoW (yyP69);
/* line 4804 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreWIntoMemory (yyP68);
  }
   return;

;
}

static void PIC_SetBitInMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP70)
# else
(yyP70)
 register int yyP70;
# endif
{
  if (equalint (yyP70, InvalidProcess)) {
/* line 4813 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
/* line 4815 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4816 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP70) == true))) goto yyL2;
  {
/* line 4817 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP70) == true))) goto yyL2;
  {
/* line 4818 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL2;
  {
/* line 4819 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
   return;
yyL2:;

/* line 4821 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4822 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BSF %d,%d\t;set bit %d in register %d\n", yyP70 / 8, yyP70 % 8, yyP70 % 8, yyP70 / 8);
/* line 4823 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CacheBit (yyP70, true);
/* line 4824 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
if (last_instr_branch==instr_count) PIC_InvalidateCachedBit(yyP70);
/* line 4825 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
  }
   return;

;
}

static void PIC_ClearBitInMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP71)
# else
(yyP71)
 register int yyP71;
# endif
{
  if (equalint (yyP71, InvalidProcess)) {
/* line 4834 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
/* line 4836 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4837 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP71) == true))) goto yyL2;
  {
/* line 4838 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP71) == false))) goto yyL2;
  {
/* line 4839 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL2;
  {
/* line 4840 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
   return;
yyL2:;

/* line 4842 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4843 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BCF %d,%d\t;clear bit %d in register %d\n", yyP71 / 8, yyP71 % 8, yyP71 % 8, yyP71 / 8);
/* line 4844 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CacheBit (yyP71, false);
/* line 4845 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
if (last_instr_branch==instr_count) PIC_InvalidateCachedBit(yyP71);
/* line 4846 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
  }
   return;

;
}

static void PIC_SkipIfBitSet
# if defined __STDC__ | defined __cplusplus
(register int yyP72)
# else
(yyP72)
 register int yyP72;
# endif
{
  if (equalint (yyP72, InvalidProcess)) {
/* line 4856 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
/* line 4858 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4859 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL2;
  {
/* line 4860 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP72) == true))) goto yyL2;
  {
/* line 4861 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP72) == false))) goto yyL2;
  {
/* line 4862 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
   return;
yyL2:;

/* line 4864 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4865 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSS %d,%d\t;skip next instruction if bit %d in register %d is set\n", yyP72 / 8, yyP72 % 8, yyP72 % 8, yyP72 / 8);
/* line 4866 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4867 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
   return;

;
}

static void PIC_SkipIfBitClear
# if defined __STDC__ | defined __cplusplus
(register int yyP73)
# else
(yyP73)
 register int yyP73;
# endif
{
  if (equalint (yyP73, InvalidProcess)) {
/* line 4877 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   return;

  }
/* line 4879 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4880 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL2;
  {
/* line 4881 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP73) == true))) goto yyL2;
  {
/* line 4882 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP73) == true))) goto yyL2;
  {
/* line 4883 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
   return;
yyL2:;

/* line 4885 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4886 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSC %d,%d\t;skip next instruction if bit %d in register %d is clear\n", yyP73 / 8, yyP73 % 8, yyP73 % 8, yyP73 / 8);
/* line 4887 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4888 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
   return;

;
}

static void PIC_SkipIfZeroFlagIsSet
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4894 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4895 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSS 3,2\t;skip next instruction if Zero flag is set\n");
/* line 4896 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4897 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
   return;

;
}

static void PIC_SkipIfZeroFlagIsClear
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4903 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4904 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSC 3,2\t;skip next instruction if Zero flag is clear\n");
/* line 4905 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4906 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
   return;

;
}

static void PIC_SkipIfCarryFlagIsSet
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4912 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4913 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSS 3,0\t;skip next instruction if Carry flag is set\n");
/* line 4914 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4915 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
   return;

;
}

static void PIC_SkipIfCarryFlagIsClear
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4921 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4922 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSC 3,0\t;skip next instruction if Carry flag is clear\n");
/* line 4923 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4924 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
   return;

;
}

static void PIC_GotoLabelIfBitSet
# if defined __STDC__ | defined __cplusplus
(register int yyP75, register int yyP74)
# else
(yyP75, yyP74)
 register int yyP75;
 register int yyP74;
# endif
{
/* line 4930 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4931 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP75) == true))) goto yyL1;
  {
/* line 4932 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP75) == true))) goto yyL1;
  {
/* line 4933 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  GOTO lab%d\n", yyP74);
/* line 4934 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4935 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
   return;
yyL1:;

/* line 4937 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4938 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP75) == true))) goto yyL2;
  {
/* line 4939 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP75) == false))) goto yyL2;
  {
/* line 4940 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
   return;
yyL2:;

/* line 4942 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4943 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSC %d,%d\t;skip next instruction if bit %d in register %d is clear\n", yyP75 / 8, yyP75 % 8, yyP75 % 8, yyP75 / 8);
/* line 4944 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  GOTO lab%d\n", yyP74);
/* line 4945 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 2;
  }
   return;

;
}

static void PIC_GotoLabelIfBitClear
# if defined __STDC__ | defined __cplusplus
(register int yyP77, register int yyP76)
# else
(yyP77, yyP76)
 register int yyP77;
 register int yyP76;
# endif
{
/* line 4951 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4952 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP77) == true))) goto yyL1;
  {
/* line 4953 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP77) == true))) goto yyL1;
  {
/* line 4954 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
   return;
yyL1:;

/* line 4956 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4957 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP77) == true))) goto yyL2;
  {
/* line 4958 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP77) == false))) goto yyL2;
  {
/* line 4959 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  GOTO lab%d\n", yyP76);
/* line 4960 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4961 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
   return;
yyL2:;

/* line 4963 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4964 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  BTFSS %d,%d\t;skip next instruction if bit %d in register %d is set\n", yyP77 / 8, yyP77 % 8, yyP77 % 8, yyP77 / 8);
/* line 4965 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  GOTO lab%d\n", yyP76);
/* line 4966 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 2;
  }
   return;

;
}

static void PIC_TestWForZero
# if defined __STDC__ | defined __cplusplus
()
# else
()
# endif
{
/* line 4973 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4974 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ADDLW 0\t;test W for Zero\n");
/* line 4975 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
  }
   return;

;
}

static void PIC_TestMemoryForZero
# if defined __STDC__ | defined __cplusplus
(register int yyP78)
# else
(yyP78)
 register int yyP78;
# endif
{
/* line 4982 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4983 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVF %d,1\t;test register %d for Zero\n", yyP78, yyP78);
/* line 4984 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
  }
   return;

;
}

static void PIC_IncrementMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP79)
# else
(yyP79)
 register int yyP79;
# endif
{
/* line 4992 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 4993 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 4994 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP79) == true))) goto yyL1;
  {
/* line 4995 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  INCF %d,1\t;increment register %d by 1\n", yyP79, yyP79);
/* line 4996 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 4997 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP79,(PIC_GetCachedByte(yyP79)+1) & 0xFF);
  }
  }
  }
   return;
yyL1:;

/* line 4999 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5000 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  INCF %d,1\t;increment register %d by 1\n", yyP79, yyP79);
/* line 5001 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5002 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP79);
  }
   return;

;
}

static void PIC_IncrementMemoryToW
# if defined __STDC__ | defined __cplusplus
(register int yyP80)
# else
(yyP80)
 register int yyP80;
# endif
{
/* line 5009 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5010 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5011 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP80) == true))) goto yyL1;
  {
/* line 5012 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  INCF %d,0\t;increment register %d by 1, and store in W\n", yyP80, yyP80);
/* line 5013 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5014 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW = (PIC_GetCachedByte(yyP80) + 1) & 0xFF;
  }
  }
  }
   return;
yyL1:;

/* line 5016 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5017 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  INCF %d,0\t;increment register %d by 1, and store in W\n", yyP80, yyP80);
/* line 5018 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5019 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_IncrementMemoryAndSkipIfZero
# if defined __STDC__ | defined __cplusplus
(register int yyP81)
# else
(yyP81)
 register int yyP81;
# endif
{
/* line 5028 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5029 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5030 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP81) == true))) goto yyL1;
  {
/* line 5031 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  INCFSZ %d,1\t;increment register %d by 1, and skip if Zero\n", yyP81, yyP81);
/* line 5032 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5033 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP81,(PIC_GetCachedByte(yyP81)+1) & 0xFF);
/* line 5034 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
  }
  }
   return;
yyL1:;

/* line 5036 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5037 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  INCFSZ %d,1\t;increment register %d by 1, and skip if Zero\n", yyP81, yyP81);
/* line 5038 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5039 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP81);
/* line 5040 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
   return;

;
}

static void PIC_DecrementMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP82)
# else
(yyP82)
 register int yyP82;
# endif
{
/* line 5048 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5049 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5050 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP82) == true))) goto yyL1;
  {
/* line 5051 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  DECF %d,1\t;decrement register %d by 1\n", yyP82, yyP82);
/* line 5052 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5053 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP82,(PIC_GetCachedByte(yyP82)-1) & 0xFF);
  }
  }
  }
   return;
yyL1:;

/* line 5055 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5056 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  DECF %d,1\t;decrement register %d by 1\n", yyP82, yyP82);
/* line 5057 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5058 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP82);
  }
   return;

;
}

static void PIC_DecrementMemoryAndSkipIfZero
# if defined __STDC__ | defined __cplusplus
(register int yyP83)
# else
(yyP83)
 register int yyP83;
# endif
{
/* line 5067 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5068 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5069 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP83) == true))) goto yyL1;
  {
/* line 5070 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  DECFSZ %d,1\t;decrement register %d by 1, and skip if Zero\n", yyP83, yyP83);
/* line 5071 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5072 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP83,(PIC_GetCachedByte(yyP83)-1) & 0xFF);
/* line 5073 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
  }
  }
   return;
yyL1:;

/* line 5075 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5076 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  DECFSZ %d,1\t;decrement register %d by 1, and skip if Zero\n", yyP83, yyP83);
/* line 5077 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5078 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP83);
/* line 5079 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   last_instr_branch = instr_count;
  }
   return;

;
}

static void PIC_LoadMemoryIntoW
# if defined __STDC__ | defined __cplusplus
(register int yyP84)
# else
(yyP84)
 register int yyP84;
# endif
{
/* line 5087 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5088 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5089 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5090 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP84) == true))) goto yyL1;
  {
/* line 5091 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedByte (yyP84) == currentW))) goto yyL1;
  {
/* line 5092 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
  }
   return;
yyL1:;

/* line 5094 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5095 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL2;
  {
/* line 5096 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP84) == true))) goto yyL2;
  {
/* line 5097 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVF %d,0\t;load register %d into W\n", yyP84, yyP84);
/* line 5098 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5099 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = PIC_GetCachedByte (yyP84);
  }
  }
  }
   return;
yyL2:;

/* line 5101 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5102 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVF %d,0\t;load register %d into W\n", yyP84, yyP84);
/* line 5103 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5104 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_LoadConstantIntoW
# if defined __STDC__ | defined __cplusplus
(register int yyP85)
# else
(yyP85)
 register int yyP85;
# endif
{
/* line 5112 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5113 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5114 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW == yyP85))) goto yyL1;
  {
/* line 5115 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
   return;
yyL1:;

/* line 5117 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5118 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL2;
  {
/* line 5119 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVLW %d\t;set W to %d\n", yyP85, yyP85);
/* line 5120 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5121 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = yyP85;
  }
  }
   return;
yyL2:;

/* line 5123 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5124 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVLW %d\t;set W to %d\n", yyP85, yyP85);
/* line 5125 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5126 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_StoreWIntoMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP86)
# else
(yyP86)
 register int yyP86;
# endif
{
/* line 5136 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5137 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5138 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP86) == true))) goto yyL1;
  {
/* line 5139 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedByte (yyP86) == currentW))) goto yyL1;
  {
/* line 5140 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5141 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
  }
   return;
yyL1:;

/* line 5143 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5144 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL2;
  {
/* line 5145 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL2;
  {
/* line 5146 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVWF %d\t;store W into register %d\n", yyP86, yyP86);
/* line 5147 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5148 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_CacheByte (yyP86, currentW);
  }
  }
  }
   return;
yyL2:;

/* line 5150 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5151 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  MOVWF %d\t;store W into register %d\n", yyP86, yyP86);
/* line 5152 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5153 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP86);
  }
   return;

;
}

static void PIC_CopyBit
# if defined __STDC__ | defined __cplusplus
(register int yyP88, register int yyP87)
# else
(yyP88, yyP87)
 register int yyP88;
 register int yyP87;
# endif
{
/* line 5163 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5164 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP88 == yyP87))) goto yyL1;
  }
   return;
yyL1:;

/* line 5167 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5168 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP88) == true))) goto yyL2;
  {
/* line 5169 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP87) == true))) goto yyL2;
  {
/* line 5170 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP88) == PIC_GetCachedBit (yyP87)))) goto yyL2;
  {
/* line 5171 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
   return;
yyL2:;

/* line 5173 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5174 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP87) == true))) goto yyL3;
  {
/* line 5175 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP87) == false))) goto yyL3;
  {
/* line 5176 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory (yyP88);
  }
  }
  }
   return;
yyL3:;

/* line 5178 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5179 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsBitCached (yyP87) == true))) goto yyL4;
  {
/* line 5180 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedBit (yyP87) == true))) goto yyL4;
  {
/* line 5181 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory (yyP88);
  }
  }
  }
   return;
yyL4:;

/* line 5183 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5184 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_ClearBitInMemory (yyP88);
/* line 5185 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SkipIfBitClear (yyP87);
/* line 5186 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_SetBitInMemory (yyP88);
  }
   return;

;
}

static void PIC_CopyByte
# if defined __STDC__ | defined __cplusplus
(register int yyP90, register int yyP89)
# else
(yyP90, yyP89)
 register int yyP90;
 register int yyP89;
# endif
{
/* line 5197 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5198 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((yyP90 == yyP89))) goto yyL1;
  }
   return;
yyL1:;

/* line 5201 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5202 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP90) == true))) goto yyL2;
  {
/* line 5203 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP89) == true))) goto yyL2;
  {
/* line 5204 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedByte (yyP90) == PIC_GetCachedByte (yyP89)))) goto yyL2;
  {
/* line 5205 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
   return;
yyL2:;

/* line 5207 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5208 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP89) == true))) goto yyL3;
  {
/* line 5209 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedByte (yyP89) == 0))) goto yyL3;
  {
/* line 5210 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreConstantIntoMemory (0, yyP90);
/* line 5211 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
   return;
yyL3:;

/* line 5213 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5214 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL4;
  {
/* line 5215 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP89) == true))) goto yyL4;
  {
/* line 5216 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_GetCachedByte (yyP89) == currentW))) goto yyL4;
  {
/* line 5217 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_StoreWIntoMemory (yyP90);
/* line 5218 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   cache_hits = cache_hits + 1;
  }
  }
  }
  }
   return;
yyL4:;

/* line 5220 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5221 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */

    PIC_LoadMemoryIntoW(yyP89);
    PIC_StoreWIntoMemory(yyP90);
    
  }
   return;

;
}

static void PIC_XORMemoryIntoW
# if defined __STDC__ | defined __cplusplus
(register int yyP91)
# else
(yyP91)
 register int yyP91;
# endif
{
/* line 5230 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5231 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5232 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP91) == true))) goto yyL1;
  {
/* line 5233 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5234 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  XORWF %d,0\t;XOR register %d into W\n", yyP91, yyP91);
/* line 5235 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5236 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW=currentW ^ PIC_GetCachedByte(yyP91);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5238 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5239 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  XORWF %d,0\t;XOR register %d into W\n", yyP91, yyP91);
/* line 5240 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5241 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_XORWIntoMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP92)
# else
(yyP92)
 register int yyP92;
# endif
{
/* line 5247 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5248 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5249 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP92) == true))) goto yyL1;
  {
/* line 5250 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5251 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  XORWF %d,1\t;XOR W into register %d\n", yyP92, yyP92);
/* line 5252 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5253 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP92,PIC_GetCachedByte(yyP92) ^ currentW);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5255 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5256 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  XORWF %d,1\t;XOR W into register %d\n", yyP92, yyP92);
/* line 5257 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5258 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP92);
  }
   return;

;
}

static void PIC_XORConstantIntoW
# if defined __STDC__ | defined __cplusplus
(register int yyP93)
# else
(yyP93)
 register int yyP93;
# endif
{
/* line 5264 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5265 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5266 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5267 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  XORLW %d\t;XOR %d with W\n", yyP93, yyP93);
/* line 5268 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5269 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW^=yyP93;
  }
  }
  }
   return;
yyL1:;

/* line 5271 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5272 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  XORLW %d\t;XOR %d with W\n", yyP93, yyP93);
/* line 5273 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5274 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_ORMemoryIntoW
# if defined __STDC__ | defined __cplusplus
(register int yyP94)
# else
(yyP94)
 register int yyP94;
# endif
{
/* line 5280 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5281 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5282 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP94) == true))) goto yyL1;
  {
/* line 5283 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5284 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  IORWF %d,0\t;OR register %d into W\n", yyP94, yyP94);
/* line 5285 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5286 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW=currentW | PIC_GetCachedByte(yyP94);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5288 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5289 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  IORWF %d,0\t;OR register %d into W\n", yyP94, yyP94);
/* line 5290 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5291 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_ORWIntoMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP95)
# else
(yyP95)
 register int yyP95;
# endif
{
/* line 5297 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5298 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5299 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP95) == true))) goto yyL1;
  {
/* line 5300 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5301 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  IORWF %d,1\t;OR W into register %d\n", yyP95, yyP95);
/* line 5302 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5303 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP95,PIC_GetCachedByte(yyP95) | currentW);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5305 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5306 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  IORWF %d,1\t;OR W into register %d\n", yyP95, yyP95);
/* line 5307 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5308 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP95);
  }
   return;

;
}

static void PIC_ANDMemoryIntoW
# if defined __STDC__ | defined __cplusplus
(register int yyP96)
# else
(yyP96)
 register int yyP96;
# endif
{
/* line 5314 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5315 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5316 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP96) == true))) goto yyL1;
  {
/* line 5317 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5318 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ANDWF %d,0\t;AND register %d into W\n", yyP96, yyP96);
/* line 5319 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5320 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW=currentW & PIC_GetCachedByte(yyP96);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5322 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5323 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ANDWF %d,0\t;AND register %d into W\n", yyP96, yyP96);
/* line 5324 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5325 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_ANDWIntoMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP97)
# else
(yyP97)
 register int yyP97;
# endif
{
/* line 5331 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5332 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5333 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP97) == true))) goto yyL1;
  {
/* line 5334 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5335 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ANDWF %d,1\t;AND W into register %d\n", yyP97, yyP97);
/* line 5336 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5337 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP97,PIC_GetCachedByte(yyP97) & currentW);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5339 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5340 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ANDWF %d,1\t;AND W into register %d\n", yyP97, yyP97);
/* line 5341 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5342 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP97);
  }
   return;

;
}

static void PIC_ANDConstantIntoW
# if defined __STDC__ | defined __cplusplus
(register int yyP98)
# else
(yyP98)
 register int yyP98;
# endif
{
/* line 5348 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5349 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5350 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5351 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ANDLW %d\t;AND %d with W\n", yyP98, yyP98);
/* line 5352 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5353 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW&=yyP98;
  }
  }
  }
   return;
yyL1:;

/* line 5355 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5356 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ANDLW %d\t;AND %d with W\n", yyP98, yyP98);
/* line 5357 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5358 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_AddConstantToW
# if defined __STDC__ | defined __cplusplus
(register int yyP99)
# else
(yyP99)
 register int yyP99;
# endif
{
/* line 5364 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5365 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5366 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5367 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ADDLW %d\t;add %d to W\n", yyP99, yyP99);
/* line 5368 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5369 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW=(currentW+1)&0xFF;
  }
  }
  }
   return;
yyL1:;

/* line 5371 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5372 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ADDLW %d\t;add %d to W\n", yyP99, yyP99);
/* line 5373 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5374 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_AddMemoryToW
# if defined __STDC__ | defined __cplusplus
(register int yyP100)
# else
(yyP100)
 register int yyP100;
# endif
{
/* line 5380 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5381 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5382 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP100) == true))) goto yyL1;
  {
/* line 5383 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ADDWF %d,0\t;add register %d to W\n", yyP100, yyP100);
/* line 5384 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5385 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW = (currentW+PIC_GetCachedByte(yyP100))&0xFF;
  }
  }
  }
   return;
yyL1:;

/* line 5387 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5388 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ADDWF %d,0\t;add register %d to W\n", yyP100, yyP100);
/* line 5389 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5390 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_AddWToMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP101)
# else
(yyP101)
 register int yyP101;
# endif
{
/* line 5396 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5397 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5398 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5399 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP101) == true))) goto yyL1;
  {
/* line 5400 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ADDWF %d,1\t;add W to register %d\n", yyP101, yyP101);
/* line 5401 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5402 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP101,(PIC_GetCachedByte(yyP101)+currentW)&0xFF);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5404 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5405 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  ADDWF %d,1\t;add W to register %d\n", yyP101, yyP101);
/* line 5406 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5407 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_ComplementMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP102)
# else
(yyP102)
 register int yyP102;
# endif
{
/* line 5415 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5416 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5417 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP102) == true))) goto yyL1;
  {
/* line 5418 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  COMF %d,1\t;complement register %d\n", yyP102, yyP102);
/* line 5419 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5420 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
PIC_CacheByte(yyP102,(~PIC_GetCachedByte(yyP102))&0xFF);
  }
  }
  }
   return;
yyL1:;

/* line 5422 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5423 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  COMF %d,1\t;complement register %d\n", yyP102, yyP102);
/* line 5424 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5425 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP102);
  }
   return;

;
}

static void PIC_ComplementMemoryToW
# if defined __STDC__ | defined __cplusplus
(register int yyP103)
# else
(yyP103)
 register int yyP103;
# endif
{
/* line 5433 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5434 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5435 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP103) == true))) goto yyL1;
  {
/* line 5436 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  COMF %d,0\t;complement register %d to W\n", yyP103, yyP103);
/* line 5437 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5438 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW=(~PIC_GetCachedByte(yyP103)&0xFF);
  }
  }
  }
   return;
yyL1:;

/* line 5440 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5441 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  COMF %d,0\t;complement register %d to W\n", yyP103, yyP103);
/* line 5442 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5443 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_RotateMemoryLeft
# if defined __STDC__ | defined __cplusplus
(register int yyP104)
# else
(yyP104)
 register int yyP104;
# endif
{
/* line 5452 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5453 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  RLF %d,1\t;rotate register %d left by 1\n", yyP104, yyP104);
/* line 5454 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5455 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP104);
  }
   return;

;
}

static void PIC_RotateMemoryRight
# if defined __STDC__ | defined __cplusplus
(register int yyP105)
# else
(yyP105)
 register int yyP105;
# endif
{
/* line 5464 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5465 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  RRF %d,1\t;rotate register %d right by 1\n", yyP105, yyP105);
/* line 5466 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5467 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   PIC_InvalidateCachedByte (yyP105);
  }
   return;

;
}

static void PIC_SubtractWFromConstant
# if defined __STDC__ | defined __cplusplus
(register int yyP106)
# else
(yyP106)
 register int yyP106;
# endif
{
/* line 5475 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5476 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5477 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5478 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  SUBLW %d\t;subtract W from %d\n", yyP106, yyP106);
/* line 5479 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5480 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW=((yyP106-currentW)&0xFF);
  }
  }
  }
   return;
yyL1:;

/* line 5482 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5483 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  SUBLW %d\t;subtract W from %d\n", yyP106, yyP106);
/* line 5484 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5485 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_SubtractWFromMemory
# if defined __STDC__ | defined __cplusplus
(register int yyP107)
# else
(yyP107)
 register int yyP107;
# endif
{
/* line 5493 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5494 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5495 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5496 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP107) == true))) goto yyL1;
  {
/* line 5497 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  SUBWF %d,0\t;subtract W from register %d, and store result in W\n", yyP107, yyP107);
/* line 5498 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5499 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW=((PIC_GetCachedByte(yyP107)-currentW)&0xFF);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5501 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5502 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  SUBWF %d,0\t;subtract W from register %d, and store result in W\n", yyP107, yyP107);
/* line 5503 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5504 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

static void PIC_SwapMemoryToW
# if defined __STDC__ | defined __cplusplus
(register int yyP108)
# else
(yyP108)
 register int yyP108;
# endif
{
/* line 5512 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5513 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((last_instr_branch != instr_count))) goto yyL1;
  {
/* line 5514 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((currentW != InvalidW))) goto yyL1;
  {
/* line 5515 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   if (! ((PIC_IsByteFullyCached (yyP108) == true))) goto yyL1;
  {
/* line 5516 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  SWAPF %d,0\t;swap the high and low nibbles of register %d, and store result in W\n", yyP108, yyP108);
/* line 5517 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5518 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
currentW=((PIC_GetCachedByte(yyP108)<<4)&0xF0) || ((PIC_GetCachedByte(yyP108)>>4)&0x0F);
  }
  }
  }
  }
   return;
yyL1:;

/* line 5520 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
  {
/* line 5521 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   fprintf (fh, "  SWAPF %d,0\t;swap the high and low nibbles of register %d, and store result in W\n", yyP108, yyP108);
/* line 5522 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   instr_count = instr_count + 1;
/* line 5523 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */
   currentW = InvalidW;
  }
   return;

;
}

void BeginPICout ()
{
/* line 756 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */

  unsigned int n;

  NextChannelId=0;
  channel_list=NULL;

  currentW=InvalidW;
  instr_count=0;
  last_instr_branch=0;
  start_process=InvalidProcess;
  current_process=InvalidProcess;
  current_label=0;
  process_list=NULL;
  cache_hits=0;
  IsTimerUsedFlag = false;
  IsMul16Needed = false;

  UsedMemory=(unsigned char*)malloc(MAX_PIC_DATA);
  if (UsedMemory==NULL) Message("pic.trafo: malloc() failed in BEGIN",xxFatal,NoPosition);
  for(n=0; n<MAX_PIC_DATA; n++)
    UsedMemory[n]=0;

  MemoryLevels=(unsigned int*)malloc(MAX_PIC_DATA*8*sizeof(unsigned int));
  if (MemoryLevels==NULL) Message("pic.trafo: malloc() failed in BEGIN",xxFatal,NoPosition);
  for(n=0; n<(MAX_PIC_DATA*8); n++)
    MemoryLevels[n]=0;

  MemoryValid=(unsigned char*)malloc(MAX_PIC_DATA);
  if (MemoryValid==NULL) Message("pic.trafo: malloc() failed in BEGIN",xxFatal,NoPosition);
  for(n=0; n<MAX_PIC_DATA; n++)
    MemoryValid[n]=0;

  MemoryContents=(unsigned char*)malloc(MAX_PIC_DATA);
  if (MemoryContents==NULL) Message("pic.trafo: malloc() failed in BEGIN",xxFatal,NoPosition);
  /* There is no need to initialise MemoryContents, since */
  /* it is not valid until MemoryValid is set. */

  Level=1;
  SharedMemory=0;
  
}

void ClosePICout ()
{
/* line 800 "/home/simon/spoc/source/spoc.1.3//source/gmd/pic.trafo" */

  /* free the memory used by the process_list */
  process_element* pe = process_list;
  while(pe!=NULL)
    {
    process_list=pe->next;
    free(pe);
    pe=process_list;
    }

  free(MemoryContents);
  free(MemoryValid);
  free(MemoryLevels);
  free(UsedMemory);
  
}
